/* /////////////////////////////////////////////////////////////////////////////
 * File:        winstl_int_to_string.h
 *
 * Purpose:     WinSTL .
 *
 * Created:     31st July 2002
 * Updated:     28th July 2003
 *
 * Author:      Matthew Wilson, Synesis Software Pty Ltd.
 *
 * License:     (Licensed under the Synesis Software Standard Source License)
 *
 *              Copyright (C) 2002-2003, Synesis Software Pty Ltd.
 *
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/winstl
 *                          http://www.winstl.org/
 *
 *              email:      submissions@winstl.org  for submissions
 *                          admin@winstl.org        for other enquiries
 *
 *              Redistribution and use in source and binary forms, with or
 *              without modification, are permitted provided that the following
 *              conditions are met:
 *
 *              (i) Redistributions of source code must retain the above
 *              copyright notice and contact information, this list of
 *              conditions and the following disclaimer.
 *
 *              (ii) Any derived versions of this software (howsoever modified)
 *              remain the sole property of Synesis Software.
 *
 *              (iii) Any derived versions of this software (howsoever modified)
 *              remain subject to all these conditions.
 *
 *              (iv) Neither the name of Synesis Software nor the names of any
 *              subdivisions, employees or agents of Synesis Software, nor the
 *              names of any other contributors to this software may be used to
 *              endorse or promote products derived from this software without
 *              specific prior written permission.
 *
 *              This source code is provided by Synesis Software "as is" and any
 *              warranties, whether expressed or implied, including, but not
 *              limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software be liable for any direct, indirect,
 *              incidental, special, exemplary, or consequential damages
 *              (including, but not limited to, procurement of substitute goods
 *              or services; loss of use, data, or profits; or business
 *              interruption) however caused and on any theory of liability,
 *              whether in contract, strict liability, or tort (including
 *              negligence or otherwise) arising in any way out of the use of
 *              this software, even if advised of the possibility of such
 *              damage.
 *
 * ////////////////////////////////////////////////////////////////////////// */


#ifndef _WINSTL_INCL_H_WINSTL_INT_TO_STRING
#define _WINSTL_INCL_H_WINSTL_INT_TO_STRING

#ifndef __STLSOFT_DOCUMENTATION_SKIP_SECTION
#define _WINSTL_VER_H_WINSTL_INT_TO_STRING_MAJOR    1
#define _WINSTL_VER_H_WINSTL_INT_TO_STRING_MINOR    0
#define _WINSTL_VER_H_WINSTL_INT_TO_STRING_REVISION 4
#define _WINSTL_VER_H_WINSTL_INT_TO_STRING_EDIT     6
#endif /* !__STLSOFT_DOCUMENTATION_SKIP_SECTION */

/* /////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#ifndef _WINSTL_INCL_H_WINSTL
 #include "winstl.h"                    // Include the WinSTL root header
#endif /* !_WINSTL_INCL_H_WINSTL */
#ifndef _STLSOFT_INCL_H_STLSOFT_INTEGER_TO_STRING
 #include "stlsoft_integer_to_string.h" // stlsoft::integer_to_string
#endif /* !_STLSOFT_INCL_H_STLSOFT_INTEGER_TO_STRING */
#ifndef _WINSTL_INCL_H_THREAD_MUTEX
 #include "winstl_thread_mutex.h"       // thread_mutex
#endif /* !_WINSTL_INCL_H_THREAD_MUTEX */
#ifndef _STLSOFT_INCL_H_STLSOFT_LOCK_SCOPE
 #include "stlsoft_lock_scope.h"        // stlsoft::lock_scope
#endif /* !_STLSOFT_INCL_H_STLSOFT_LOCK_SCOPE */

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#ifndef _WINSTL_NO_NAMESPACE
 #ifdef _STLSOFT_NO_NAMESPACE
/* There is no stlsoft namespace, so must define ::winstl */
namespace winstl
{
 #else
/* Define stlsoft::winstl_project */

namespace stlsoft
{

namespace winstl_project
{

 #endif /* _STLSOFT_NO_NAMESPACE */
#endif /* !_WINSTL_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

/// \weakgroup libraries STLSoft Libraries
/// \brief The individual libraries

/// \weakgroup libraries_conversion Conversion Library
/// \ingroup libraries
/// \brief This library provides conversion facilities 

/// \weakgroup winstl_libraries_conversion Conversion Library (WinSTL)
/// \ingroup WinSTL libraries_performance
/// \brief This library provides conversion facilities for the Win32 API
/// @{

/* /////////////////////////////////////////////////////////////////////////////
 * Pre-processor options
 */

#if defined(_WINSTL_INT_TO_STRING_USE_DECLSPECTHREAD_FOR_EXES)
 #if defined(_DLL) || \
     defined(__DLL__) || \
     defined(_WINDLL) || \
     defined(_USRDLL) || \
     defined(_AFXDLL)
  #pragma message("Using _WINSTL_INT_TO_STRING_USE_DECLSPECTHREAD_FOR_EXES when building DLLs will result in their not being loadable dynamically (via LoadLibrary())")
 #endif /* dll */
#endif /* _WINSTL_INT_TO_STRING_USE_DECLSPECTHREAD_FOR_EXES */

/* /////////////////////////////////////////////////////////////////////////////
 * Functions
 */

#ifdef __STLSOFT_CF_NAMESPACE_SUPPORT
namespace int_to_string_tls
{
#endif /* __STLSOFT_CF_NAMESPACE_SUPPORT */

template< ws_typename_param_k C
        , ws_size_t           CCH
        >
struct Slot
{
    Slot(Slot *next)
        : next(next)
    {}
    ~Slot()
    {
        delete next;
    }

    // Use the process heap because:
    // 
    // 1. Don't want to worry about thread-specificity, since 
    //   deallocation will occur in a different thread to allocation
    // 2. Don't want to worry about linkage to any specific CRT or
    //      other library
    // 3. Doesn't matter how fast it is
    // 4. Want it to be *highly* unlikely that allocation will fail, 
    //    which is indeed pretty unheard of when using the Win32 
    //    process heap.
    // 5. Want a C++-exception free solution, so use the Win32-system
    //    out-of-memory exception, and not have to worry about any 
    //    linkage pains.
    void *operator new(size_t cb)
    {
        return ::HeapAlloc(::GetProcessHeap(), HEAP_GENERATE_EXCEPTIONS, cb);
    }
    void operator delete(void *pv)
    {
        ::HeapFree(::GetProcessHeap(), 0, pv);
    }

    C       buff[CCH];
    Slot    *next;
};

template< ws_typename_param_k C
        , ws_size_t           CCH
        >
struct Index
{
    typedef Slot<C, CCH>    Slot;

    // This is admittedly totally gross, but it works and will be portable
    // across different compilers. The reason it works is that s_index is
    // static, and therefore all its members will be 0. This facilitates
	// using interlocking and spin-locks in order to manage the lifetime
	// correctly, and control access to the instance's constructor.
	//
	// Note that no members are initialised in a constructor member
	// initialisation list (MIL). They are initialised within the
	// initialise

    Index()
    {
		// Since multiple threads could get into here before they
		// are all eventually blocked out by the static controller
		// variable, we must guard against such multiple
		// construction. Hence, if this is the only thread currently
		// engaged in construction, we can proceed to initialisation.
		// otherwise we must spin.
        if(1 == ::InterlockedIncrement((LPLONG)&m_ctor))
        {
            // Initialisation.

            struct thread_mx_
                : public thread_mutex
            {
            public:
                thread_mx_()
                {}

                void *operator new(size_t , void *p)
                {
                    return p;
                }
            };

			// The construction thread could be suspended after a
			// successful completion of the constructor, but before
			// the hidden boolean managing static creation was 
			// updated. Therefore, we need to reference-count the
			// construction, which is done via another interlocked
			// count, this time on m_init.

			if(1 == ::InterlockedIncrement((LPLONG)&m_init))
			{
				// The constructor has not yet been called through 
				// to this point, t

				new (&mx()) thread_mx_();

				m_index =   ::TlsAlloc();

				// Now release the construction count. From this point
				// onwards there are no guarantees as to concurrency, so
				// no more initialisation can take place.
				InterlockedDecrement((LPLONG)&m_ctor);

				// Use Win32 exception because:
				//
				// 1. Process cannot recover from this error in any 
				// meaningful way
				// 2. Do not want to couple to C++ exception-handling
				// and there is no graceful way to allow this to be
				// parameterisable. (May allow a pp-discriminated
				// mechanism in next version.)
				if(TLS_OUT_OF_INDEXES == m_index)
				{
					::RaiseException(STATUS_NO_MEMORY, EXCEPTION_NONCONTINUABLE, 0, 0);
				}
			}
        }
        else
        {
            // Came in on the initialisation of this
            // instance on another thread. Simply
            // spin until the initialisation is complete
            // and then leave.

			// Another thread may own the construction task, but may
			// not yet have completed it. Therefore, we cannot leave
			// the constructor until the constructing thread has 
			// completed, so spin here until m_ctor is 0.

            for(;;)
            {
                if(0 == InterlockedDecrement((LPLONG)&m_ctor))
                {
                    ::InterlockedIncrement((LPLONG)&m_init);
                    break;
                }
                else
                {
					// This looks horribly inefficient, but in
					// actuality this will happen incredibly rarely,
					// and even when it does, the spinning/sleeping
					// thread is only waiting around for something that
					// is (a) quick, and (b) only going to happen once.

                    ::Sleep(1);
                    InterlockedIncrement((LPLONG)&m_ctor);
                }
            }
        }
    }

    ~Index()
    {
        if(0 == ::InterlockedDecrement((LPLONG)&m_init))
        {
            // Walk the slot list and free. This can be as slow as 
            // you like, since performance is not important here
            delete m_top;

            // Now release the index
            ::TlsFree(m_index);

			// Need to explicitly destroy the mutex.
            mx().~thread_mutex();
        }
    }

    Slot *GetSlot()
    {
        // NOTE: This does not need to be thread-safe
        return reinterpret_cast<Slot*>(::TlsGetValue(m_index));
    }

    Slot *AllocSlot()
    {
        Slot    *next;

        { // Protect linked-list manipulation
            lock_scope<thread_mutex, thread_mutex_lock_traits>  lock(mx());

            m_top = next = new Slot(m_top);
        }

        ::TlsSetValue(m_index, next);

        return next;
    }

// Implementation
private:
    thread_mutex &mx()
    {
        return *static_cast<thread_mutex*>(static_cast<void*>(&m__mx._mx[0]));
    }

private:
#if 0
// In an ideal world the member layout would be as follows:
    ws_dword_t      m_index;
    Slot            *m_top;
    thread_mutex    m_mx;
#else
// But we're not in an ideal world, so it is like this
    ws_dword_t      m_index;
    Slot            *m_top;
    struct
    {
        ss_byte_t   _mx[sizeof(thread_mutex)];
    }               m__mx;
    ws_sint32_t     m_init; // Construction count
    ws_sint32_t     m_ctor; // Ctor entry count
#endif /* 0 */
};

#ifdef __STLSOFT_CF_NAMESPACE_SUPPORT
} // namespace int_to_string_tls
#endif /* __STLSOFT_CF_NAMESPACE_SUPPORT */


template< ws_typename_param_k C
        , ws_size_t           CCH
        >
inline C *i2str_get_tss_buffer()
{
#if defined(_WINSTL_INT_TO_STRING_USE_DECLSPECTHREAD_FOR_EXES)
    __declspec(thread) static C s_buffer[CCH];

    return s_buffer;
#else

#ifdef __STLSOFT_CF_NAMESPACE_SUPPORT
    typedef int_to_string_tls::Index<C, CCH>    Index;
    typedef int_to_string_tls::Slot<C, CCH>     Slot;
#else
    typedef Index<C, CCH>                       Index;
    typedef Slot<C, CCH>                        Slot;
#endif /* __STLSOFT_CF_NAMESPACE_SUPPORT */

    static Index    s_index;
    Slot            *slot   =   s_index.GetSlot();

    if(NULL == slot)
    {
        slot = s_index.AllocSlot();
    }

    return slot->buff;
#endif /* dll */
}


template<ws_typename_param_k C>
inline C const *int_to_string(ws_sint8_t value)
{
    const ws_size_t CCH     = 21; // 5 fits 8-bit + sign
    C               *buffer = i2str_get_tss_buffer<C, CCH>();

    return stlsoft::integer_to_string(buffer, CCH, value);
}

template<ws_typename_param_k C>
inline C const *int_to_string(ws_uint8_t value)
{
    const ws_size_t CCH     = 21; // 4 fits 8-bit
    C               *buffer = i2str_get_tss_buffer<C, CCH>();

    return stlsoft::integer_to_string(buffer, CCH, value);
}

template<ws_typename_param_k C>
inline C const *int_to_string(ws_sint16_t value)
{
    const ws_size_t CCH     = 21; // 7 fits 16-bit + sign
    C               *buffer = i2str_get_tss_buffer<C, CCH>();

    return stlsoft::integer_to_string(buffer, CCH, value);
}

template<ws_typename_param_k C>
inline C const *int_to_string(ws_uint16_t value)
{
    const ws_size_t CCH     = 21; // 6 fits 16-bit
    C               *buffer = i2str_get_tss_buffer<C, CCH>();

    return stlsoft::integer_to_string(buffer, CCH, value);
}

template<ws_typename_param_k C>
inline C const *int_to_string(ws_sint32_t value)
{
    const ws_size_t CCH     = 21; // 12 fits 32-bit + sign
    C               *buffer = i2str_get_tss_buffer<C, CCH>();

    return stlsoft::integer_to_string(buffer, CCH, value);
}

template<ws_typename_param_k C>
inline C const *int_to_string(ws_uint32_t value)
{
    const ws_size_t CCH     = 21; // 11 fits 32-bit
    C               *buffer = i2str_get_tss_buffer<C, CCH>();

    return stlsoft::integer_to_string(buffer, CCH, value);
}

template<ws_typename_param_k C>
inline C const *int_to_string(ws_sint64_t const &value)
{
    const ws_size_t CCH     = 21; // fits 64-bit + sign
    C               *buffer = i2str_get_tss_buffer<C, CCH>();

    return stlsoft::integer_to_string(buffer, CCH, value);
}

template<ws_typename_param_k C>
inline C const *int_to_string(ws_uint64_t const &value)
{
    const ws_size_t CCH     = 21; // fits 64-bit
    C               *buffer = i2str_get_tss_buffer<C, CCH>();

    return stlsoft::integer_to_string(buffer, CCH, value);
}

/* ////////////////////////////////////////////////////////////////////////// */

/// @} // end of group winstl_libraries_conversion

/* ////////////////////////////////////////////////////////////////////////// */

#ifndef _WINSTL_NO_NAMESPACE
 #ifdef _STLSOFT_NO_NAMESPACE
} // namespace winstl
 #else
} // namespace winstl_project
} // namespace stlsoft
 #endif /* _STLSOFT_NO_NAMESPACE */
#endif /* !_WINSTL_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* !_WINSTL_INCL_H_WINSTL_INT_TO_STRING */

/* ////////////////////////////////////////////////////////////////////////// */
