/* ////////////////////////////////////////////////////////////////////////////
 * File:        winstl_process_mutex.h
 *
 * Purpose:     Inter-process mutext, based on Windows MUTEX.
 *
 * Date:        15th May 2002
 * Updated:     21st July 2003
 *
 * Author:      Matthew Wilson, Synesis Software Pty Ltd.
 *
 * License:     (Licensed under the Synesis Software Standard Source License)
 *
 *              Copyright (C) 2002-2003, Synesis Software Pty Ltd.
 *
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/winstl
 *                          http://www.winstl.org/
 *
 *              email:      submissions@winstl.org  for submissions
 *                          admin@winstl.org        for other enquiries
 *
 *              Redistribution and use in source and binary forms, with or
 *              without modification, are permitted provided that the following
 *              conditions are met:
 *
 *              (i) Redistributions of source code must retain the above
 *              copyright notice and contact information, this list of
 *              conditions and the following disclaimer.
 *
 *              (ii) Any derived versions of this software (howsoever modified)
 *              remain the sole property of Synesis Software.
 *
 *              (iii) Any derived versions of this software (howsoever modified)
 *              remain subject to all these conditions.
 *
 *              (iv) Neither the name of Synesis Software nor the names of any
 *              subdivisions, employees or agents of Synesis Software, nor the
 *              names of any other contributors to this software may be used to
 *              endorse or promote products derived from this software without
 *              specific prior written permission.
 *
 *              This source code is provided by Synesis Software "as is" and any
 *              warranties, whether expressed or implied, including, but not
 *              limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software be liable for any direct, indirect,
 *              incidental, special, exemplary, or consequential damages
 *              (including, but not limited to, procurement of substitute goods
 *              or services; loss of use, data, or profits; or business
 *              interruption) however caused and on any theory of liability,
 *              whether in contract, strict liability, or tort (including
 *              negligence or otherwise) arising in any way out of the use of
 *              this software, even if advised of the possibility of such
 *              damage.
 *
 * ////////////////////////////////////////////////////////////////////////// */


#ifndef _WINSTL_INCL_H_WINSTL_PROCESS_MUTEX
#define _WINSTL_INCL_H_WINSTL_PROCESS_MUTEX

#ifndef __STLSOFT_DOCUMENTATION_SKIP_SECTION
#define _WINSTL_VER_H_WINSTL_PROCESS_MUTEX_MAJOR    1
#define _WINSTL_VER_H_WINSTL_PROCESS_MUTEX_MINOR    2
#define _WINSTL_VER_H_WINSTL_PROCESS_MUTEX_REVISION 2
#define _WINSTL_VER_H_WINSTL_PROCESS_MUTEX_EDIT     7
#endif /* !__STLSOFT_DOCUMENTATION_SKIP_SECTION */

/* /////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#ifndef _WINSTL_INCL_H_WINSTL
 #include "winstl.h"    // Include the WinSTL root header
#endif /* !_WINSTL_INCL_H_WINSTL */

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#ifndef _WINSTL_NO_NAMESPACE
 #ifdef _STLSOFT_NO_NAMESPACE
/* There is no stlsoft namespace, so must define ::winstl */
namespace winstl
{
 #else
/* Define stlsoft::winstl_project */

namespace stlsoft
{

namespace winstl_project
{

 #endif /* _STLSOFT_NO_NAMESPACE */
#endif /* !_WINSTL_NO_NAMESPACE */

/* /////////////////////////////////////////////////////////////////////////////
 * Classes
 */

#ifdef __SYNSOFT_DBS_COMPILER_SUPPORTS_PRAGMA_MESSAGE
#pragma message(_sscomp_fileline_message("This needs to be parameterised with a winstl::system_resource_policy, which would control whether to throw if MX create fails"))
#endif /* __SYNSOFT_DBS_COMPILER_SUPPORTS_PRAGMA_MESSAGE */


// class process_mutex
/// This class provides an implementation of the mutex model based on the Win32 CRITICAL_SECTION
class process_mutex
{
public:
    typedef process_mutex   class_type;

/// Construction
/// @{
public:
    /// Creates an instance of the mutex
    process_mutex() winstl_throw_0()
        : m_mx(_create_mutex(NULL, false, static_cast<ws_char_a_t const *>(0), m_bCreated))
        , m_bAbandoned(false)
    {}
    /// Creates an instance of the mutex
    ws_explicit_k process_mutex(ws_char_a_t const *name) winstl_throw_0()
        : m_mx(_create_mutex(NULL, false, name, m_bCreated))
        , m_bAbandoned(false)
    {}
    /// Creates an instance of the mutex
    ws_explicit_k process_mutex(ws_char_w_t const *name) winstl_throw_0()
        : m_mx(_create_mutex(NULL, false, name, m_bCreated))
        , m_bAbandoned(false)
    {}
    /// Creates an instance of the mutex
    ws_explicit_k process_mutex(bool bInitialOwer) winstl_throw_0()
        : m_mx(_create_mutex(NULL, bInitialOwer, static_cast<ws_char_a_t const *>(0), m_bCreated))
        , m_bAbandoned(false)
    {}
    /// Creates an instance of the mutex
    ws_explicit_k process_mutex(ws_char_a_t const *name, bool bInitialOwer) winstl_throw_0()
        : m_mx(_create_mutex(NULL, bInitialOwer, name, m_bCreated))
        , m_bAbandoned(false)
    {}
    /// Creates an instance of the mutex
    ws_explicit_k process_mutex(ws_char_w_t const *name, bool bInitialOwer) winstl_throw_0()
        : m_mx(_create_mutex(NULL, bInitialOwer, name, m_bCreated))
        , m_bAbandoned(false)
    {}
    /// Creates an instance of the mutex
    ws_explicit_k process_mutex(ws_char_a_t const *name, bool bInitialOwer, LPSECURITY_ATTRIBUTES psa) winstl_throw_0()
        : m_mx(_create_mutex(psa, bInitialOwer, name, m_bCreated))
        , m_bAbandoned(false)
    {}
    /// Creates an instance of the mutex
    ws_explicit_k process_mutex(ws_char_w_t const *name, bool bInitialOwer, LPSECURITY_ATTRIBUTES psa) winstl_throw_0()
        : m_mx(_create_mutex(psa, bInitialOwer, name, m_bCreated))
        , m_bAbandoned(false)
    {}

    /// Destroys an instance of the mutex
    ~process_mutex() winstl_throw_0()
    {
        if(m_mx != NULL)
        {
            ::CloseHandle(m_mx);
        }
    }

#if 0
    void close() winstl_throw_0()
    {
        if(m_mx != NULL)
        {
            ::CloseHandle(m_mx);
            m_mx = NULL;
        }
    }
#endif /* 0 */

/// @}

/// Operations
/// @{
public:
    /// Acquires a lock on the mutex, pending the thread until the lock is aquired
    void lock() winstl_throw_0()
    {
        DWORD   dwRes   =   ::WaitForSingleObject(m_mx, INFINITE);

        m_bAbandoned = (dwRes == WAIT_ABANDONED);
    }
    /// Acquires a lock on the mutex, pending the thread until the lock is aquired
    bool lock(ws_dword_t wait) winstl_throw_0()
    {
        DWORD   dwRes   =   ::WaitForSingleObject(m_mx, wait);

        m_bAbandoned = (dwRes == WAIT_ABANDONED);

        return (dwRes == WAIT_OBJECT_0);
    }
    /// Attempts to lock the mutex
    ///
    /// \return <b>true</b> if the mutex was aquired, or <b>false</b> if not
    bool try_lock()
    {
        return lock(0);
    }
    /// Releases an aquired lock on the mutex
    void unlock() winstl_throw_0()
    {
        ::ReleaseMutex(m_mx);
    }

/// @}

/// Attributes
/// @{
public:
    /// Indicates whether this object instance created the underlying mutex object
    ///
    /// \return true The mutex object was created by this instance
    /// \return false The mutex object was not created by this instance
    /// \note For unnamed mutexes this will always be false
    ws_bool_t created() const
    {
        return m_bCreated;
    }

    /// Indicates whether a successful call to lock occurred because the underlying
    /// mutex was previously held by a thread that abandoned.
    ///
    /// \return true The mutex object was abandoned by its previous owning thread
    /// \return false The mutex object was not abandoned by its previous owning thread
    /// \note This attribute is meaningful with respect to the result of the last call to lock() or try_lock(). Subsequent calls to unlock() do not affect this attribute.
    ws_bool_t abandoned() const
    {
        return m_bAbandoned;
    }

/// @}

// Implementation
private:
    static HANDLE _create_mutex(LPSECURITY_ATTRIBUTES psa, bool bInitialOwner, ws_char_a_t const *name, bool &bCreated)
    {
        HANDLE  mx  =   ::CreateMutexA(psa, bInitialOwner, name);

        bCreated = (mx != NULL && ::GetLastError() != ERROR_ALREADY_EXISTS);

        return mx;
    }
    static HANDLE _create_mutex(LPSECURITY_ATTRIBUTES psa, bool bInitialOwner, ws_char_w_t const *name, bool &bCreated)
    {
        HANDLE  mx  =   ::CreateMutexW(psa, bInitialOwner, name);

        bCreated = (mx != NULL && ::GetLastError() != ERROR_ALREADY_EXISTS);

        return mx;
    }
    static HANDLE _open_mutex(ws_dword_t access, bool bInheritHandle, ws_char_a_t const *name, bool &bCreated)
    {
        HANDLE  mx  =   ::OpenMutexA(access, bInheritHandle, name);

        bCreated = (mx != NULL && ::GetLastError() != ERROR_ALREADY_EXISTS);

        return mx;
    }
    static HANDLE _open_mutex(ws_dword_t access, bool bInheritHandle, ws_char_w_t const *name, bool &bCreated)
    {
        HANDLE  mx  =   ::OpenMutexW(access, bInheritHandle, name);

        bCreated = (mx != NULL && ::GetLastError() != ERROR_ALREADY_EXISTS);

        return mx;
    }

// Members
private:
    HANDLE      m_mx;           // The underlying mutex object
    ws_bool_t   m_bCreated;     // Did this object (thread) create the underlying mutex object
    ws_bool_t   m_bAbandoned;   // Did the previous owner abandon the underlying mutex object

// Not to be implemented
private:
    process_mutex(class_type const &rhs);
    process_mutex &operator =(class_type const &rhs);
};

/* /////////////////////////////////////////////////////////////////////////////
 * Control shims
 */

/// \weakgroup concepts STLSoft Concepts

/// \weakgroup concepts_shims Shims
/// \ingroup concepts 

/// \weakgroup concepts_shims_sync_control Synchronisation Control Shims
/// \ingroup concepts_shims
/// \brief These \ref concepts_shims "shims" control the behaviour of synchronisation objects

/// \defgroup winstl_sync_control_shims Synchronisation Control Shims (WinSTL)
/// \ingroup WinSTL concepts_shims_sync_control
/// \brief These \ref concepts_shims "shims" control the behaviour of Win32 synchronisation objects
/// @{

/// This control ref concepts_shims "shim" aquires a lock on the given mutex
///
/// \param mx The mutex on which to aquire the lock
inline void lock_instance(process_mutex &mx)
{
    mx.lock();
}

/// This control ref concepts_shims "shim" releases a lock on the given mutex
///
/// \param mx The mutex on which to release the lock
inline void unlock_instance(process_mutex &mx)
{
    mx.unlock();
}

/// @} // end of group winstl_sync_control_shims

/* /////////////////////////////////////////////////////////////////////////////
 * lock_traits (for the compilers that do not support Koenig Lookup)
 */

// class lock_traits
/// Traits for the process_mutex class (for compilers that do not support Koenig Lookup)
struct process_mutex_lock_traits
{
public:
    /// The lockable type
    typedef process_mutex                lock_type;
    typedef process_mutex_lock_traits    class_type;

// Operations
public:
    /// Lock the given process_mutex instance
    static void lock(process_mutex &c)
    {
        lock_instance(c);
    }

    /// Unlock the given process_mutex instance
    static void unlock(process_mutex &c)
    {
        unlock_instance(c);
    }
};

/* ////////////////////////////////////////////////////////////////////////// */

#ifndef _WINSTL_NO_NAMESPACE
 #ifdef _STLSOFT_NO_NAMESPACE
} // namespace winstl
 #else
} // namespace winstl_project
} // namespace stlsoft
 #endif /* _STLSOFT_NO_NAMESPACE */
#endif /* !_WINSTL_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* !_WINSTL_INCL_H_WINSTL_PROCESS_MUTEX */

/* ////////////////////////////////////////////////////////////////////////// */
