/* /////////////////////////////////////////////////////////////////////////////
 * File:        winstl_token_information.h
 *
 * Purpose:     Helper for accessing token information.
 *
 * Created:     20th June 2003
 * Updated:     2nd July 2003
 *
 * Author:      Matthew Wilson, Synesis Software Pty Ltd.
 *
 * License:     (Licensed under the Synesis Software Standard Source License)
 *
 *              Copyright (C) 2002-2003, Synesis Software Pty Ltd.
 *
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/winstl
 *                          http://www.winstl.org/
 *
 *              email:      submissions@winstl.org  for submissions
 *                          admin@winstl.org        for other enquiries
 *
 *              Redistribution and use in source and binary forms, with or
 *              without modification, are permitted provided that the following
 *              conditions are met:
 *
 *              (i) Redistributions of source code must retain the above
 *              copyright notice and contact information, this list of
 *              conditions and the following disclaimer.
 *
 *              (ii) Any derived versions of this software (howsoever modified)
 *              remain the sole property of Synesis Software.
 *
 *              (iii) Any derived versions of this software (howsoever modified)
 *              remain subject to all these conditions.
 *
 *              (iv) Neither the name of Synesis Software nor the names of any
 *              subdivisions, employees or agents of Synesis Software, nor the
 *              names of any other contributors to this software may be used to
 *              endorse or promote products derived from this software without
 *              specific prior written permission.
 *
 *              This source code is provided by Synesis Software "as is" and any
 *              warranties, whether expressed or implied, including, but not
 *              limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software be liable for any direct, indirect,
 *              incidental, special, exemplary, or consequential damages
 *              (including, but not limited to, procurement of substitute goods
 *              or services; loss of use, data, or profits; or business
 *              interruption) however caused and on any theory of liability,
 *              whether in contract, strict liability, or tort (including
 *              negligence or otherwise) arising in any way out of the use of
 *              this software, even if advised of the possibility of such
 *              damage.
 *
 * ////////////////////////////////////////////////////////////////////////// */


#ifndef _WINSTL_INCL_H_WINSTL_TOKEN_INFORMATION
#define _WINSTL_INCL_H_WINSTL_TOKEN_INFORMATION

#ifndef __STLSOFT_DOCUMENTATION_SKIP_SECTION
#define _WINSTL_VER_H_WINSTL_TOKEN_INFORMATION_MAJOR    1
#define _WINSTL_VER_H_WINSTL_TOKEN_INFORMATION_MINOR    0
#define _WINSTL_VER_H_WINSTL_TOKEN_INFORMATION_REVISION 3
#define _WINSTL_VER_H_WINSTL_TOKEN_INFORMATION_EDIT     8
#endif /* !__STLSOFT_DOCUMENTATION_SKIP_SECTION */

/* /////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#ifndef _WINSTL_INCL_H_WINSTL
 #include "winstl.h"                        // Include the WinSTL root header
#endif /* _WINSTL_INCL_H_WINSTL */
#ifndef _WINSTL_INCL_H_WINSTL_PROCESSHEAP_ALLOCATOR
 #include "winstl_processheap_allocator.h"  // winstl::processheap_allocator
#endif /* !_WINSTL_INCL_H_WINSTL_PROCESSHEAP_ALLOCATOR */
#ifndef _STLSOFT_INCL_H_STLSOFT_EXCEPTIONS
 #include "stlsoft_exceptions.h"            // stlsoft::null_exception
#endif /* !_STLSOFT_INCL_H_STLSOFT_EXCEPTIONS */

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#ifndef _WINSTL_NO_NAMESPACE
 #ifdef _STLSOFT_NO_NAMESPACE
/* There is no stlsoft namespace, so must define ::winstl */
namespace winstl
{
 #else
/* Define stlsoft::winstl_project */

namespace stlsoft
{

namespace winstl_project
{

 #endif /* _STLSOFT_NO_NAMESPACE */
#endif /* !_WINSTL_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

/// \weakgroup libraries STLSoft Libraries
/// \brief The individual libraries

/// \weakgroup libraries_security Security Library
/// \ingroup libraries
/// \brief This library provides facilities for manipulating security APIs

/// \defgroup winstl_security_library Security Library (WinSTL)
/// \ingroup WinSTL libraries_security
/// \brief This library provides facilities for manipulating the Win32 security API
/// @{

/* /////////////////////////////////////////////////////////////////////////////
 * Classes
 */

template <TOKEN_INFORMATION_CLASS C>
struct token_information_traits;


template <>
struct token_information_traits<TokenUser>
{
    typedef TOKEN_USER  data_type;
};

template <>
struct token_information_traits<TokenGroups>
{
    typedef TOKEN_GROUPS    data_type;
};

template <>
struct token_information_traits<TokenPrivileges>
{
    typedef TOKEN_PRIVILEGES    data_type;
};

template <>
struct token_information_traits<TokenOwner>
{
    typedef TOKEN_OWNER data_type;
};

template <>
struct token_information_traits<TokenPrimaryGroup>
{
    typedef TOKEN_PRIMARY_GROUP data_type;
};

template <>
struct token_information_traits<TokenDefaultDacl>
{
    typedef TOKEN_DEFAULT_DACL  data_type;
};

template <>
struct token_information_traits<TokenSource>
{
    typedef TOKEN_SOURCE        data_type;
};

template <>
struct token_information_traits<TokenType>
{
    typedef TOKEN_TYPE  data_type;
};

template <>
struct token_information_traits<TokenImpersonationLevel>
{
    typedef SECURITY_IMPERSONATION_LEVEL    data_type;
};

template <>
struct token_information_traits<TokenStatistics>
{
    typedef TOKEN_STATISTICS    data_type;
};

template <>
struct token_information_traits<TokenRestrictedSids>
{
    typedef TOKEN_GROUPS    data_type;
};

template <>
struct token_information_traits<TokenSessionId>
{
    typedef DWORD   data_type;
};

// token_information
/// Provides typed access to token information.
///
template<   TOKEN_INFORMATION_CLASS C
#ifdef __STLSOFT_CF_TEMPLATE_CLASS_DEFAULT_CLASS_ARGUMENT_SUPPORT
#ifdef __SYNSOFT_DBS_COMPILER_SUPPORTS_PRAGMA_MESSAGE
 #pragma message(_sscomp_fileline_message("Note that we have to have data_type as a parameter, otherwise VC5&6 have a cow"))
#endif /* __SYNSOFT_DBS_COMPILER_SUPPORTS_PRAGMA_MESSAGE */
        ,   ws_typename_param_k     X = stlsoft_ns_qual(null_exception)
        ,   ws_typename_param_k     D = token_information_traits<C>::data_type
        ,   ws_typename_param_k     A = processheap_allocator<ss_byte_t>
#else
        ,   ws_typename_param_k     X
        ,   ws_typename_param_k     D /* = token_information_traits<C>::data_type */
        ,   ws_typename_param_k     A /* = processheap_allocator<ss_byte_t> */
#endif /* __STLSOFT_CF_TEMPLATE_CLASS_DEFAULT_CLASS_ARGUMENT_SUPPORT */
        >
class token_information
{
public:
    typedef token_information<C, X, D, A>               class_type;
    typedef token_information_traits<C>                 traits_type;
    typedef X                                           exception_thrower_type;
    typedef D                                           data_type;
    typedef A                                           allocator_type;
//  typedef ws_typename_type_k traits_type::data_type   data_type;

// Construction
public:
    /// \brief Constructs an instance from the given access token
    ///
    ws_explicit_k token_information(HANDLE hToken)
        : m_data(0)
    {
        DWORD   cbRequired;
        DWORD   dwError;

        ::GetTokenInformation(hToken, C, NULL, 0, &cbRequired);
        dwError = ::GetLastError();
        if(ERROR_INSUFFICIENT_BUFFER != dwError)
        {
            // Report error
            exception_thrower_type()(dwError);
        }
        else
        {
            data_type   *data = reinterpret_cast<data_type*>(allocator_type().allocate(cbRequired));

            if(NULL == data)
            {
                // Report error
                exception_thrower_type()(ERROR_NOT_ENOUGH_MEMORY);

                // Set the last error, in case the client code is not using exception reporting
                ::SetLastError(ERROR_NOT_ENOUGH_MEMORY);
            }
            else
            {
                if(!::GetTokenInformation(hToken, C, data, cbRequired, &cbRequired))
                {
                    // Scope the last error, in case the client code is not using exception reporting
                    LastErrorScope  scope;

                    allocator_type().deallocate(reinterpret_cast<ss_byte_t*>(data));

                    // Report error
                    exception_thrower_type()(DWORD((scope)));
                }
                else
                {
                    // Success
                    m_data = data;

                    ::SetLastError(ERROR_SUCCESS);
                }
            }
        }
    }
    ~token_information()
    {
        allocator_type().deallocate(reinterpret_cast<ss_byte_t*>(m_data));
    }

// Conversion
public:
    operator data_type *()
    {
        return m_data;
    }
    operator data_type const *() const
    {
        return m_data;
    }

    data_type *operator ->()
    {
        return m_data;
    }
    data_type const *operator ->() const
    {
        return m_data;
    }
/*
    operator ws_bool_t () const
    {
        return 0 != m_data;
    }
*/
    ws_bool_t operator !() const
    {
        return 0 == m_data;
    }

/// Implementation
private:

/// Members
private:
    data_type   *m_data;

/// Not to be implemented
private:
    token_information(token_information const &);
    token_information &operator =(token_information const &);
};

/* ////////////////////////////////////////////////////////////////////////// */

/// @} // end of group winstl_security_library

/* ////////////////////////////////////////////////////////////////////////// */

#ifndef _WINSTL_NO_NAMESPACE
 #ifdef _STLSOFT_NO_NAMESPACE
} // namespace winstl
 #else
} // namespace winstl_project
} // namespace stlsoft
 #endif /* _STLSOFT_NO_NAMESPACE */
#endif /* !_WINSTL_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* _WINSTL_INCL_H_WINSTL_TOKEN_INFORMATION */

/* ////////////////////////////////////////////////////////////////////////// */
