/* /////////////////////////////////////////////////////////////////////////////
 * File:        winstl_undocumented.h (formerly MCPriv.h, ::SynesisDev)
 *
 * Purpose:     Colour constants.
 *
 * Created:     20th October 1994
 * Updated:     2nd July 2003
 *
 * Author:      Matthew Wilson, Synesis Software Pty Ltd.
 *
 * License:     (Licensed under the Synesis Software Standard Source License)
 *
 *              Copyright (C) 2002-2003, Synesis Software Pty Ltd.
 *
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/winstl
 *                          http://www.winstl.org/
 *
 *              email:      submissions@winstl.org  for submissions
 *                          admin@winstl.org        for other enquiries
 *
 *              Redistribution and use in source and binary forms, with or
 *              without modification, are permitted provided that the following
 *              conditions are met:
 *
 *              (i) Redistributions of source code must retain the above
 *              copyright notice and contact information, this list of
 *              conditions and the following disclaimer.
 *
 *              (ii) Any derived versions of this software (howsoever modified)
 *              remain the sole property of Synesis Software.
 *
 *              (iii) Any derived versions of this software (howsoever modified)
 *              remain subject to all these conditions.
 *
 *              (iv) Neither the name of Synesis Software nor the names of any
 *              subdivisions, employees or agents of Synesis Software, nor the
 *              names of any other contributors to this software may be used to
 *              endorse or promote products derived from this software without
 *              specific prior written permission.
 *
 *              This source code is provided by Synesis Software "as is" and any
 *              warranties, whether expressed or implied, including, but not
 *              limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software be liable for any direct, indirect,
 *              incidental, special, exemplary, or consequential damages
 *              (including, but not limited to, procurement of substitute goods
 *              or services; loss of use, data, or profits; or business
 *              interruption) however caused and on any theory of liability,
 *              whether in contract, strict liability, or tort (including
 *              negligence or otherwise) arising in any way out of the use of
 *              this software, even if advised of the possibility of such
 *              damage.
 *
 * ////////////////////////////////////////////////////////////////////////// */


#ifndef _WINSTL_INCL_H_WINSTL_UNDOCUMENTED
#define _WINSTL_INCL_H_WINSTL_UNDOCUMENTED

#ifndef __STLSOFT_DOCUMENTATION_SKIP_SECTION
#define _WINSTL_VER_H_WINSTL_UNDOCUMENTED_MAJOR     1
#define _WINSTL_VER_H_WINSTL_UNDOCUMENTED_MINOR     0
#define _WINSTL_VER_H_WINSTL_UNDOCUMENTED_REVISION  4
#define _WINSTL_VER_H_WINSTL_UNDOCUMENTED_EDIT      8
#endif /* !__STLSOFT_DOCUMENTATION_SKIP_SECTION */

/* ////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#ifndef _WINSTL_INCL_H_WINSTL
 #include "winstl.h"                // Include the WinSTL root header
#endif /* !_WINSTL_INCL_H_WINSTL */

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#ifndef _WINSTL_NO_NAMESPACE
 #ifdef _STLSOFT_NO_NAMESPACE
/* There is no stlsoft namespace, so must define ::winstl */
namespace winstl
{
 #else
/* Define stlsoft::winstl_project */

namespace stlsoft
{

namespace winstl_project
{

 #endif /* _STLSOFT_NO_NAMESPACE */
#endif /* !_WINSTL_NO_NAMESPACE */

/* /////////////////////////////////////////////////////////////////////////////
 * Functions
 */

/** _get_undoc_clipboard_fmt
 *
 * \note This has to be implemented as a class, because the function version
 * does not correctly work in respect of the static. Specifically, it does not
 * distinguish between different values of the parameterising constant, so that,
 * say, _get_undoc_clipboard_fmt<1>(. . .) will cause fmt to be correctly
 * initialised, but _get_undoc_clipboard_fmt<2>(. . .) will use the same value.
 * This behaviour is demonstrated on VC5 & VC6, but works fine with Intel C++
 * 6 and 7, and VC7.
 */

#if defined(__STLSOFT_COMPILER_IS_BORLAND) || \
    (   defined(__STLSOFT_COMPILER_IS_DMC) && \
        __DMC__ >= 0x829) || \
    defined(__STLSOFT_COMPILER_IS_GCC) || \
    defined(__STLSOFT_COMPILER_IS_INTEL) || \
    (   defined(__STLSOFT_COMPILER_IS_MSVC) && \
        _MSC_VER >= 1300)
    
template <int i>
inline UINT _get_undoc_clipboard_fmt(LPCTSTR lpszFormat)
{
    static UINT fmt = ::RegisterClipboardFormat(lpszFormat);

    return fmt;
}

#else

template <int i>
struct _get_undoc_clipboard_fmt
{
public:
    explicit _get_undoc_clipboard_fmt(LPCTSTR lpszFormat)
#ifdef __STLSOFT_COMPILER_IS_DMC /* DMC++ pre 8.29 cannot work with the static _get() method */
        : m_fmt(_register(lpszFormat))
#else
        : m_fmt(_get(lpszFormat))
#endif /* __STLSOFT_COMPILER_IS_DMC */
    {}

    operator UINT () const
    {
        return m_fmt;
    }

private:
    static UINT _register(LPCTSTR lpszFormat)
    {
        return ::RegisterClipboardFormat(lpszFormat);
    }

    static UINT _get(LPCTSTR lpszFormat)
    {
        static UINT fmt = _register(lpszFormat);

        return fmt;
    }

private:
    UINT    m_fmt;
};

#endif /* compiler */

/* /////////////////////////////////////////////////////////////////////////////
 * Constants
 */

/* Messages */

#ifndef WM_CONTEXTMENU
 #define WM_CONTEXTMENU         0x007B
#endif /* !WM_CONTEXTMENU */

/* Clipboard formats */

#ifdef __SYNSOFT_DBS_COMPILER_SUPPORTS_PRAGMA_MESSAGE
 #pragma message(_sscomp_fileline_message("These should go into winstl_shell(_functions).h or something"))
#endif /* __SYNSOFT_DBS_COMPILER_SUPPORTS_PRAGMA_MESSAGE */

#ifndef CF_FILENAMEA
 #define CF_FILENAMEA           winstl_ns_qual(_get_undoc_clipboard_fmt)<1>(TEXT("FileName"))
#endif /* !CF_FILENAMEA */

#ifndef CF_FILENAMEW
 #define CF_FILENAMEW           winstl_ns_qual(_get_undoc_clipboard_fmt)<2>(TEXT("FileNameW"))
#endif /* !CF_FILENAMEW */

#ifndef CF_FILENAME
 #ifdef UNICODE
  #define CF_FILENAME           CF_FILENAMEW
 #else
  #define CF_FILENAME           CF_FILENAMEA
 #endif /* UNICODE */
#endif /* !CF_FILENAME */

#ifndef CF_IDLIST
 #define CF_IDLIST              winstl_ns_qual(_get_undoc_clipboard_fmt)<3>(TEXT("Shell IDList Array"))
#endif /* !CF_IDLIST */

#ifndef CF_NETRESOURCE
 #define CF_NETRESOURCE         winstl_ns_qual(_get_undoc_clipboard_fmt)<4>(TEXT("Net Resource"))
#endif /* !CF_NETRESOURCE */

/* ////////////////////////////////////////////////////////////////////////// */

#ifndef _WINSTL_NO_NAMESPACE
 #ifdef _STLSOFT_NO_NAMESPACE
} // namespace winstl
 #else
} // namespace winstl_project
} // namespace stlsoft
 #endif /* _STLSOFT_NO_NAMESPACE */
#endif /* !_WINSTL_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* _WINSTL_INCL_H_WINSTL_UNDOCUMENTED */

/* ////////////////////////////////////////////////////////////////////////// */
