/* /////////////////////////////////////////////////////////////////////////////
 * File:        wtlstl_simple_context_menu_handler.h (formerly MECtxHnd.h; ::SynesisAtl)
 *
 * Purpose:     Contains the SimpleContextMenuHandler template class.
 *
 * Created:     15th January 2000
 * Updated:     2nd July 2003
 *
 * Author:      Matthew Wilson, Synesis Software Pty Ltd.
 *
 * License:     (Licensed under the Synesis Software Standard Source License)
 *
 *              Copyright (C) 2002-2003, Synesis Software Pty Ltd.
 *
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/wtlstl
 *                          http://www.wtlstl.org/
 *
 *              email:      submissions@wtlstl.org  for submissions
 *                          admin@wtlstl.org        for other enquiries
 *
 *              Redistribution and use in source and binary forms, with or
 *              without modification, are permitted provided that the following
 *              conditions are met:
 *
 *              (i) Redistributions of source code must retain the above
 *              copyright notice and contact information, this list of
 *              conditions and the following disclaimer.
 *
 *              (ii) Any derived versions of this software (howsoever modified)
 *              remain the sole property of Synesis Software.
 *
 *              (iii) Any derived versions of this software (howsoever modified)
 *              remain subject to all these conditions.
 *
 *              (iv) Neither the name of Synesis Software nor the names of any
 *              subdivisions, employees or agents of Synesis Software, nor the
 *              names of any other contributors to this software may be used to
 *              endorse or promote products derived from this software without
 *              specific prior written permission.
 *
 *              This source code is provided by Synesis Software "as is" and any
 *              warranties, whether expressed or implied, including, but not
 *              limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software be liable for any direct, indirect,
 *              incidental, special, exemplary, or consequential damages
 *              (including, but not limited to, procurement of substitute goods
 *              or services; loss of use, data, or profits; or business
 *              interruption) however caused and on any theory of liability,
 *              whether in contract, strict liability, or tort (including
 *              negligence or otherwise) arising in any way out of the use of
 *              this software, even if advised of the possibility of such
 *              damage.
 *
 * ////////////////////////////////////////////////////////////////////////// */


#ifndef _WINSTL_INCL_H_WINSTL_SIMPLE_CONTEXT_MENU_HANDLER
#define _WINSTL_INCL_H_WINSTL_SIMPLE_CONTEXT_MENU_HANDLER

#ifndef __STLSOFT_DOCUMENTATION_SKIP_SECTION
#define _WINSTL_VER_H_WINSTL_SIMPLE_CONTEXT_MENU_HANDLER_MAJOR    1
#define _WINSTL_VER_H_WINSTL_SIMPLE_CONTEXT_MENU_HANDLER_MINOR    3
#define _WINSTL_VER_H_WINSTL_SIMPLE_CONTEXT_MENU_HANDLER_REVISION 1
#define _WINSTL_VER_H_WINSTL_SIMPLE_CONTEXT_MENU_HANDLER_EDIT     45
#endif /* !__STLSOFT_DOCUMENTATION_SKIP_SECTION */

/* /////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#ifndef _WINSTL_INCL_H_WINSTL
 #include "wtlstl.h"                // Include the WTLSTL root header
#endif /* !_WINSTL_INCL_H_WINSTL */

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#ifndef _WINSTL_NO_NAMESPACE
 #ifdef _STLSOFT_NO_NAMESPACE
/* There is no stlsoft namespace, so must define ::wtlstl */
namespace wtlstl
{
 #else
/* Define stlsoft::wtlstl_project */

namespace stlsoft
{

namespace wtlstl_project
{

 #endif /* _STLSOFT_NO_NAMESPACE */
#endif /* !_WINSTL_NO_NAMESPACE */

/* /////////////////////////////////////////////////////////////////////////////
 * Macros
 */

template <ss_typename_param_k C>
struct SimpleContextMenuItem_
{
    typedef void (C::*PFN)();

    UINT    idsMenuItem;
    UINT    idsCommandString;
    PFN     pfn;
};

#define BEGIN_SIMPLE_CONTEXT_MENU_MAP() \
private: \
    typedef wtlstl_ns_qual(SimpleContextMenuItem_)<class_type> ContextMenuItem; \
public: /* Alas this has to be the case, but at least ContextMenuItem is not */ \
    static ContextMenuItem const *WINAPI _GetContextMenuEntries(size_t *psize = NULL) \
    { \
        static ContextMenuItem const _entries[] = \
        {   \

#define SIMPLE_CONTEXT_MENU_ENTRY(idsm, idscs, pfn) \
            { idsm, idscs, pfn },

#define END_SIMPLE_CONTEXT_MENU_MAP() \
            { 0, 0, NULL } \
        }; \
        if(NULL != psize) \
        { \
            *psize = stlsoft_num_elements(_entries) - 1; \
        } \
        return _entries; \
    }


/* /////////////////////////////////////////////////////////////////////////////
 * Classes
 */

template <ss_typename_param_k C>
class SimpleContextMenuHandler
    : public IContextMenu
{
protected:
    typedef C                               boltee_type;
    typedef SimpleContextMenuHandler<C>     context_menu_handler_type;
private:
    typedef SimpleContextMenuItem_<C>       ContextMenuItem;

// IContextMenu
private:
    STDMETHOD(QueryContextMenu)(HMENU hmenu, UINT indexMenu, UINT idCmdFirst, UINT idCmdLast, UINT uFlags)
    {
        HRESULT hr;

        if(uFlags & CMF_DEFAULTONLY)
        {
            hr = S_OK;
        }
        else
        {
            size_t                  cItems;
            ContextMenuItem const   *items  =   boltee_type::_GetContextMenuEntries(&cItems);

            // Ensure we have enough room for all the menu items we want to present
            if(idCmdLast < idCmdFirst + cItems + 1)
            {
                hr = E_FAIL;
            }
            else
            {
                int     index;

                // For each item, load the string and add to the menu.
                for(hr = S_OK, index = 0; index < cItems; ++index)
                {
                    TCHAR                   szMenuItem[256];
                    ContextMenuItem const   &item   = items[index];

                    if( 0 == ::LoadString(_Module.GetResourceInstance(), item.idsMenuItem, szMenuItem, stlsoft_num_elements(szMenuItem)) ||
                        !::InsertMenu(hmenu, indexMenu++, MF_STRING | MF_BYPOSITION, idCmdFirst++, szMenuItem))
                    {
                        hr = HRESULT_FROM_WIN32(::GetLastError());

                        break;
                    }
                    else
                    {
                        ATLTRACE(_T("Menu item: %d => %d, %d\n"), index, item.idsMenuItem, item.idsCommandString);
                    }
                }

                if(!FAILED(hr))
                {
                    // Note: return S_OK + # of items
                    hr = MAKE_HRESULT(SEVERITY_SUCCESS, FACILITY_NULL, index);
                }
            }
        }

        return hr;
    }

    STDMETHOD(InvokeCommand)(LPCMINVOKECOMMANDINFO lpici)
    {
        HRESULT hr;

        if(0 == HIWORD(lpici->lpVerb))
        {
            UINT                    cmdOffset   =   LOWORD(lpici->lpVerb);
            size_t                  cItems;
            ContextMenuItem const   *items      =   boltee_type::_GetContextMenuEntries(&cItems);

            if(cmdOffset < cItems)
            {
                // Run the handler for the item
                (static_cast<boltee_type*>(this)->*items[cmdOffset].pfn)();

                hr = S_OK;
            }
            else
            {
                _ASSERTE(!"Unexpected menu item invoked");
                hr = E_UNEXPECTED;
            }
        }
        else
        {
            // No need to support string based command.
            hr = E_INVALIDARG;
        }

        return hr;
    }

    STDMETHOD(GetCommandString)(UINT cmdOffset, UINT uType, UINT *pwReserved, LPSTR pszName, UINT cchMax)
    {
        HRESULT hr;

        if(pszName != NULL)
        {
            *pszName = 0;
        }

        typedef int (WINAPI *PfnLoadString)(HINSTANCE , UINT , LPVOID , int);

        PfnLoadString   fns[2] = 
        {
                (PfnLoadString)LoadStringA
            ,   (PfnLoadString)LoadStringW
        };

        switch(uType)
        {
            default:
                _ASSERTE(!"Unrecognised GetCommandString() type");
            case    GCS_VERBA:
            case    GCS_VERBW:
                // Verbs are not supported currently
                hr = E_NOTIMPL;
                break;
            case    GCS_VALIDATEA:
            case    GCS_VALIDATEW:
                _ASSERTE(!"GCS_VALIDATE never expected by menu handlers");
                hr = E_UNEXPECTED;
                break;
            case    GCS_HELPTEXTA:
            case    GCS_HELPTEXTW:
                {
                    size_t                  cItems;
                    ContextMenuItem const   *items  =   boltee_type::_GetContextMenuEntries(&cItems);

                    if(cmdOffset < cItems)
                    {
                        ContextMenuItem const &item = items[cmdOffset];

                        if(fns[(uType & GCS_UNICODE) != 0]( _Module.GetResourceInstance(), 
                                                            item.idsCommandString,
                                                            pszName,
                                                            cchMax) > 0)
                        {
                            ATLTRACE(_T("Menu item: %d => %d, %d\n"), cmdOffset, item.idsMenuItem, item.idsCommandString);

                            hr = S_OK;
                        }
                        else
                        {
                            hr = E_INVALIDARG;
                        }
                    }
                    else
                    {
                        hr = E_UNEXPECTED;
                    }
                }
                break;
        }

        return hr;
    }
};

/* ////////////////////////////////////////////////////////////////////////// */

#ifndef _WINSTL_NO_NAMESPACE
 #ifdef _STLSOFT_NO_NAMESPACE
} // namespace wtlstl
 #else
} // namespace wtlstl_project
} // namespace stlsoft
 #endif /* _STLSOFT_NO_NAMESPACE */
#endif /* !_WINSTL_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* _WINSTL_INCL_H_WINSTL_SIMPLE_CONTEXT_MENU_HANDLER */

/* ////////////////////////////////////////////////////////////////////////// */
