/* ////////////////////////////////////////////////////////////////////////////
 * File:        mfcstl_resource_string.h (Formerly MWResStr.h: ::SynesisWin)
 *
 * Purpose:     resource_string class.
 *
 * Created:     1st November 1994
 * Updated:     2nd July 2003
 *
 * Author:      Matthew Wilson, Synesis Software Pty Ltd.
 *
 * License:     (Licensed under the Synesis Software Standard Source License)
 *
 *              Copyright (C) 2002-2003, Synesis Software Pty Ltd.
 *
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/mfcstl
 *                          http://www.mfcstl.org/
 *
 *              email:      submissions@mfcstl.org  for submissions
 *                          admin@mfcstl.org        for other enquiries
 *
 *              Redistribution and use in source and binary forms, with or
 *              without modification, are permitted provided that the following
 *              conditions are met:
 *
 *              (i) Redistributions of source code must retain the above
 *              copyright notice and contact information, this list of
 *              conditions and the following disclaimer.
 *
 *              (ii) Any derived versions of this software (howsoever modified)
 *              remain the sole property of Synesis Software.
 *
 *              (iii) Any derived versions of this software (howsoever modified)
 *              remain subject to all these conditions.
 *
 *              (iv) Neither the name of Synesis Software nor the names of any
 *              subdivisions, employees or agents of Synesis Software, nor the
 *              names of any other contributors to this software may be used to
 *              endorse or promote products derived from this software without
 *              specific prior written permission.
 *
 *              This source code is provided by Synesis Software "as is" and any
 *              warranties, whether expressed or implied, including, but not
 *              limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software be liable for any direct, indirect,
 *              incidental, special, exemplary, or consequential damages
 *              (including, but not limited to, procurement of substitute goods
 *              or services; loss of use, data, or profits; or business
 *              interruption) however caused and on any theory of liability,
 *              whether in contract, strict liability, or tort (including
 *              negligence or otherwise) arising in any way out of the use of
 *              this software, even if advised of the possibility of such
 *              damage.
 *
 * ////////////////////////////////////////////////////////////////////////// */


#ifndef _MFCSTL_INCL_H_MFCSTL_RESOURCE_STRING
#define _MFCSTL_INCL_H_MFCSTL_RESOURCE_STRING

#ifndef __STLSOFT_DOCUMENTATION_SKIP_SECTION
#define _MFCSTL_VER_H_MFCSTL_RESOURCE_STRING_MAJOR      1
#define _MFCSTL_VER_H_MFCSTL_RESOURCE_STRING_MINOR      5
#define _MFCSTL_VER_H_MFCSTL_RESOURCE_STRING_REVISION   3
#define _MFCSTL_VER_H_MFCSTL_RESOURCE_STRING_EDIT       33
#endif /* !__STLSOFT_DOCUMENTATION_SKIP_SECTION */

/* /////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#ifndef _MFCSTL_INCL_H_MFCSTL
 #include "mfcstl.h"    // Include the MFCSTL root header
#endif /* !_MFCSTL_INCL_H_MFCSTL */
#include <afxwin.h>     // AfxThrowResourceException()

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 *
 * The MFCSTL components are contained within the mfcstl namespace. This is
 * actually an alias for stlsoft::mfcstl_project,
 *
 * The definition matrix is as follows:
 *
 * _STLSOFT_NO_NAMESPACE    _MFCSTL_NO_NAMESPACE    mfcstl definition
 * ---------------------    --------------------    -----------------
 *  not defined              not defined             = stlsoft::mfcstl_project
 *  not defined              defined                 not defined
 *  defined                  not defined             mfcstl
 *  defined                  defined                 not defined
 *
 */

#ifndef _MFCSTL_NO_NAMESPACE
 #ifdef _STLSOFT_NO_NAMESPACE
/* There is no stlsoft namespace, so must define ::mfcstl */
namespace mfcstl
{
 #else
/* Define stlsoft::mfcstl_project */

namespace stlsoft
{

namespace mfcstl_project
{

 #endif /* _STLSOFT_NO_NAMESPACE */
#endif /* !_MFCSTL_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

/// \weakgroup libraries STLSoft Libraries
/// \brief The individual libraries

/// \weakgroup libraries_string String Library
/// \ingroup libraries
/// \brief This library provides facilities for defining and manipulating strings

/// \defgroup mfcstl_string_library String Library (MFCSTL)
/// \ingroup MFCSTL libraries_string
/// \brief This library provides facilities for defining and manipulating MFC string types
/// @{

/* ////////////////////////////////////////////////////////////////////////// */

// resource_string
/// Instances of this class represent Windows string resources, and are 
/// constructed from instance identifiers.
class resource_string
    : public CString
{
private:
    typedef CString         parent_class_type;
    typedef resource_string class_type;
public:
    /// The type of the const (non-mutating) iterator
    typedef LPCTSTR         const_iterator;
    /// The size type
    typedef UINT            size_type;

// Construction
public:
    /// Constructs an around the string loaded from the given \c id
    ///
    /// \param id identifier of the string resource to load
    ms_explicit_k resource_string(ms_uint_t id) mfcstl_throw_2(CMemoryException*, CResourceException*);
    /// Constructs an around the string loaded from the given \c id and \c hinst
    ///
    /// \param hinst The module from which to load the string
    /// \param id identifier of the string resource to load
    resource_string(HINSTANCE hinst, ms_uint_t id) mfcstl_throw_2(CMemoryException*, CResourceException*);
    /// Copy constructor
    ///
    /// \param rhs The instance from which to copy-construct
    resource_string(resource_string const &rhs);
    /// Copy constructor
    ///
    /// \param rhs The instance from which to copy-construct
    resource_string(CString const &rhs);

    /// Copy assignment operator
    ///
    /// \param rhs The instance from which to copy-assign
    resource_string const &operator =(resource_string const &rhs);
    /// Copy assignment operator
    ///
    /// \param rhs The instance from which to copy-assign
    resource_string const &operator =(CString const &rhs);

// Iteration
public:
    /// Begins the iteration
    ///
    /// \return An iterator representing the start of the sequence
    const_iterator  begin() const;
    /// Ends the iteration
    ///
    /// \return An iterator representing the end of the sequence
    const_iterator  end() const;

// Attributes
public:
    /// Returns the number of elements in the sequence
    size_type size() const;
    /// Returns the number of elements in the sequence
    size_type length() const;
    /// Returns a pointer to constant data representing the managed string
    LPCTSTR c_str() const;
};

///////////////////////////////////////////////////////////////////////////////
// Implementation

inline resource_string::resource_string(ms_uint_t id) mfcstl_throw_2(CMemoryException*, CResourceException*)
{
    if(!parent_class_type::LoadString(id))
    {
        AfxThrowResourceException();
    }
}

inline resource_string::resource_string(HINSTANCE hinst, ms_uint_t id) mfcstl_throw_2(CMemoryException*, CResourceException*)
{
    TCHAR sz[1024];

    if(0 == ::LoadString(hinst, id, sz, stlsoft_num_elements(sz)))
    {
        AfxThrowResourceException();
    }
    else
    {
        parent_class_type   *pThis = this;

        pThis->operator =(sz);
    }
}

inline resource_string::resource_string(resource_string const &rhs)
  : parent_class_type(rhs)
{}

inline resource_string::resource_string(CString const &rhs)
  : parent_class_type(rhs)
{}

inline resource_string const &resource_string::operator =(resource_string const &rhs)
{
    parent_class_type   *pThis = this;

    pThis->operator =(rhs);

    return *this;
}

inline resource_string const &resource_string::operator =(CString const &rhs)
{
    parent_class_type   *pThis = this;

    pThis->operator =(rhs);

    return *this;
}

inline resource_string::const_iterator resource_string::begin() const
{
    return *this;
}

inline resource_string::const_iterator resource_string::end() const
{
    return begin() + length();
}

inline resource_string::size_type resource_string::size() const
{
    return GetLength();
}

inline resource_string::size_type resource_string::length() const
{
    return GetLength();
}

inline LPCTSTR resource_string::c_str() const
{
    return *this;
}

/* ////////////////////////////////////////////////////////////////////////// */

/// @} // end of group mfcstl_string_library

/* ////////////////////////////////////////////////////////////////////////// */

#ifndef _MFCSTL_NO_NAMESPACE
 #ifdef _STLSOFT_NO_NAMESPACE
} // namespace mfcstl
 #else
} // namespace mfcstl_project
} // namespace stlsoft
 #endif /* _STLSOFT_NO_NAMESPACE */
#endif /* !_MFCSTL_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* !_MFCSTL_INCL_H_MFCSTL_RESOURCE_STRING */

/* ////////////////////////////////////////////////////////////////////////// */
