/* /////////////////////////////////////////////////////////////////////////////
 * File:        whereis.cpp
 *
 * Purpose:     Implementation file for the Synesis Software whereis utility
 *
 * Created:     19th January 1996
 * Updated:     21st August 2003
 *
 * Author:      Matthew Wilson, Synesis Software Pty Ltd.
 *
 *              The timestr() function and the verbose formatting in trace_file
 *              used by kind permission of Walter Bright and Digital Mars.
 *
 * License:     (Licensed under the Synesis Software Standard Public License)
 *
 *              Copyright (C) 1999-2003, Synesis Software Pty Ltd.
 *              Copyright (C) 1987-2003, Digital Mars.
 *
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au
 *                          http://www.synesis.com.au/software
 *
 *                          http://www.digitalmars.com/
 *
 *              email:      software@synesis.com.au
 *
 *                          contact@digitalmars.com
 *
 *              Redistribution and use in source and binary forms, with or
 *              without modification, are permitted provided that the following
 *              conditions are met:
 *
 *              (i) Redistributions of source code must retain the above
 *              copyright notice and contact information, this list of
 *              conditions and the following disclaimer.
 *
 *              (ii) Redistributions in binary form must reproduce the above
 *              copyright notice and contact information, this list of
 *              conditions and the following disclaimer in the documentation
 *              and/or other materials provided with the distribution.
 *
 *              (iii) Any derived versions of this software (howsoever modified)
 *              remain the sole property of Synesis Software.
 *
 *              (iv) Any derived versions of this software (howsoever modified)
 *              remain subject to all these conditions.
 *
 *              (v) Neither the name of Synesis Software nor the names of any
 *              subdivisions, employees or agents of Synesis Software, nor the
 *              names of any other contributors to this software may be used to
 *              endorse or promote products derived from this software without
 *              specific prior written permission.
 *
 *              This source code is provided by Synesis Software "as is" and any
 *              warranties, whether expressed or implied, including, but not
 *              limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software be liable for any direct, indirect,
 *              incidental, special, exemplary, or consequential damages
 *              (including, but not limited to, procurement of substitute goods
 *              or services; loss of use, data, or profits; or business
 *              interruption) however caused and on any theory of liability,
 *              whether in contract, strict liability, or tort (including
 *              negligence or otherwise) arising in any way out of the use of
 *              this software, even if advised of the possibility of such
 *              damage.
 *
 * ////////////////////////////////////////////////////////////////////////// */


/* /////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#include <stdio.h>

#include <stlsoft.h>

/* Regrettably, something in this program is too complex for early versions of 
 * many compilers that the STLSoft libraries support. With some it is a compiler
 * issue, with others it either crashes or produces erroneous output. Thus, the
 * following restrictions are mandated:
 */
#if defined(__STLSOFT_COMPILER_IS_BORLAND)
 #if __BORLANDC__ < 0x0560
  #error whereis.cpp cannot be successfully built by Borland C/C++ prior to version 5.6 
 #endif /* __BORLANDC__ < 0x0560 */
#elif defined(__STLSOFT_COMPILER_IS_DMC)
 #if __DMC__ < 0x0832
  #error whereis.cpp cannot be successfully built by Digital Mars C/C++ prior to version 8.32 
 #endif /* __DMC__ < 0x0832 */
#elif defined(__STLSOFT_COMPILER_IS_GCC)
 #if __GNUC__ < 3 && \
     __GNUC_MINOR__ < 95
  #error whereis.cpp cannot be successfully built by GNU C/C++ prior to version 6.0
 #endif /* __GNUC__ < 3 && __GNUC_MINOR__ < 95 */
#elif defined(__STLSOFT_COMPILER_IS_INTEL)
 #if __INTEL_COMPILER < 600
  #error whereis.cpp cannot be successfully built by Intel C/C++ prior to version 6.0
 #endif /* __INTEL_COMPILER < 600 */
#elif defined(__STLSOFT_COMPILER_IS_MWERKS)
 /* No problems with Metrowerks */
#elif defined(__STLSOFT_COMPILER_IS_MSVC)
 #if _MSC_VER < 1100
  #error whereis.cpp cannot be successfully built by Microsoft Visual C/C++ prior to version 5.0
 #endif /* _MSC_VER < 1100 */
#else
 #error Unrecognised compiler
#endif /* compiler */

/* /////////////////////////////////////////////////////////////////////////////
 * Warnings
 */

#if defined(__STLSOFT_COMPILER_IS_MSVC)
 #pragma warning(disable : 4702)
#endif /* __STLSOFT_COMPILER_IS_INTEL */

/* ////////////////////////////////////////////////////////////////////////// */

#ifdef _SYNSOFT_INTERNAL_BUILD
 #include "MLCompId.h"
#else
 #define _nameSynesisSoftware				"Synesis Software (Pty) Ltd"
 #define _wwwSynesisSoftware_SystemTools	"http://synesis.com.au/"
 #define _emailSynesisSoftware_SystemTools	"software@synesis.com.au"
#endif /* _SYNSOFT_INTERNAL_BUILD */

/* ////////////////////////////////////////////////////////////////////////// */

stlsoft_ns_using(ss_sint64_t)
stlsoft_ns_using(ss_bool_t)

#include <stlsoft_limit_traits.h>

#include <stlsoft_string_access.h>

stlsoft_ns_using(c_str_ptr)

#include <stlsoft_simple_algorithms.h>

stlsoft_ns_using(remove_duplicates_from_unordered_sequence)

#include <stlsoft_string_tokeniser.h>

stlsoft_ns_using(string_tokeniser)

#if defined(unix) || \
    defined(__unix) || \
    defined(__unix__) || \
    (	defined(__xlC__) && \
        defined(_POWER) && \
        defined(_AIX))

 #include <unixstl.h>

 #include <unixstl_findfile_sequence.h>

 typedef unixstl_ns_qual(findfile_sequence_a)	findfile_sequence_a;

 #include <unixstl_functionals.h>

 unixstl_ns_using(compare_path)

 #include <unixstl_environment_variable.h>

 unixstl_ns_using(basic_environment_variable)

#elif defined(WIN32) || \
	  defined(_WIN32)

 #include <winstl.h>

 #include <winstl_findfile_sequence.h>

 typedef winstl_ns_qual(findfile_sequence_a)	findfile_sequence_a;

 #include <winstl_functionals.h>

 winstl_ns_using(compare_path)

 #include <winstl_environment_variable.h>

 winstl_ns_using(basic_environment_variable)

#else

 #error Operating system not recognised

#endif /* operating system */
#include <algorithm>

stlsoft_ns_using_std(copy)
stlsoft_ns_using_std(for_each)
stlsoft_ns_using_std(remove_if)

#include <functional>

stlsoft_ns_using_std(unary_function)

#include <iterator>

stlsoft_ns_using_std(back_inserter)

#include <string>

stlsoft_ns_using_std(string)

#include <vector>

stlsoft_ns_using_std(vector)

#include <list>

stlsoft_ns_using_std(list)

/* /////////////////////////////////////////////////////////////////////////////
 * Warnings
 */

#if defined(__STLSOFT_COMPILER_IS_INTEL)
 #pragma warning(disable : 383)
 #pragma warning(disable : 444)
#endif /* __STLSOFT_COMPILER_IS_INTEL */

/* /////////////////////////////////////////////////////////////////////////////
 * Tracing
 */

//#define WHEREIS_TRACE

#if !defined(WHEREIS_TRACE) && \
	!defined(_DEBUG)
 #define WHEREIS_TRACE
#endif /* !WHEREIS_TRACE && !_DEBUG */

/* /////////////////////////////////////////////////////////////////////////////
 * Version information
 *
 * Normally, Synesis Software components & tools are "versioned" by making use
 * of tool-generated header files, one of which - MBldHdr.h - defines symbolic
 * constants representing major, minor, revision and build numbers.
 *
 *
 * Since this file is to be available in a more public forum, including on the
 * Digital Mars site, these constants are merely defined here, rather than
 * encumber any developers who may wish to compile it with a lot of
 * Synesis-specific gunk.
 */

#ifdef _SYNSOFT_INTERNAL_BUILD
 #include "MBldHdr.h"

 const int	_mccVerHi	=	__SYV_MAJOR;
 const int	_mccVerLo	=	__SYV_MINOR;
 const int	_mccVerRev	=	__SYV_REVISION;
 const int	_mccBldNum	=	__SYV_BUILDNUMBER;

#else
 const int	_mccVerHi	=	1;
 const int	_mccVerLo	=	5;
 const int	_mccVerRev	=	3;
 const int	_mccBldNum	=	41;
#endif /* _SYNSOFT_INTERNAL_BUILD */

/* /////////////////////////////////////////////////////////////////////////////
 * Typedefs
 */

typedef	string_tokeniser<string, char>	tokeniser_string_t;
typedef	vector<string>					searchpath_sorted_t;
//typedef	list<string>				searchpath_sorted_t;

/* /////////////////////////////////////////////////////////////////////////////
 * Macros and definitions
 *
 * Visual C++, sigh. uses %I64d", rather than "%lld", so that is abstracted here
 */

#if defined(__STLSOFT_COMPILER_IS_BORLAND) || \
	defined(__STLSOFT_COMPILER_IS_GCC) || \
	defined(__STLSOFT_COMPILER_IS_MSVC)
#define	_FMT_SINT64_WIDTH(x)			"%" #x "I64d"
#else
#define	_FMT_SINT64_WIDTH(x)			"%" #x "lld"
#endif /* __BORLANDC__ || _MSC_VER */

/* /////////////////////////////////////////////////////////////////////////////
 * Constants
 */

char const	DELIMITER	=	';';

/* /////////////////////////////////////////////////////////////////////////////
 * Forward declarations
 *
 * (Primarily to keep Metrowerks happy, but that's a good thing to do!)
 */

static char *timestr(char *buffer, FILETIME const *ft);	// Make Metrowerks happy

/* /////////////////////////////////////////////////////////////////////////////
 * Helper Functions
 */

char *timestr(char *buffer, FILETIME const *ft)
{
    FILETIME	ftLocal;
    SYSTEMTIME	st;
	char		dateString[41];
	char		timeString[41];

	::FileTimeToLocalFileTime(ft, &ftLocal);
    ::FileTimeToSystemTime(&ftLocal, &st);

	::GetDateFormat(LOCALE_USER_DEFAULT, 0, &st, NULL, dateString, stlsoft_num_elements(dateString));
	::GetTimeFormat(LOCALE_USER_DEFAULT, 0, &st, NULL, timeString, stlsoft_num_elements(timeString));

#if 0
	// User-locale format
	sprintf(buffer, "%s  %s", dateString, timeString);
#else
	// Digital Mars format (US based)
    sprintf(buffer, "%02d/%02d/%02d  %2d:%02d", st.wMonth, st.wDay, st.wYear % 100, st.wHour, st.wMinute);
#endif /* 0 */

    return buffer;
}

/* /////////////////////////////////////////////////////////////////////////////
 * Functionals
 */

// struct trace_file
//
// This functional traces individual file records to stdout.

struct trace_file
	: public unary_function<findfile_sequence_a::value_type const &, void>
{
public:
	trace_file(bool bVerbose)
		: m_bVerbose(bVerbose)
	{}

	void operator ()(findfile_sequence_a::value_type const &value) const
	{
		if(m_bVerbose)
		{
			findfile_sequence_a::find_data_type const	&findData	=	value.get_find_data();
			DWORD const									&att		=	findData.dwFileAttributes;
			ss_sint64_t									size;
			char										atts[8 + 1];
			char										timebuf[50];
			int											i;

			for (i = 0; i < 8; ++i)
			{
				static char tab[] = "RHSVDAXX";

				if (att & (1 << i))
					atts[i] = tab[i];
				else
					atts[i] = '-';
			}
			atts[8] = 0;

			timestr(timebuf, &findData.ftLastWriteTime);
			size = findData.nFileSizeHigh * __STLSOFT_GEN_SINT64_SUFFIX(0x100000000) + findData.nFileSizeLow;
			printf("%s  %s  " _FMT_SINT64_WIDTH(9) "\t%s\n", timebuf, atts, size, value.get_path());
		}
		else
		{
			fprintf(stdout, "%s\n", c_str_ptr(value));
		}
	}

// Members
private:
	ss_bool_t	m_bVerbose;
};

// struct process_searchspec
//
// This functional processes a search-spec within a given directory, passing any
// file to be processed by trace_file.

struct process_searchspec
	: public unary_function<tokeniser_string_t::value_type const &, void>
{
public:
	process_searchspec(string const &path, ss_bool_t bVerbose)
		: m_path(path)
		, m_bVerbose(bVerbose)
	{}

	void operator ()(tokeniser_string_t::value_type const &value) const
	{
		findfile_sequence_a	files(c_str_ptr(m_path), c_str_ptr(value), findfile_sequence_a::files);

		for_each(files.begin(), files.end(), trace_file(m_bVerbose));
	}

// Members
private:
	string const	m_path;
	ss_bool_t		m_bVerbose;

// Not to be implemented
private:
//	process_searchspec(process_searchspec const &);
	process_searchspec &operator =(process_searchspec const &);
};

// struct process_path
//
// This functional creates a search sequence for each path, and then processes
// each one with the trace_file functional.

struct process_path
	: public unary_function<string const &, void>
{
public:
	process_path(string const &searchSpec, ss_bool_t bVerbose)
		: m_searchSpec(searchSpec)
		, m_bVerbose(bVerbose)
	{}

	void operator ()(string const &value) const
	{
		// Now split with the string_tokeniser into all the constituent paths
		tokeniser_string_t		specs(m_searchSpec, DELIMITER);

		for_each(specs.begin(), specs.end(), process_searchspec(value, m_bVerbose));
	}

// Members
private:
	string const	m_searchSpec;
	ss_bool_t		m_bVerbose;

// Not to be implemented
private:
//	process_path(process_path const &);
	process_path &operator =(process_path const &);
};

// struct trace_dir
//
// This functional provides processing of a directory, used by the recursive
// option. It process the current directory (via process_path), and then all
// subdirectories (via trace_dir) "calling" itself recursively.

struct trace_dir
	: public unary_function<string const &, void>
{
public:
	trace_dir(string const &searchSpec, ss_bool_t bVerbose)
		: m_searchSpec(searchSpec)
		, m_bVerbose(bVerbose)
	{}

	void operator ()(char const *value) const
	{
		process_path	f(m_searchSpec, m_bVerbose);

		f(value);

		findfile_sequence_a		directories(c_str_ptr(value), "*.*", findfile_sequence_a::directories);

		for_each(directories.begin(), directories.end(), trace_dir(m_searchSpec, m_bVerbose));
	}

	template <ss_typename_param_k S>
	void operator ()(S const &value) const
	{
		operator ()(c_str_ptr(value));
	}

private:
	string const	m_searchSpec;
	ss_bool_t		m_bVerbose;

// Not to be implemented
private:
//	trace_dir(trace_dir const &);
	trace_dir &operator =(trace_dir const &);
};

/* /////////////////////////////////////////////////////////////////////////////
 * Forward declarations
 */

static void usage(int bExit);
static char const *convert_path_separators(char const *path, char *buffer);
static char const *get_driveroots(char *buffer);

/* /////////////////////////////////////////////////////////////////////////////
 * Helper functionals
 */

struct is_empty_path
{
	template <ss_typename_param_k S>
	bool operator ()(S const &s) const
	{
		return c_str_ptr(s)[0] == '\0';
	}
};

struct print_path
{
	template <ss_typename_param_k S>
	void operator ()(S const &s) const
	{
		fprintf(stdout, " %s\n", c_str_ptr(s));
	}
};

#ifdef WHEREIS_TRACE
struct trace_path
{
	template <ss_typename_param_k S>
	void operator ()(S const &s) const
	{
		char	sz[2 + _MAX_PATH + 1];

		sprintf(sz, " %s\n", c_str_ptr(s));

		OutputDebugStringA(sz);
	}
};
#endif /* WHEREIS_TRACE */

/* /////////////////////////////////////////////////////////////////////////////
 * main
 */

int main(int argc, char **argv)
{
	if(	argc < 2 ||
		argc > 5)
	{
		usage(true);
	}
	else
	{
		bool		bFromEnvironment	=	false;
		bool		bSearchGivenRoots	=	false;
		bool		bSearchCwd			=	false;
		bool		bDumpSearchPaths	=	false;
		bool		bVerbose			=	true;
		bool		bRecursive			=	false;
		char const	*rootSpec			=	NULL;
		char const	*searchSpec			=	NULL;
		char const	*envSpec			=	NULL;
		char		driveroots[105]		=	"";

		for(int i = 1; i < argc; ++i)
		{
			char const	*arg	=	argv[i];

			if(arg[0] == '-')
			{
				switch(arg[1])
				{
					case	'w':
					case	'W':
						bSearchCwd			=	true;
						break;
					case	'p':
					case	'P':
						bFromEnvironment	=	true;
						envSpec				=	"PATH";
						break;
					case	'h':
					case	'H':
						bSearchGivenRoots	=	true;
						rootSpec = get_driveroots(driveroots);
						break;
					case	'i':
					case	'I':
						bFromEnvironment	=	true;
						envSpec				=	"INCLUDE";
						break;
					case	'l':
					case	'L':
						bFromEnvironment	=	true;
						envSpec				=	"LIB";
						break;
					case	'e':
					case	'E':
						bFromEnvironment	=	true;
						envSpec				=	arg + 2;
						break;
					case	'r':
					case	'R':
						bSearchGivenRoots	=	true;
						rootSpec = arg + 2;
						break;
					case	'd':
					case	'D':
						bDumpSearchPaths	=	true;
						break;
					case	'v':
					case	'V':
						bVerbose			=	true;
						break;
					case	'u':
					case	'U':
						bRecursive			=	true;
						break;
					case	's':
					case	'S':
						bVerbose			=	false;
						break;
					default:
						fprintf(stderr, "Unrecognised argument \"%s\"\n", arg);
						usage(true);
						break;
				}
			}
			else
			{
				if(rootSpec == NULL)
				{
					rootSpec = arg;
				}
				else if(searchSpec == NULL)
				{
					searchSpec = arg;
				}
				else
				{
					fprintf(stderr, "<root-paths> and <search-spec> already specified. Argument \"%s\"\n", arg);
					usage(true);
				}
			}
		}

		if(	((bSearchCwd != false) +
			(bFromEnvironment != false) +
			(bSearchGivenRoots != false)) > 1)
		{
			fprintf(stderr, "Cannot specify two or more of -w, -r, -e, -l, -i, -p\n");
			usage(true);
		}

		// Validate the args. This entails just moving the root-dir into the
		// search-spec if no root-dir was specified. This simplifies the
		// previous command-line processing.
		if(NULL == searchSpec)
		{
			if( NULL != rootSpec &&
				!bSearchGivenRoots)
			{
				searchSpec = rootSpec;
				rootSpec = NULL;
			}
			else
			{
				fprintf(stderr, "Unexpected condition\n");
				usage(true);
			}
		}

		char	_rootDir[_MAX_PATH + 1];

		if(bFromEnvironment)
		{
			rootSpec = NULL;
		}
		else
		{
			// If no directory is specified, the current directory is assumed
			if(	rootSpec == NULL ||
				*rootSpec == '\0')
			{
				::GetCurrentDirectory(winstl_num_elements(_rootDir), _rootDir);
				rootSpec = _rootDir;
			}
		}

		if(	bFromEnvironment &&
			(	envSpec == NULL ||
				*envSpec == '\0'))
		{
			fprintf(stderr, "Cannot search from environment when no environment variable specified\n");
			usage(true);
		}

		// Handle forward slashes
		string	_rootDir_(rootSpec != NULL ? rootSpec : "");
		string	_searchSpec_(searchSpec != NULL ? searchSpec : "");

		rootSpec	=	convert_path_separators(rootSpec, const_cast<char*>(_rootDir_.data()));
		searchSpec	=	convert_path_separators(searchSpec, const_cast<char*>(_searchSpec_.data()));

		// Handle illegal characters
		if(strpbrk(searchSpec, ":\\") != 0)
		{
			fprintf(stderr, "<search-spec> contains illegal characters: \"%s\"\n", searchSpec);
			usage(true);
		}

		searchpath_sorted_t		sorted;

		// Commence the search
		if(bFromEnvironment)
		{
			basic_environment_variable<char>		ENVVAR(envSpec);
			tokeniser_string_t						paths(ENVVAR, DELIMITER);

#ifdef __STLSOFT_COMPILER_IS_DMC
			{
				tokeniser_string_t::const_iterator	begin	=	paths.begin();
				tokeniser_string_t::const_iterator	end		=	paths.end();

				for(; begin != end; ++begin)
				{
					sorted.push_back(*begin);
				}
			}
#else
			copy(paths.begin(), paths.end(), back_inserter(sorted));
#endif /* __STLSOFT_COMPILER_IS_DMC */

			// The roots may contain duplicate elements, so 
			remove_duplicates_from_unordered_sequence(sorted, compare_path<char>());

			// They may also contain empty paths, so
			sorted.erase(remove_if(sorted.begin(), sorted.end(), is_empty_path()), sorted.end());

			if(bDumpSearchPaths)
			{
				fprintf(stdout, "Search roots:\n");
				for_each(sorted.begin(), sorted.end(), print_path());
			}

			// Now execute over the duplicate-free (but still in the correct search order)
			// sequence, applying the process_path functional to each one.
			if(bRecursive)
			{
				for_each(sorted.begin(), sorted.end(), trace_dir(searchSpec, bVerbose));
			}
			else
			{
				for_each(sorted.begin(), sorted.end(), process_path(searchSpec, bVerbose));
			}
		}
		else
		{
			tokeniser_string_t						roots(rootSpec, DELIMITER);

#ifdef __STLSOFT_COMPILER_IS_DMC
			{
				tokeniser_string_t::const_iterator	begin	=	roots.begin();
				tokeniser_string_t::const_iterator	end		=	roots.end();

				for(; begin != end; ++begin)
				{
					sorted.push_back(*begin);
				}
			}
#else
			copy(roots.begin(), roots.end(), back_inserter(sorted));
#endif /* __STLSOFT_COMPILER_IS_DMC */

			// The roots may contain duplicate elements, so 
			remove_duplicates_from_unordered_sequence(sorted, compare_path<char>());

			// They may also contain empty paths, so
			sorted.erase(remove_if(sorted.begin(), sorted.end(), is_empty_path()), sorted.end());

			if(bDumpSearchPaths)
			{
				fprintf(stdout, "Search roots:\n");
				for_each(sorted.begin(), sorted.end(), print_path());
			}

			// Now execute over the duplicate-free (but still in the correct search order)
			// sequence, applying the trace_dir functional to each one.
			for_each(sorted.begin(), sorted.end(), trace_dir(searchSpec, bVerbose));
		}
	}

	return 0;
}

/* /////////////////////////////////////////////////////////////////////////////
 * usage
 */

void usage(int bExit)
{
    fprintf(stderr, "\n");
    fprintf(stderr, "  Synesis Software whereis Utility v%d.%d.%d.%04d\n\n", _mccVerHi, _mccVerLo, _mccVerRev, _mccBldNum);
    fprintf(stderr, "    incorporating Digital Mars technology\n\n");
    fprintf(stderr, "  USAGE: whereis [{-w | -r<root-paths> | -p | -i | -l | -e<env-var> | -h}] [-u] [-d] [{-v | -s}] <search-spec>\n");
    fprintf(stderr, "    where:\n\n");
    fprintf(stderr, "    -w             - searches from the current working directory\n");
    fprintf(stderr, "    -r<root-paths> - searches from the given root path(s), separated by \';\',\n");
    fprintf(stderr, "                     eg.\n");
    fprintf(stderr, "                       -r\"c:\\windows;x:\\bin\"\n");
    fprintf(stderr, "    -p             - searches in the Windows paths (the directories specified in the PATH environment variable)\n");
    fprintf(stderr, "    -i             - searches in the directories specified in the INCLUDE environment variable\n");
    fprintf(stderr, "    -l             - searches in the directories specified in the LIB environment variable\n");
    fprintf(stderr, "    -e<env-var>    - searches in the directories specified in the environment variable <env-var>\n");
    fprintf(stderr, "    -h             - searches from the roots of all drives on the system\n");
    fprintf(stderr, "    -d             - displays the path(s) searched\n");
    fprintf(stderr, "    -u             - recursive search. (Default except for environment variable searches.)\n");
    fprintf(stderr, "    -v             - verbose output. Prints time, attributes, size and path. (default)\n");
    fprintf(stderr, "    -s             - succinct output. Prints path only\n");
    fprintf(stderr, "    <search-spec>  - one or more file search specifications, separated by \';\',\n");
    fprintf(stderr, "                     eg.\n");
    fprintf(stderr, "                       \"*.exe\"\n");
    fprintf(stderr, "                       \"myfile.ext\"\n");
    fprintf(stderr, "                       \"*.exe;*.dll\"\n");
    fprintf(stderr, "                       \"*.xl?;report.*\"\n");
    fprintf(stderr, "\n");
	fprintf(stderr, "  Contact %s\n", _nameSynesisSoftware);
	fprintf(stderr, "    at \"%s\",\n", _wwwSynesisSoftware_SystemTools);
	fprintf(stderr, "    or, via email, at \"%s\"\n", _emailSynesisSoftware_SystemTools);
    fprintf(stderr, "\n");
	fprintf(stderr, "  Contact Digital Mars\n");
	fprintf(stderr, "    at \"www.digitalmars.com\",\n");
	fprintf(stderr, "    or, via email, at \"software@digitalmars.com\"\n");
    fprintf(stderr, "\n");

    if(bExit)
    {
        exit(1);
    }
}

char const *convert_path_separators(char const *path, char *buffer)
{
	if(	path != 0 &&
		0 != strchr(path, '/'))
	{
		char	*pch;

		path = strcpy(buffer, path);

		for(pch = buffer; 0 != (pch = strchr(pch, '/')); )
		{
			*pch = '\\';
		}
	}

	return path;
}

char const *get_driveroots(char *const buffer)
{
	char	drv[4] = "?:\\";

	*buffer = '\0';

	for(char ch = 'a'; ch <= 'z'; ++ch)
	{
		drv[0] = ch;

		long	type	=	GetDriveTypeA(drv);

		if(DRIVE_FIXED == type)
		{
			strcat(buffer, drv);
			strcat(buffer, ";");
		}
	}

	return buffer;
}

/* ////////////////////////////////////////////////////////////////////////// */
