/* /////////////////////////////////////////////////////////////////////////////
 * File:        stlsoft_explicit_inheritance_veneer.h
 *
 * Purpose:     Explicit inheritance veneer class.
 *
 * Created:     13th January 2003
 * Updated:     2nd July 2003
 *
 * Author:      Matthew Wilson, Synesis Software Pty Ltd.
 *
 * License:     (Licensed under the Synesis Software Standard Source License)
 *
 *              Copyright (C) 2002-2003, Synesis Software Pty Ltd.
 *
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/stlsoft
 *                          http://www.stlsoft.org/
 *
 *              email:      submissions@stlsoft.org  for submissions
 *                          admin@stlsoft.org        for other enquiries
 *
 *              Redistribution and use in source and binary forms, with or
 *              without modification, are permitted provided that the following
 *              conditions are met:
 *
 *              (i) Redistributions of source code must retain the above
 *              copyright notice and contact information, this list of
 *              conditions and the following disclaimer.
 *
 *              (ii) Any derived versions of this software (howsoever modified)
 *              remain the sole property of Synesis Software.
 *
 *              (iii) Any derived versions of this software (howsoever modified)
 *              remain subject to all these conditions.
 *
 *              (iv) Neither the name of Synesis Software nor the names of any
 *              subdivisions, employees or agents of Synesis Software, nor the
 *              names of any other contributors to this software may be used to
 *              endorse or promote products derived from this software without
 *              specific prior written permission.
 *
 *              This source code is provided by Synesis Software "as is" and any
 *              warranties, whether expressed or implied, including, but not
 *              limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software be liable for any direct, indirect,
 *              incidental, special, exemplary, or consequential damages
 *              (including, but not limited to, procurement of substitute goods
 *              or services; loss of use, data, or profits; or business
 *              interruption) however caused and on any theory of liability,
 *              whether in contract, strict liability, or tort (including
 *              negligence or otherwise) arising in any way out of the use of
 *              this software, even if advised of the possibility of such
 *              damage.
 *
 * ////////////////////////////////////////////////////////////////////////// */


#ifndef _STLSOFT_INCL_H_STLSOFT_EXPLICIT_INHERITANCE_VENEER
#define _STLSOFT_INCL_H_STLSOFT_EXPLICIT_INHERITANCE_VENEER

#ifndef __STLSOFT_DOCUMENTATION_SKIP_SECTION
#define _STLSOFT_VER_H_STLSOFT_EXPLICIT_INHERITANCE_VENEER_MAJOR    1
#define _STLSOFT_VER_H_STLSOFT_EXPLICIT_INHERITANCE_VENEER_MINOR    2
#define _STLSOFT_VER_H_STLSOFT_EXPLICIT_INHERITANCE_VENEER_REVISION 1
#define _STLSOFT_VER_H_STLSOFT_EXPLICIT_INHERITANCE_VENEER_EDIT     8
#endif /* !__STLSOFT_DOCUMENTATION_SKIP_SECTION */

/* /////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#ifndef _STLSOFT_INCL_H_STLSOFT
 #include "stlsoft.h"               // Include the STLSoft root header
#endif /* !_STLSOFT_INCL_H_STLSOFT */
#ifndef _STLSOFT_INCL_H_STLSOFT_CONSTRAINTS
 #include "stlsoft_constraints.h"   // is_veneer
#endif /* !_STLSOFT_INCL_H_STLSOFT_CONSTRAINTS */

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#ifndef _STLSOFT_NO_NAMESPACE
namespace stlsoft
{
#endif /* _STLSOFT_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////////
 * Classes
 */


// class explicit_inheritance_veneer

/// Applies a \c protected veneer over a type, which can be accessed only
/// through conversions to the veneered type
///
/// \param T The type to be subject to the veneer
template <ss_typename_param_k T>
struct explicit_inheritance_veneer
    : protected T
#ifdef __STLSOFT_CF_VENEER_SUPPORT
    , protected is_veneer<T, explicit_inheritance_veneer<T> >
#endif /* __STLSOFT_CF_VENEER_SUPPORT */
{
public:
    /// The base class type
    typedef T   base_type;

// Conversion
public:
    /// The conversion operator that provides access to the base type
    base_type const *operator &() const
    {
        return this;
    }

// Implementation
protected:
    /// Access a mutable reference to the base type
    base_type &base_type_value()
    {
        return *this;
    }
    /// Access a non-mutable reference to the base type
    base_type const &base_type_value() const
    {
        return *this;
    }
};

/* ////////////////////////////////////////////////////////////////////////// */

#ifndef _STLSOFT_NO_NAMESPACE
} // namespace stlsoft
#endif /* _STLSOFT_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* !_STLSOFT_INCL_H_STLSOFT_EXPLICIT_INHERITANCE_VENEER */

/* ////////////////////////////////////////////////////////////////////////// */
