/* /////////////////////////////////////////////////////////////////////////////
 * File:        stlsoft_fixed_array.h (originally MTArray.h, ::SynesisStl)
 *
 * Purpose:     Contains the array1D, array2D template classes.
 *
 * Created:     4th August 1998
 * Updated:     6th August 2003
 *
 * Author:      Matthew Wilson, Synesis Software Pty Ltd.
 *
 * License:     (Licensed under the Synesis Software Standard Source License)
 *
 *              Copyright (C) 2002-2003, Synesis Software Pty Ltd.
 *
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/stlsoft
 *                          http://www.stlsoft.org/
 *
 *              email:      submissions@stlsoft.org  for submissions
 *                          admin@stlsoft.org        for other enquiries
 *
 *              Redistribution and use in source and binary forms, with or
 *              without modification, are permitted provided that the following
 *              conditions are met:
 *
 *              (i) Redistributions of source code must retain the above
 *              copyright notice and contact information, this list of
 *              conditions and the following disclaimer.
 *
 *              (ii) Any derived versions of this software (howsoever modified)
 *              remain the sole property of Synesis Software.
 *
 *              (iii) Any derived versions of this software (howsoever modified)
 *              remain subject to all these conditions.
 *
 *              (iv) Neither the name of Synesis Software nor the names of any
 *              subdivisions, employees or agents of Synesis Software, nor the
 *              names of any other contributors to this software may be used to
 *              endorse or promote products derived from this software without
 *              specific prior written permission.
 *
 *              This source code is provided by Synesis Software "as is" and any
 *              warranties, whether expressed or implied, including, but not
 *              limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software be liable for any direct, indirect,
 *              incidental, special, exemplary, or consequential damages
 *              (including, but not limited to, procurement of substitute goods
 *              or services; loss of use, data, or profits; or business
 *              interruption) however caused and on any theory of liability,
 *              whether in contract, strict liability, or tort (including
 *              negligence or otherwise) arising in any way out of the use of
 *              this software, even if advised of the possibility of such
 *              damage.
 *
 * ////////////////////////////////////////////////////////////////////////// */


#ifndef _STLSOFT_INCL_H_STLSOFT_FIXED_ARRAY
#define _STLSOFT_INCL_H_STLSOFT_FIXED_ARRAY

#ifndef __STLSOFT_DOCUMENTATION_SKIP_SECTION
#define _STLSOFT_VER_H_STLSOFT_FIXED_ARRAY_MAJOR     1
#define _STLSOFT_VER_H_STLSOFT_FIXED_ARRAY_MINOR     7
#define _STLSOFT_VER_H_STLSOFT_FIXED_ARRAY_REVISION  6
#define _STLSOFT_VER_H_STLSOFT_FIXED_ARRAY_EDIT      115
#endif /* !__STLSOFT_DOCUMENTATION_SKIP_SECTION */

/* /////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#ifndef _STLSOFT_INCL_H_STLSOFT
 #include "stlsoft.h"                   // Include the STLSoft root header
#endif /* !_STLSOFT_INCL_H_STLSOFT */
#ifndef _STLSOFT_INCL_H_STLSOFT_NEW_ALLOCATOR
 #include "stlsoft_new_allocator.h"     // new_allocator
#endif /* !_STLSOFT_INCL_H_STLSOFT_NEW_ALLOCATOR */
#ifndef _STLSOFT_INCL_H_STLSOFT_ITERATOR
 #include "stlsoft_iterator.h"          // stlsoft::iterator, stlsoft::reverse_iterator
#endif /* _STLSOFT_INCL_H_STLSOFT_ITERATOR */
#ifndef _STLSOFT_INCL_H_STLSOFT_ARRAY_POLICIES
 #include "stlsoft_array_policies.h"    // stlsoft::do_construction
#endif /* !_STLSOFT_INCL_H_STLSOFT_ARRAY_POLICIES */
#if defined(__STLSOFT_COMPILER_IS_MSVC) && \
    defined(__STLSOFT_CF_STL_IS_STLPORT)
 #include <string>                      // std::string - sigh!
#endif /* __STLSOFT_COMPILER_IS_MSVC && __STLSOFT_CF_STL_IS_STLPORT */
#include <stdexcept>                    // std::out_of_range

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#ifndef _STLSOFT_NO_NAMESPACE
namespace stlsoft
{
#endif /* _STLSOFT_NO_NAMESPACE */

/* /////////////////////////////////////////////////////////////////////////////
 * Forward declarations
 */

#ifndef __STLSOFT_DOCUMENTATION_SKIP_SECTION

template<   ss_typename_param_k T
        ,   ss_typename_param_k A
        ,   ss_typename_param_k P
        >
class fixed_array_1d;

template<   ss_typename_param_k T
        ,   ss_typename_param_k A
        ,   ss_typename_param_k P
        >
class fixed_array_2d;

template<   ss_typename_param_k T
        ,   ss_typename_param_k A
        ,   ss_typename_param_k P
        >
class fixed_array_3d;

template<   ss_typename_param_k T
        ,   ss_typename_param_k A
        ,   ss_typename_param_k P
        >
class fixed_array_4d;

template<   ss_typename_param_k T
        ,   ss_typename_param_k A
        ,   ss_typename_param_k P
        >
class fixed_array_5d;

#endif /* !__STLSOFT_DOCUMENTATION_SKIP_SECTION */

/* /////////////////////////////////////////////////////////////////////////////
 * Classes
 */

// class fixed_array_1d
/// 1 dimensional fixed array
///
/// \param T The value type
/// \param A The allocator type
/// \param P The construction policy type
template<   ss_typename_param_k T
#ifdef __STLSOFT_CF_TEMPLATE_CLASS_DEFAULT_CLASS_ARGUMENT_SUPPORT
        ,   ss_typename_param_k A = new_allocator<T>
        ,   ss_typename_param_k P = do_construction<T>
#else
        ,   ss_typename_param_k A
        ,   ss_typename_param_k P
#endif /* __STLSOFT_CF_TEMPLATE_CLASS_DEFAULT_CLASS_ARGUMENT_SUPPORT */
        >
class fixed_array_1d
    : public A
{
public:
    typedef fixed_array_1d<T, A, P>                         class_type;
    typedef T                                               dimension_type;
    typedef A                                               allocator_type;
    typedef P                                               policy_type;

    typedef T                                               value_type;
    typedef value_type                                      &reference;
    typedef value_type const                                &const_reference;
    typedef value_type                                      *pointer;
    typedef value_type const                                *const_pointer;
    typedef ss_size_t                                       size_type;
    typedef ss_size_t                                       index_type;
    typedef ss_ptrdiff_t                                    difference_type;

    typedef
#if !defined(__STLSOFT_COMPILER_IS_BORLAND)
           ss_typename_type_k
#endif /* __STLSOFT_COMPILER_IS_BORLAND */
                       pointer_iterator <   value_type
                                        ,   pointer
                                        ,   reference
                                        >::iterator_type    iterator;
    typedef
#if !defined(__STLSOFT_COMPILER_IS_BORLAND)
         ss_typename_type_k
#endif /* __STLSOFT_COMPILER_IS_BORLAND */
                       pointer_iterator <   value_type const
                                        ,   const_pointer
                                        ,   const_reference
                                        >::iterator_type    const_iterator;

    typedef reverse_iterator_base       <   iterator
                                        ,   value_type
                                        ,   reference
                                        ,   pointer
                                        ,   difference_type
                                        >                   reverse_iterator;

    typedef const_reverse_iterator_base <   const_iterator
                                        ,   value_type const
                                        ,   const_reference
                                        ,   const_pointer
                                        ,   difference_type
                                        >                   const_reverse_iterator;

// Construction
protected:
                  fixed_array_1d(T *data, index_type d0);
public:
    ss_explicit_k fixed_array_1d(index_type d0);
                  fixed_array_1d(index_type d0, value_type const &t);
                 ~fixed_array_1d() stlsoft_throw_0();

// Access
public:
    reference               at(index_type i0);
    const_reference         at(index_type i0) const;

    reference               at_unchecked(index_type i0);
    const_reference         at_unchecked(index_type i0) const;

    reference               operator [](index_type i0);
    const_reference         operator [](index_type i0) const;

    reference               front();
    reference               back();
    const_reference         front() const;
    const_reference         back() const;

// State
public:
    index_type              dimension0() const;
    index_type              size() const;
    ss_bool_t               empty() const;
    static size_type        max_size();

// Iteration
public:
    iterator                begin();
    iterator                end();
    const_iterator          begin() const;
    const_iterator          end() const;

    reverse_iterator        rbegin();
    reverse_iterator        rend();
    const_reverse_iterator  rbegin() const;
    const_reverse_iterator  rend() const;

// Access
public:
    value_type const    *data() const;

// Implementation
private:
    index_type  _calc_index(index_type i0) const;
    void        _range_check(index_type i0) const stlsoft_throw_1(stlsoft_ns_qual_std(out_of_range) );

// Members
private:
    T * const           m_data;
    const index_type    m_d0;
    ss_bool_t           m_bProxy;

    friend class fixed_array_2d<T, A, P>;

// Not to be implemented
private:
    fixed_array_1d(class_type const &rhs);
    class_type const &operator =(class_type const &rhs);
};

// class fixed_array_2d
/// 2 dimensional fixed array
///
/// \param T The value type
/// \param A The allocator type
/// \param P The construction policy type
template<   ss_typename_param_k T
#ifdef __STLSOFT_CF_TEMPLATE_CLASS_DEFAULT_CLASS_ARGUMENT_SUPPORT
        ,   ss_typename_param_k A = new_allocator<T>
        ,   ss_typename_param_k P = do_construction<T>
#else
        ,   ss_typename_param_k A
        ,   ss_typename_param_k P
#endif /* __STLSOFT_CF_TEMPLATE_CLASS_DEFAULT_CLASS_ARGUMENT_SUPPORT */
        >
class fixed_array_2d
    : public A
{
public:
    typedef fixed_array_2d<T, A, P>                         class_type;
    typedef fixed_array_1d<T, A, P>                         dimension_type;
    typedef A                                               allocator_type;
    typedef P                                               policy_type;

    typedef T                                               value_type;
    typedef value_type                                      &reference;
    typedef value_type const                                &const_reference;
    typedef value_type                                      *pointer;
    typedef value_type const                                *const_pointer;
    typedef ss_size_t                                       size_type;
    typedef ss_size_t                                       index_type;
    typedef ss_ptrdiff_t                                    difference_type;

    typedef
#if !defined(__STLSOFT_COMPILER_IS_BORLAND)
           ss_typename_type_k
#endif /* __STLSOFT_COMPILER_IS_BORLAND */
                       pointer_iterator <   value_type
                                        ,   pointer
                                        ,   reference
                                        >::iterator_type    iterator;
    typedef
#if !defined(__STLSOFT_COMPILER_IS_BORLAND)
         ss_typename_type_k
#endif /* __STLSOFT_COMPILER_IS_BORLAND */
                       pointer_iterator <   value_type const
                                        ,   const_pointer
                                        ,   const_reference
                                        >::iterator_type    const_iterator;

    typedef reverse_iterator_base       <   iterator
                                        ,   value_type
                                        ,   reference
                                        ,   pointer
                                        ,   difference_type
                                        >                   reverse_iterator;

    typedef const_reverse_iterator_base <   const_iterator
                                        ,   value_type const
                                        ,   const_reference
                                        ,   const_pointer
                                        ,   difference_type
                                        >                   const_reverse_iterator;

// Construction
protected:
    fixed_array_2d(T *data, index_type d0, index_type d1);
public:
    fixed_array_2d(index_type d0, index_type d1);
    fixed_array_2d(index_type d0, index_type d1, value_type const &t);
    ~fixed_array_2d() stlsoft_throw_0();

// Access
public:
    reference               at(index_type i0, index_type i1);
    const_reference         at(index_type i0, index_type i1) const;

    reference               at_unchecked(index_type i0, index_type i1);
    const_reference         at_unchecked(index_type i0, index_type i1) const;

    dimension_type          at(index_type i0);
    const dimension_type    at(index_type i0) const;

    dimension_type          at_unchecked(index_type i0);
    const dimension_type    at_unchecked(index_type i0) const;

    dimension_type          operator [](index_type i0);
    const dimension_type    operator [](index_type i0) const;

    reference               front();
    reference               back();
    const_reference         front() const;
    const_reference         back() const;

// State
public:
    index_type              dimension0() const;
    index_type              dimension1() const;
    index_type              size() const;
    ss_bool_t               empty() const;
    static size_type        max_size();

// Iteration
public:
    iterator                begin();
    iterator                end();
    const_iterator          begin() const;
    const_iterator          end() const;

    reverse_iterator        rbegin();
    reverse_iterator        rend();
    const_reverse_iterator  rbegin() const;
    const_reverse_iterator  rend() const;

// Access
public:
    value_type const        *data() const;

// Implementation
private:
    index_type  _calc_index(index_type i0, index_type i1) const;
    void        _range_check(index_type i0, index_type i1) const stlsoft_throw_1(stlsoft_ns_qual_std(out_of_range) );
    void        _range_check(index_type i0) const stlsoft_throw_1(stlsoft_ns_qual_std(out_of_range) );

// Members
private:
    T * const           m_data;
    const index_type    m_d0;
    const index_type    m_d1;
    ss_bool_t           m_bProxy;

    friend class fixed_array_3d<T, A, P>;

// Not to be implemented
private:
    fixed_array_2d(class_type const &rhs);
    class_type const &operator =(class_type const &rhs);
};

// class fixed_array_3d
/// 3 dimensional fixed array
///
/// \param T The value type
/// \param A The allocator type
/// \param P The construction policy type
template<   ss_typename_param_k T
#ifdef __STLSOFT_CF_TEMPLATE_CLASS_DEFAULT_CLASS_ARGUMENT_SUPPORT
        ,   ss_typename_param_k A = new_allocator<T>
        ,   ss_typename_param_k P = do_construction<T>
#else
        ,   ss_typename_param_k A
        ,   ss_typename_param_k P
#endif /* __STLSOFT_CF_TEMPLATE_CLASS_DEFAULT_CLASS_ARGUMENT_SUPPORT */
        >
class fixed_array_3d
    : public A
{
public:
    typedef fixed_array_3d<T, A, P>                         class_type;
    typedef fixed_array_2d<T, A, P>                         dimension_type;
    typedef A                                               allocator_type;
    typedef P                                               policy_type;

    typedef T                                               value_type;
    typedef value_type                                      &reference;
    typedef value_type const                                &const_reference;
    typedef value_type                                      *pointer;
    typedef value_type const                                *const_pointer;
    typedef ss_size_t                                       size_type;
    typedef ss_size_t                                       index_type;
    typedef ss_ptrdiff_t                                    difference_type;

    typedef
#if !defined(__STLSOFT_COMPILER_IS_BORLAND)
           ss_typename_type_k
#endif /* __STLSOFT_COMPILER_IS_BORLAND */
                       pointer_iterator <   value_type
                                        ,   pointer
                                        ,   reference
                                        >::iterator_type    iterator;
    typedef
#if !defined(__STLSOFT_COMPILER_IS_BORLAND)
         ss_typename_type_k
#endif /* __STLSOFT_COMPILER_IS_BORLAND */
                       pointer_iterator <   value_type const
                                        ,   const_pointer
                                        ,   const_reference
                                        >::iterator_type    const_iterator;

    typedef reverse_iterator_base       <   iterator
                                        ,   value_type
                                        ,   reference
                                        ,   pointer
                                        ,   difference_type
                                        >                   reverse_iterator;

    typedef const_reverse_iterator_base <   const_iterator
                                        ,   value_type const
                                        ,   const_reference
                                        ,   const_pointer
                                        ,   difference_type
                                        >                   const_reverse_iterator;

// Construction
protected:
    fixed_array_3d(pointer data, index_type d0, index_type d1, index_type d2);

public:
    fixed_array_3d(index_type d0, index_type d1, index_type d2);
    fixed_array_3d(index_type d0, index_type d1, index_type d2, value_type const &t);
    ~fixed_array_3d() stlsoft_throw_0();

// Access
public:
    reference               at(index_type i0, index_type i1, index_type i2);
    const_reference         at(index_type i0, index_type i1, index_type i3) const;

    reference               at_unchecked(index_type i0, index_type i1, index_type i2);
    const_reference         at_unchecked(index_type i0, index_type i1, index_type i2) const;

    dimension_type          at(index_type i0);
    const dimension_type    at(index_type i0) const;

    dimension_type          at_unchecked(index_type i0);
    const dimension_type    at_unchecked(index_type i0) const;

    dimension_type          operator [](index_type i0);
    const dimension_type    operator [](index_type i0) const;

    reference               front();
    reference               back();
    const_reference         front() const;
    const_reference         back() const;

// State
public:
    index_type              dimension0() const;
    index_type              dimension1() const;
    index_type              dimension2() const;
    index_type              size() const;
    ss_bool_t               empty() const;
    static size_type        max_size();

// Iteration
public:
    iterator                begin();
    iterator                end();
    const_iterator          begin() const;
    const_iterator          end() const;

    reverse_iterator        rbegin();
    reverse_iterator        rend();
    const_reverse_iterator  rbegin() const;
    const_reverse_iterator  rend() const;

// Access
public:
    value_type const        *data() const;

// Implementation
private:
    index_type  _calc_index(index_type i0, index_type i1, index_type i2) const;
    void        _range_check(index_type i0, index_type i1, index_type i2) const stlsoft_throw_1(stlsoft_ns_qual_std(out_of_range) );
    void        _range_check(index_type i0) const stlsoft_throw_1(stlsoft_ns_qual_std(out_of_range) );

// Members
private:
    T * const           m_data;
    const index_type    m_d0;
    const index_type    m_d1;
    const index_type    m_d2;
    ss_bool_t           m_bProxy;

    friend class fixed_array_4d<T, A, P>;

// Not to be implemented
private:
    fixed_array_3d(class_type const &rhs);
    class_type const &operator =(class_type const &rhs);
};


// class fixed_array_4d
/// 4 dimensional fixed array
///
/// \param T The value type
/// \param A The allocator type
/// \param P The construction policy type
template<   ss_typename_param_k T
#ifdef __STLSOFT_CF_TEMPLATE_CLASS_DEFAULT_CLASS_ARGUMENT_SUPPORT
        ,   ss_typename_param_k A = new_allocator<T>
        ,   ss_typename_param_k P = do_construction<T>
#else
        ,   ss_typename_param_k A
        ,   ss_typename_param_k P
#endif /* __STLSOFT_CF_TEMPLATE_CLASS_DEFAULT_CLASS_ARGUMENT_SUPPORT */
        >
class fixed_array_4d
    : public A
{
public:
    typedef fixed_array_4d<T, A, P>                         class_type;
    typedef fixed_array_3d<T, A, P>                         dimension_type;
    typedef A                                               allocator_type;
    typedef P                                               policy_type;

    typedef T                                               value_type;
    typedef value_type                                      &reference;
    typedef value_type const                                &const_reference;
    typedef value_type                                      *pointer;
    typedef value_type const                                *const_pointer;
    typedef ss_size_t                                       size_type;
    typedef ss_size_t                                       index_type;
    typedef ss_ptrdiff_t                                    difference_type;

    typedef
#if !defined(__STLSOFT_COMPILER_IS_BORLAND)
           ss_typename_type_k
#endif /* __STLSOFT_COMPILER_IS_BORLAND */
                       pointer_iterator <   value_type
                                        ,   pointer
                                        ,   reference
                                        >::iterator_type    iterator;
    typedef
#if !defined(__STLSOFT_COMPILER_IS_BORLAND)
         ss_typename_type_k
#endif /* __STLSOFT_COMPILER_IS_BORLAND */
                       pointer_iterator <   value_type const
                                        ,   const_pointer
                                        ,   const_reference
                                        >::iterator_type    const_iterator;

    typedef reverse_iterator_base       <   iterator
                                        ,   value_type
                                        ,   reference
                                        ,   pointer
                                        ,   difference_type
                                        >                   reverse_iterator;

    typedef const_reverse_iterator_base <   const_iterator
                                        ,   value_type const
                                        ,   const_reference
                                        ,   const_pointer
                                        ,   difference_type
                                        >                   const_reverse_iterator;

// Construction
protected:
    fixed_array_4d(T *data, index_type d0, index_type d1, index_type d2, index_type d3);
public:
    fixed_array_4d(index_type d0, index_type d1, index_type d2, index_type d3);
    fixed_array_4d(index_type d0, index_type d1, index_type d2, index_type d3, value_type const &t);
    ~fixed_array_4d() stlsoft_throw_0();

// Access
public:
    reference               at(index_type i0, index_type i1, index_type i2, index_type i3);
    const_reference         at(index_type i0, index_type i1, index_type i2, index_type i3) const;

    reference               at_unchecked(index_type i0, index_type i1, index_type i2, index_type i3);
    const_reference         at_unchecked(index_type i0, index_type i1, index_type i2, index_type i3) const;

    dimension_type          at(index_type i0);
    const dimension_type    at(index_type i0) const;

    dimension_type          at_unchecked(index_type i0);
    const dimension_type    at_unchecked(index_type i0) const;

    dimension_type          operator [](index_type i0);
    const dimension_type    operator [](index_type i0) const;

    reference               front();
    reference               back();
    const_reference         front() const;
    const_reference         back() const;

// State
public:
    index_type              dimension0() const;
    index_type              dimension1() const;
    index_type              dimension2() const;
    index_type              dimension3() const;
    index_type              size() const;
    ss_bool_t               empty() const;
    static size_type        max_size();

// Iteration
public:
    iterator                begin();
    iterator                end();
    const_iterator          begin() const;
    const_iterator          end() const;

    reverse_iterator        rbegin();
    reverse_iterator        rend();
    const_reverse_iterator  rbegin() const;
    const_reverse_iterator  rend() const;

// Access
public:
    value_type const        *data() const;

// Implementation
protected:
    index_type  _calc_index(index_type i0, index_type i1, index_type i2, index_type i3) const;
    void        _range_check(index_type i0, index_type i1, index_type i2, index_type i3) const stlsoft_throw_1(stlsoft_ns_qual_std(out_of_range) );
    void        _range_check(index_type i0) const stlsoft_throw_1(stlsoft_ns_qual_std(out_of_range) );

// Members
protected:
    T * const           m_data;
    const index_type    m_d0;
    const index_type    m_d1;
    const index_type    m_d2;
    const index_type    m_d3;
    ss_bool_t           m_bProxy;

    friend class fixed_array_5d<T, A, P>;

// Not to be implemented
private:
    fixed_array_4d(class_type const &rhs);
    class_type const &operator =(class_type const &rhs);
};

/* /////////////////////////////////////////////////////////////////////////////
 * Implementation
 */

#ifndef __STLSOFT_DOCUMENTATION_SKIP_SECTION

// fixed_array_1d

template <ss_typename_param_k T, ss_typename_param_k A, ss_typename_param_k P>
inline ss_typename_type_k fixed_array_1d<T, A, P>::index_type fixed_array_1d<T, A, P>::_calc_index(ss_typename_type_k fixed_array_1d<T, A, P>::index_type i0) const
{
    return i0;
}

template <ss_typename_param_k T, ss_typename_param_k A, ss_typename_param_k P>
inline void fixed_array_1d<T, A, P>::_range_check(ss_typename_type_k fixed_array_1d<T, A, P>::index_type i0) const stlsoft_throw_1(stlsoft_ns_qual_std(out_of_range) )
{
#ifdef __STLSOFT_CF_EXCEPTION_SUPPORT
    if(!(i0 < m_d0))
    {
        stlsoft_ns_qual_std(out_of_range)   x("fixed array index out of range");

#ifdef __STLSOFT_COMPILER_IS_MSVC
        if(1) // This is needed to avoid Visual C++ warning 4702: 'unreachable code'
#endif /* __STLSOFT_COMPILER_IS_MSVC */
        {
            throw x;
        }
    }
#else
    stlsoft_message_assert("fixed array index out of range", i0 < m_d0);
#endif /* __STLSOFT_CF_EXCEPTION_SUPPORT */
}

template <ss_typename_param_k T, ss_typename_param_k A, ss_typename_param_k P>
inline fixed_array_1d<T, A, P>::fixed_array_1d(T *data, ss_typename_type_k fixed_array_1d<T, A, P>::index_type d0)
    : m_data(data)
    , m_d0(d0)
    , m_bProxy(true)
{}

template <ss_typename_param_k T, ss_typename_param_k A, ss_typename_param_k P>
inline fixed_array_1d<T, A, P>::fixed_array_1d(ss_typename_type_k fixed_array_1d<T, A, P>::index_type d0)
    : m_data(allocator_type::allocate(d0, NULL))
    , m_d0(d0)
    , m_bProxy(false)
{
//  inplace_range_construct<policy_type::value, allocator_type>(begin(), end);

    if(policy_type::value)
    {
        for(index_type i0 = 0; i0 < m_d0; ++i0)
        {
            allocator_type::construct(m_data + _calc_index(i0));
        }
    }
}

template <ss_typename_param_k T, ss_typename_param_k A, ss_typename_param_k P>
inline fixed_array_1d<T, A, P>::fixed_array_1d(index_type d0, value_type const &t)
    : m_data(allocator_type::allocate(d0, NULL))
    , m_d0(d0)
    , m_bProxy(false)
{
    if(policy_type::value)
    {
        for(index_type i0 = 0; i0 < m_d0; ++i0)
        {
            allocator_type::construct(m_data + _calc_index(i0), t);
        }
    }
}

template <ss_typename_param_k T, ss_typename_param_k A, ss_typename_param_k P>
inline fixed_array_1d<T, A, P>::~fixed_array_1d() stlsoft_throw_0()
{
    if(!m_bProxy)
    {
        if(policy_type::value)
        {
            for(index_type i0 = 0; i0 < m_d0; ++i0)
            {
                allocator_type::destroy(m_data + _calc_index(i0));
            }
        }

        allocator_type::deallocate(m_data, m_d0);
    }
}

template <ss_typename_param_k T, ss_typename_param_k A, ss_typename_param_k P>
inline ss_typename_type_k fixed_array_1d<T, A, P>::reference fixed_array_1d<T, A, P>::at(ss_typename_type_k fixed_array_1d<T, A, P>::index_type i0)
{
    _range_check(i0);

    return m_data[i0];
}

template <ss_typename_param_k T, ss_typename_param_k A, ss_typename_param_k P>
inline ss_typename_type_k fixed_array_1d<T, A, P>::const_reference fixed_array_1d<T, A, P>::at(ss_typename_type_k fixed_array_1d<T, A, P>::index_type i0) const
{
    _range_check(i0);

    return m_data[i0];
}

template <ss_typename_param_k T, ss_typename_param_k A, ss_typename_param_k P>
inline ss_typename_type_k fixed_array_1d<T, A, P>::reference fixed_array_1d<T, A, P>::at_unchecked(ss_typename_type_k fixed_array_1d<T, A, P>::index_type i0)
{
    stlsoft_message_assert("fixed array index out of range", i0 < m_d0);

    return m_data[i0];
}

template <ss_typename_param_k T, ss_typename_param_k A, ss_typename_param_k P>
inline ss_typename_type_k fixed_array_1d<T, A, P>::const_reference fixed_array_1d<T, A, P>::at_unchecked(ss_typename_type_k fixed_array_1d<T, A, P>::index_type i0) const
{
    stlsoft_message_assert("fixed array index out of range", i0 < m_d0);

    return m_data[i0];
}

template <ss_typename_param_k T, ss_typename_param_k A, ss_typename_param_k P>
inline ss_typename_type_k fixed_array_1d<T, A, P>::reference fixed_array_1d<T, A, P>::operator [](ss_typename_type_k fixed_array_1d<T, A, P>::index_type i0)
{
    stlsoft_message_assert("fixed array index out of range", i0 < m_d0);

    return m_data[i0];
}

template <ss_typename_param_k T, ss_typename_param_k A, ss_typename_param_k P>
inline ss_typename_type_k fixed_array_1d<T, A, P>::const_reference fixed_array_1d<T, A, P>::operator [](ss_typename_type_k fixed_array_1d<T, A, P>::index_type i0) const
{
    stlsoft_message_assert("fixed array index out of range", i0 < m_d0);

    return m_data[i0];
}

template <ss_typename_param_k T, ss_typename_param_k A, ss_typename_param_k P>
inline ss_typename_type_k fixed_array_1d<T, A, P>::reference fixed_array_1d<T, A, P>::front()
{
    return at(0);
}

template <ss_typename_param_k T, ss_typename_param_k A, ss_typename_param_k P>
inline ss_typename_type_k fixed_array_1d<T, A, P>::reference fixed_array_1d<T, A, P>::back()
{
    return at(m_d0 - 1);
}

template <ss_typename_param_k T, ss_typename_param_k A, ss_typename_param_k P>
inline ss_typename_type_k fixed_array_1d<T, A, P>::const_reference fixed_array_1d<T, A, P>::front() const
{
    return at(0);
}

template <ss_typename_param_k T, ss_typename_param_k A, ss_typename_param_k P>
inline ss_typename_type_k fixed_array_1d<T, A, P>::const_reference fixed_array_1d<T, A, P>::back() const
{
    return at(m_d0 - 1);
}

template <ss_typename_param_k T, ss_typename_param_k A, ss_typename_param_k P>
inline ss_typename_type_k fixed_array_1d<T, A, P>::index_type fixed_array_1d<T, A, P>::dimension0() const
{
    return m_d0;
}

template <ss_typename_param_k T, ss_typename_param_k A, ss_typename_param_k P>
inline ss_typename_type_k fixed_array_1d<T, A, P>::index_type fixed_array_1d<T, A, P>::size() const
{
    return m_d0;
}

template <ss_typename_param_k T, ss_typename_param_k A, ss_typename_param_k P>
inline ss_bool_t fixed_array_1d<T, A, P>::empty() const
{
    return false;
}

template <ss_typename_param_k T, ss_typename_param_k A, ss_typename_param_k P>
inline /* static */ ss_typename_type_k fixed_array_1d<T, A, P>::size_type fixed_array_1d<T, A, P>::max_size()
{
    return static_cast<size_type>(-1) / sizeof(T);
}

template <ss_typename_param_k T, ss_typename_param_k A, ss_typename_param_k P>
inline ss_typename_type_k fixed_array_1d<T, A, P>::iterator fixed_array_1d<T, A, P>::begin()
{
    return m_data;
}

template <ss_typename_param_k T, ss_typename_param_k A, ss_typename_param_k P>
inline ss_typename_type_k fixed_array_1d<T, A, P>::iterator fixed_array_1d<T, A, P>::end()
{
    return m_data + size();
}

template <ss_typename_param_k T, ss_typename_param_k A, ss_typename_param_k P>
inline ss_typename_type_k fixed_array_1d<T, A, P>::const_iterator fixed_array_1d<T, A, P>::begin() const
{
    return m_data;
}

template <ss_typename_param_k T, ss_typename_param_k A, ss_typename_param_k P>
inline ss_typename_type_k fixed_array_1d<T, A, P>::const_iterator fixed_array_1d<T, A, P>::end() const
{
    return m_data + size();
}

template <ss_typename_param_k T, ss_typename_param_k A, ss_typename_param_k P>
inline ss_typename_type_k fixed_array_1d<T, A, P>::reverse_iterator fixed_array_1d<T, A, P>::rbegin()
{
    return reverse_iterator(end());
}

template <ss_typename_param_k T, ss_typename_param_k A, ss_typename_param_k P>
inline ss_typename_type_k fixed_array_1d<T, A, P>::reverse_iterator fixed_array_1d<T, A, P>::rend()
{
    return reverse_iterator(begin());
}

template <ss_typename_param_k T, ss_typename_param_k A, ss_typename_param_k P>
inline ss_typename_type_k fixed_array_1d<T, A, P>::const_reverse_iterator fixed_array_1d<T, A, P>::rbegin() const
{
    return const_reverse_iterator(end());
}

template <ss_typename_param_k T, ss_typename_param_k A, ss_typename_param_k P>
inline ss_typename_type_k fixed_array_1d<T, A, P>::const_reverse_iterator fixed_array_1d<T, A, P>::rend() const
{
    return const_reverse_iterator(begin());
}

template <ss_typename_param_k T, ss_typename_param_k A, ss_typename_param_k P>
inline ss_typename_type_k fixed_array_1d<T, A, P>::value_type const *fixed_array_1d<T, A, P>::data() const
{
    return m_data;
}


// fixed_array_2d

template <ss_typename_param_k T, ss_typename_param_k A, ss_typename_param_k P>
inline ss_typename_type_k fixed_array_2d<T, A, P>::index_type fixed_array_2d<T, A, P>::_calc_index(ss_typename_type_k fixed_array_2d<T, A, P>::index_type i0, ss_typename_type_k fixed_array_2d<T, A, P>::index_type i1) const
{
    return (i0 * m_d1) + i1;
}

template <ss_typename_param_k T, ss_typename_param_k A, ss_typename_param_k P>
inline void fixed_array_2d<T, A, P>::_range_check(ss_typename_type_k fixed_array_2d<T, A, P>::index_type i0, ss_typename_type_k fixed_array_2d<T, A, P>::index_type i1) const stlsoft_throw_1(stlsoft_ns_qual_std(out_of_range) )
{
#ifdef __STLSOFT_CF_EXCEPTION_SUPPORT
    if( !(i0 < m_d0) ||
        !(i1 < m_d1))
    {
        stlsoft_ns_qual_std(out_of_range)   x("fixed array index out of range");

#ifdef __STLSOFT_COMPILER_IS_MSVC
        if(1) // This is needed to avoid Visual C++ warning 4702: 'unreachable code'
#endif /* __STLSOFT_COMPILER_IS_MSVC */
        {
            throw x;
        }
    }
#else
    stlsoft_message_assert("fixed array index out of range", i0 < m_d0 && i1 < m_d1);
#endif /* __STLSOFT_CF_EXCEPTION_SUPPORT */
}

template <ss_typename_param_k T, ss_typename_param_k A, ss_typename_param_k P>
inline void fixed_array_2d<T, A, P>::_range_check(ss_typename_type_k fixed_array_2d<T, A, P>::index_type i0) const stlsoft_throw_1(stlsoft_ns_qual_std(out_of_range) )
{
#ifdef __STLSOFT_CF_EXCEPTION_SUPPORT
    if(!(i0 < m_d0))
    {
        stlsoft_ns_qual_std(out_of_range)   x("fixed array index out of range");

#ifdef __STLSOFT_COMPILER_IS_MSVC
        if(1) // This is needed to avoid Visual C++ warning 4702: 'unreachable code'
#endif /* __STLSOFT_COMPILER_IS_MSVC */
        {
            throw x;
        }
    }
#else
    stlsoft_message_assert("fixed array index out of range", i0 < m_d0);
#endif /* __STLSOFT_CF_EXCEPTION_SUPPORT */
}

template <ss_typename_param_k T, ss_typename_param_k A, ss_typename_param_k P>
inline fixed_array_2d<T, A, P>::fixed_array_2d(T *data, ss_typename_type_k fixed_array_2d<T, A, P>::index_type d0, ss_typename_type_k fixed_array_2d<T, A, P>::index_type d1)
    : m_data(data)
    , m_d0(d0)
    , m_d1(d1)
    , m_bProxy(true)
{}

template <ss_typename_param_k T, ss_typename_param_k A, ss_typename_param_k P>
inline fixed_array_2d<T, A, P>::fixed_array_2d(ss_typename_type_k fixed_array_2d<T, A, P>::index_type d0, ss_typename_type_k fixed_array_2d<T, A, P>::index_type d1)
    : m_data(allocator_type::allocate(d0 * d1, NULL))
    , m_d0(d0)
    , m_d1(d1)
    , m_bProxy(false)
{
    if(policy_type::value)
    {
        for(index_type i0 = 0; i0 < m_d0; ++i0)
        {
            for(index_type i1 = 0; i1 < m_d1; ++i1)
            {
                allocator_type::construct(m_data + _calc_index(i0, i1));
            }
        }
    }
}

template <ss_typename_param_k T, ss_typename_param_k A, ss_typename_param_k P>
inline fixed_array_2d<T, A, P>::fixed_array_2d(ss_typename_type_k fixed_array_2d<T, A, P>::index_type d0, ss_typename_type_k fixed_array_2d<T, A, P>::index_type d1, value_type const &t)
    : m_data(allocator_type::allocate(d0 * d1, NULL))
    , m_d0(d0)
    , m_d1(d1)
    , m_bProxy(false)
{
    if(policy_type::value)
    {
        for(index_type i0 = 0; i0 < m_d0; ++i0)
        {
            for(index_type i1 = 0; i1 < m_d1; ++i1)
            {
                allocator_type::construct(m_data + _calc_index(i0, i1), t);
            }
        }
    }
}

template <ss_typename_param_k T, ss_typename_param_k A, ss_typename_param_k P>
inline fixed_array_2d<T, A, P>::~fixed_array_2d() stlsoft_throw_0()
{
    if(!m_bProxy)
    {
        if(policy_type::value)
        {
            for(index_type i0 = 0; i0 < m_d0; ++i0)
            {
                for(index_type i1 = 0; i1 < m_d1; ++i1)
                {
                    allocator_type::destroy(m_data + _calc_index(i0, i1));
                }
            }
        }

        allocator_type::deallocate(m_data, m_d0 * m_d1);
    }
}

template <ss_typename_param_k T, ss_typename_param_k A, ss_typename_param_k P>
inline ss_typename_type_k fixed_array_2d<T, A, P>::reference fixed_array_2d<T, A, P>::at(ss_typename_type_k fixed_array_2d<T, A, P>::index_type i0, ss_typename_type_k fixed_array_2d<T, A, P>::index_type i1)
{
    _range_check(i0, i1);

    return *(m_data + _calc_index(i0, i1));
}

template <ss_typename_param_k T, ss_typename_param_k A, ss_typename_param_k P>
inline ss_typename_type_k fixed_array_2d<T, A, P>::const_reference fixed_array_2d<T, A, P>::at(ss_typename_type_k fixed_array_2d<T, A, P>::index_type i0, ss_typename_type_k fixed_array_2d<T, A, P>::index_type i1) const
{
    _range_check(i0, i1);

    return *(m_data + _calc_index(i0, i1));
}

template <ss_typename_param_k T, ss_typename_param_k A, ss_typename_param_k P>
inline ss_typename_type_k fixed_array_2d<T, A, P>::reference fixed_array_2d<T, A, P>::at_unchecked(ss_typename_type_k fixed_array_2d<T, A, P>::index_type i0, ss_typename_type_k fixed_array_2d<T, A, P>::index_type i1)
{
    stlsoft_message_assert("fixed array index out of range", i0 < m_d0 && i1 < m_d1);

    return *(m_data + _calc_index(i0, i1));
}

template <ss_typename_param_k T, ss_typename_param_k A, ss_typename_param_k P>
inline ss_typename_type_k fixed_array_2d<T, A, P>::const_reference fixed_array_2d<T, A, P>::at_unchecked(ss_typename_type_k fixed_array_2d<T, A, P>::index_type i0, ss_typename_type_k fixed_array_2d<T, A, P>::index_type i1) const
{
    stlsoft_message_assert("fixed array index out of range", i0 < m_d0 && i1 < m_d1);

    return *(m_data + _calc_index(i0, i1));
}

template <ss_typename_param_k T, ss_typename_param_k A, ss_typename_param_k P>
inline ss_typename_type_k fixed_array_2d<T, A, P>::dimension_type fixed_array_2d<T, A, P>::at(ss_typename_type_k fixed_array_2d<T, A, P>::index_type i0)
{
    _range_check(i0);

    return dimension_type(m_data + i0 * m_d1, m_d1);
}

template <ss_typename_param_k T, ss_typename_param_k A, ss_typename_param_k P>
inline ss_typename_type_k fixed_array_2d<T, A, P>::dimension_type const fixed_array_2d<T, A, P>::at(ss_typename_type_k fixed_array_2d<T, A, P>::index_type i0) const
{
    _range_check(i0);

    return dimension_type(m_data + i0 * m_d1, m_d1);
}

template <ss_typename_param_k T, ss_typename_param_k A, ss_typename_param_k P>
inline ss_typename_type_k fixed_array_2d<T, A, P>::dimension_type fixed_array_2d<T, A, P>::at_unchecked(ss_typename_type_k fixed_array_2d<T, A, P>::index_type i0)
{
    stlsoft_message_assert("fixed array index out of range", i0 < m_d0);

    return dimension_type(m_data + i0 * m_d1, m_d1);
}

template <ss_typename_param_k T, ss_typename_param_k A, ss_typename_param_k P>
inline ss_typename_type_k fixed_array_2d<T, A, P>::dimension_type const fixed_array_2d<T, A, P>::at_unchecked(ss_typename_type_k fixed_array_2d<T, A, P>::index_type i0) const
{
    stlsoft_message_assert("fixed array index out of range", i0 < m_d0);

    return dimension_type(m_data + i0 * m_d1, m_d1);
}

template <ss_typename_param_k T, ss_typename_param_k A, ss_typename_param_k P>
inline ss_typename_type_k fixed_array_2d<T, A, P>::dimension_type fixed_array_2d<T, A, P>::operator [](ss_typename_type_k fixed_array_2d<T, A, P>::index_type i0)
{
    stlsoft_message_assert("fixed array index out of range", i0 < m_d0);

    return dimension_type(m_data + i0 * m_d1, m_d1);
}

template <ss_typename_param_k T, ss_typename_param_k A, ss_typename_param_k P>
inline ss_typename_type_k fixed_array_2d<T, A, P>::dimension_type const fixed_array_2d<T, A, P>::operator [](ss_typename_type_k fixed_array_2d<T, A, P>::index_type i0) const
{
    stlsoft_message_assert("fixed array index out of range", i0 < m_d0);

    return dimension_type(m_data + i0 * m_d1, m_d1);
}

template <ss_typename_param_k T, ss_typename_param_k A, ss_typename_param_k P>
inline ss_typename_type_k fixed_array_2d<T, A, P>::reference fixed_array_2d<T, A, P>::front()
{
    return at(0, 0);
}

template <ss_typename_param_k T, ss_typename_param_k A, ss_typename_param_k P>
inline ss_typename_type_k fixed_array_2d<T, A, P>::reference fixed_array_2d<T, A, P>::back()
{
    return at(m_d0 - 1, m_d1 - 1);
}

template <ss_typename_param_k T, ss_typename_param_k A, ss_typename_param_k P>
inline ss_typename_type_k fixed_array_2d<T, A, P>::const_reference fixed_array_2d<T, A, P>::front() const
{
    return at(0, 0);
}

template <ss_typename_param_k T, ss_typename_param_k A, ss_typename_param_k P>
inline ss_typename_type_k fixed_array_2d<T, A, P>::const_reference fixed_array_2d<T, A, P>::back() const
{
    return at(m_d0 - 1, m_d1 - 1);
}

template <ss_typename_param_k T, ss_typename_param_k A, ss_typename_param_k P>
inline ss_typename_type_k fixed_array_2d<T, A, P>::index_type fixed_array_2d<T, A, P>::dimension0() const
{
    return m_d0;
}

template <ss_typename_param_k T, ss_typename_param_k A, ss_typename_param_k P>
inline ss_typename_type_k fixed_array_2d<T, A, P>::index_type fixed_array_2d<T, A, P>::dimension1() const
{
    return m_d1;
}

template <ss_typename_param_k T, ss_typename_param_k A, ss_typename_param_k P>
inline ss_typename_type_k fixed_array_2d<T, A, P>::index_type fixed_array_2d<T, A, P>::size() const
{
    return m_d0 * m_d1;
}

template <ss_typename_param_k T, ss_typename_param_k A, ss_typename_param_k P>
inline ss_bool_t fixed_array_2d<T, A, P>::empty() const
{
    return false;
}

template <ss_typename_param_k T, ss_typename_param_k A, ss_typename_param_k P>
inline /* static */ ss_typename_type_k fixed_array_2d<T, A, P>::size_type fixed_array_2d<T, A, P>::max_size()
{
    return static_cast<size_type>(-1) / sizeof(T);
}

template <ss_typename_param_k T, ss_typename_param_k A, ss_typename_param_k P>
inline ss_typename_type_k fixed_array_2d<T, A, P>::iterator fixed_array_2d<T, A, P>::begin()
{
    return m_data;
}

template <ss_typename_param_k T, ss_typename_param_k A, ss_typename_param_k P>
inline ss_typename_type_k fixed_array_2d<T, A, P>::iterator fixed_array_2d<T, A, P>::end()
{
    return m_data + size();
}

template <ss_typename_param_k T, ss_typename_param_k A, ss_typename_param_k P>
inline ss_typename_type_k fixed_array_2d<T, A, P>::const_iterator fixed_array_2d<T, A, P>::begin() const
{
    return m_data;
}

template <ss_typename_param_k T, ss_typename_param_k A, ss_typename_param_k P>
inline ss_typename_type_k fixed_array_2d<T, A, P>::const_iterator fixed_array_2d<T, A, P>::end() const
{
    return m_data + size();
}

template <ss_typename_param_k T, ss_typename_param_k A, ss_typename_param_k P>
inline ss_typename_type_k fixed_array_2d<T, A, P>::reverse_iterator fixed_array_2d<T, A, P>::rbegin()
{
    return reverse_iterator(end());
}

template <ss_typename_param_k T, ss_typename_param_k A, ss_typename_param_k P>
inline ss_typename_type_k fixed_array_2d<T, A, P>::reverse_iterator fixed_array_2d<T, A, P>::rend()
{
    return reverse_iterator(begin());
}

template <ss_typename_param_k T, ss_typename_param_k A, ss_typename_param_k P>
inline ss_typename_type_k fixed_array_2d<T, A, P>::const_reverse_iterator fixed_array_2d<T, A, P>::rbegin() const
{
    return const_reverse_iterator(end());
}

template <ss_typename_param_k T, ss_typename_param_k A, ss_typename_param_k P>
inline ss_typename_type_k fixed_array_2d<T, A, P>::const_reverse_iterator fixed_array_2d<T, A, P>::rend() const
{
    return const_reverse_iterator(begin());
}

template <ss_typename_param_k T, ss_typename_param_k A, ss_typename_param_k P>
inline ss_typename_type_k fixed_array_2d<T, A, P>::value_type const *fixed_array_2d<T, A, P>::data() const
{
    return m_data;
}

// fixed_array_3d

template <ss_typename_param_k T, ss_typename_param_k A, ss_typename_param_k P>
inline ss_typename_type_k fixed_array_3d<T, A, P>::index_type fixed_array_3d<T, A, P>::_calc_index(ss_typename_type_k fixed_array_3d<T, A, P>::index_type i0, ss_typename_type_k fixed_array_3d<T, A, P>::index_type i1, ss_typename_type_k fixed_array_3d<T, A, P>::index_type i2) const
{
    return ((i0 * m_d1) + i1) * m_d2 + i2;
}

template <ss_typename_param_k T, ss_typename_param_k A, ss_typename_param_k P>
inline void fixed_array_3d<T, A, P>::_range_check(ss_typename_type_k fixed_array_3d<T, A, P>::index_type i0, ss_typename_type_k fixed_array_3d<T, A, P>::index_type i1, ss_typename_type_k fixed_array_3d<T, A, P>::index_type i2) const stlsoft_throw_1(stlsoft_ns_qual_std(out_of_range) )
{
#ifdef __STLSOFT_CF_EXCEPTION_SUPPORT
    if( !(i0 < m_d0) ||
        !(i1 < m_d1) ||
        !(i2 < m_d2))
    {
        stlsoft_ns_qual_std(out_of_range)   x("fixed array index out of range");

#ifdef __STLSOFT_COMPILER_IS_MSVC
        if(1) // This is needed to avoid Visual C++ warning 4702: 'unreachable code'
#endif /* __STLSOFT_COMPILER_IS_MSVC */
        {
            throw x;
        }
    }
#else
    stlsoft_message_assert("fixed array index out of range", i0 < m_d0 && i1 < m_d1 && i2 < m_d2);
#endif /* __STLSOFT_CF_EXCEPTION_SUPPORT */
}

template <ss_typename_param_k T, ss_typename_param_k A, ss_typename_param_k P>
inline void fixed_array_3d<T, A, P>::_range_check(ss_typename_type_k fixed_array_3d<T, A, P>::index_type i0) const stlsoft_throw_1(stlsoft_ns_qual_std(out_of_range) )
{
#ifdef __STLSOFT_CF_EXCEPTION_SUPPORT
    if(!(i0 < m_d0))
    {
        stlsoft_ns_qual_std(out_of_range)   x("fixed array index out of range");

#ifdef __STLSOFT_COMPILER_IS_MSVC
        if(1) // This is needed to avoid Visual C++ warning 4702: 'unreachable code'
#endif /* __STLSOFT_COMPILER_IS_MSVC */
        {
            throw x;
        }
    }
#else
    stlsoft_message_assert("fixed array index out of range", i0 < m_d0);
#endif /* __STLSOFT_CF_EXCEPTION_SUPPORT */
}

template <ss_typename_param_k T, ss_typename_param_k A, ss_typename_param_k P>
inline fixed_array_3d<T, A, P>::fixed_array_3d(ss_typename_type_k fixed_array_3d<T, A, P>::pointer data, ss_typename_type_k fixed_array_3d<T, A, P>::index_type d0, ss_typename_type_k fixed_array_3d<T, A, P>::index_type d1, ss_typename_type_k fixed_array_3d<T, A, P>::index_type d2)
    : m_data(data)
    , m_d0(d0)
    , m_d1(d1)
    , m_d2(d2)
    , m_bProxy(true)
{}

template <ss_typename_param_k T, ss_typename_param_k A, ss_typename_param_k P>
inline fixed_array_3d<T, A, P>::fixed_array_3d(ss_typename_type_k fixed_array_3d<T, A, P>::index_type d0, ss_typename_type_k fixed_array_3d<T, A, P>::index_type d1, ss_typename_type_k fixed_array_3d<T, A, P>::index_type d2)
    : m_data(allocator_type::allocate(d0 * d1 * d2, NULL))
    , m_d0(d0)
    , m_d1(d1)
    , m_d2(d2)
    , m_bProxy(false)
{
    if(policy_type::value)
    {
        for(index_type i0 = 0; i0 < m_d0; ++i0)
        {
            for(index_type i1 = 0; i1 < m_d1; ++i1)
            {
                for(index_type i2 = 0; i2 < m_d2; ++i2)
                {
                    allocator_type::construct(m_data + _calc_index(i0, i1, i2));
                }
            }
        }
    }
}

template <ss_typename_param_k T, ss_typename_param_k A, ss_typename_param_k P>
inline fixed_array_3d<T, A, P>::fixed_array_3d(ss_typename_type_k fixed_array_3d<T, A, P>::index_type d0, ss_typename_type_k fixed_array_3d<T, A, P>::index_type d1, ss_typename_type_k fixed_array_3d<T, A, P>::index_type d2, ss_typename_type_k fixed_array_3d<T, A, P>::value_type const &t)
    : m_data(allocator_type::allocate(d0 * d1 * d2, NULL))
    , m_d0(d0)
    , m_d1(d1)
    , m_d2(d2)
    , m_bProxy(false)
{
    if(policy_type::value)
    {
        for(index_type i0 = 0; i0 < m_d0; ++i0)
        {
            for(index_type i1 = 0; i1 < m_d1; ++i1)
            {
                for(index_type i2 = 0; i2 < m_d2; ++i2)
                {
                    allocator_type::construct(m_data + _calc_index(i0, i1, i2), t);
                }
            }
        }
    }
}

template <ss_typename_param_k T, ss_typename_param_k A, ss_typename_param_k P>
inline fixed_array_3d<T, A, P>::~fixed_array_3d() stlsoft_throw_0()
{
    if(!m_bProxy)
    {
        if(policy_type::value)
        {
            for(index_type i0 = 0; i0 < m_d0; ++i0)
            {
                for(index_type i1 = 0; i1 < m_d1; ++i1)
                {
                    for(index_type i2 = 0; i2 < m_d2; ++i2)
                    {
                        allocator_type::destroy(m_data + _calc_index(i0, i1, i2));
                    }
                }
            }
        }

        allocator_type::deallocate(m_data, size());
    }
}

template <ss_typename_param_k T, ss_typename_param_k A, ss_typename_param_k P>
inline ss_typename_type_k fixed_array_3d<T, A, P>::reference fixed_array_3d<T, A, P>::at(ss_typename_type_k fixed_array_3d<T, A, P>::index_type i0, ss_typename_type_k fixed_array_3d<T, A, P>::index_type i1, ss_typename_type_k fixed_array_3d<T, A, P>::index_type i2)
{
    _range_check(i0, i1, i2);

    return *(m_data + _calc_index(i0, i1, i2));
}

template <ss_typename_param_k T, ss_typename_param_k A, ss_typename_param_k P>
inline ss_typename_type_k fixed_array_3d<T, A, P>::const_reference fixed_array_3d<T, A, P>::at(ss_typename_type_k fixed_array_3d<T, A, P>::index_type i0, ss_typename_type_k fixed_array_3d<T, A, P>::index_type i1, ss_typename_type_k fixed_array_3d<T, A, P>::index_type i2) const
{
    _range_check(i0, i1, i2);

    return *(m_data + _calc_index(i0, i1, i2));
}

template <ss_typename_param_k T, ss_typename_param_k A, ss_typename_param_k P>
inline ss_typename_type_k fixed_array_3d<T, A, P>::reference fixed_array_3d<T, A, P>::at_unchecked(ss_typename_type_k fixed_array_3d<T, A, P>::index_type i0, ss_typename_type_k fixed_array_3d<T, A, P>::index_type i1, ss_typename_type_k fixed_array_3d<T, A, P>::index_type i2)
{
    stlsoft_message_assert("fixed array index out of range", i0 < m_d0 && i1 < m_d1 && i2 < m_d2);

    return *(m_data + _calc_index(i0, i1, i2));
}

template <ss_typename_param_k T, ss_typename_param_k A, ss_typename_param_k P>
inline ss_typename_type_k fixed_array_3d<T, A, P>::const_reference fixed_array_3d<T, A, P>::at_unchecked(ss_typename_type_k fixed_array_3d<T, A, P>::index_type i0, ss_typename_type_k fixed_array_3d<T, A, P>::index_type i1, ss_typename_type_k fixed_array_3d<T, A, P>::index_type i2) const
{
    stlsoft_message_assert("fixed array index out of range", i0 < m_d0 && i1 < m_d1 && i2 < m_d2);

    return *(m_data + _calc_index(i0, i1, i2));
}

template <ss_typename_param_k T, ss_typename_param_k A, ss_typename_param_k P>
inline ss_typename_type_k fixed_array_3d<T, A, P>::dimension_type fixed_array_3d<T, A, P>::at(ss_typename_type_k fixed_array_3d<T, A, P>::index_type i0)
{
    _range_check(i0);

    return dimension_type(m_data + i0 * m_d1 * m_d2, m_d1, m_d2);
}

template <ss_typename_param_k T, ss_typename_param_k A, ss_typename_param_k P>
inline ss_typename_type_k fixed_array_3d<T, A, P>::dimension_type const fixed_array_3d<T, A, P>::at(ss_typename_type_k fixed_array_3d<T, A, P>::index_type i0) const
{
    _range_check(i0);

    return dimension_type(m_data + i0 * m_d1 * m_d2, m_d1, m_d2);
}

template <ss_typename_param_k T, ss_typename_param_k A, ss_typename_param_k P>
inline ss_typename_type_k fixed_array_3d<T, A, P>::dimension_type fixed_array_3d<T, A, P>::at_unchecked(ss_typename_type_k fixed_array_3d<T, A, P>::index_type i0)
{
    stlsoft_message_assert("fixed array index out of range", i0 < m_d0);

    return dimension_type(m_data + i0 * m_d1 * m_d2, m_d1, m_d2);
}

template <ss_typename_param_k T, ss_typename_param_k A, ss_typename_param_k P>
inline ss_typename_type_k fixed_array_3d<T, A, P>::dimension_type const fixed_array_3d<T, A, P>::at_unchecked(ss_typename_type_k fixed_array_3d<T, A, P>::index_type i0) const
{
    stlsoft_message_assert("fixed array index out of range", i0 < m_d0);

    return dimension_type(m_data + i0 * m_d1 * m_d2, m_d1, m_d2);
}

template <ss_typename_param_k T, ss_typename_param_k A, ss_typename_param_k P>
inline ss_typename_type_k fixed_array_3d<T, A, P>::dimension_type fixed_array_3d<T, A, P>::operator [](ss_typename_type_k fixed_array_3d<T, A, P>::index_type i0)
{
    stlsoft_message_assert("fixed array index out of range", i0 < m_d0);

    return dimension_type(m_data + i0 * m_d1 * m_d2, m_d1, m_d2);
}

template <ss_typename_param_k T, ss_typename_param_k A, ss_typename_param_k P>
inline ss_typename_type_k fixed_array_3d<T, A, P>::dimension_type const fixed_array_3d<T, A, P>::operator [](ss_typename_type_k fixed_array_3d<T, A, P>::index_type i0) const
{
    stlsoft_message_assert("fixed array index out of range", i0 < m_d0);

    return dimension_type(m_data + i0 * m_d1 * m_d2, m_d1, m_d2);
}

template <ss_typename_param_k T, ss_typename_param_k A, ss_typename_param_k P>
inline ss_typename_type_k fixed_array_3d<T, A, P>::reference fixed_array_3d<T, A, P>::front()
{
    return at(0, 0, 0);
}

template <ss_typename_param_k T, ss_typename_param_k A, ss_typename_param_k P>
inline ss_typename_type_k fixed_array_3d<T, A, P>::reference fixed_array_3d<T, A, P>::back()
{
    return at(m_d0 - 1, m_d1 - 1, m_d2 - 1);
}

template <ss_typename_param_k T, ss_typename_param_k A, ss_typename_param_k P>
inline ss_typename_type_k fixed_array_3d<T, A, P>::const_reference fixed_array_3d<T, A, P>::front() const
{
    return at(0, 0, 0);
}

template <ss_typename_param_k T, ss_typename_param_k A, ss_typename_param_k P>
inline ss_typename_type_k fixed_array_3d<T, A, P>::const_reference fixed_array_3d<T, A, P>::back() const
{
    return at(m_d0 - 1, m_d1 - 1, m_d2 - 1);
}

template <ss_typename_param_k T, ss_typename_param_k A, ss_typename_param_k P>
inline ss_typename_type_k fixed_array_3d<T, A, P>::index_type fixed_array_3d<T, A, P>::dimension0() const
{
    return m_d0;
}

template <ss_typename_param_k T, ss_typename_param_k A, ss_typename_param_k P>
inline ss_typename_type_k fixed_array_3d<T, A, P>::index_type fixed_array_3d<T, A, P>::dimension1() const
{
    return m_d1;
}

template <ss_typename_param_k T, ss_typename_param_k A, ss_typename_param_k P>
inline ss_typename_type_k fixed_array_3d<T, A, P>::index_type fixed_array_3d<T, A, P>::dimension2() const
{
    return m_d2;
}

template <ss_typename_param_k T, ss_typename_param_k A, ss_typename_param_k P>
inline ss_typename_type_k fixed_array_3d<T, A, P>::index_type fixed_array_3d<T, A, P>::size() const
{
    return m_d0 * m_d1 * m_d2;
}

template <ss_typename_param_k T, ss_typename_param_k A, ss_typename_param_k P>
inline ss_bool_t fixed_array_3d<T, A, P>::empty() const
{
    return false;
}

template <ss_typename_param_k T, ss_typename_param_k A, ss_typename_param_k P>
inline /* static */ ss_typename_type_k fixed_array_3d<T, A, P>::size_type fixed_array_3d<T, A, P>::max_size()
{
    return static_cast<size_type>(-1) / sizeof(T);
}

template <ss_typename_param_k T, ss_typename_param_k A, ss_typename_param_k P>
inline ss_typename_type_k fixed_array_3d<T, A, P>::iterator fixed_array_3d<T, A, P>::begin()
{
    return m_data;
}

template <ss_typename_param_k T, ss_typename_param_k A, ss_typename_param_k P>
inline ss_typename_type_k fixed_array_3d<T, A, P>::iterator fixed_array_3d<T, A, P>::end()
{
    return m_data + size();
}

template <ss_typename_param_k T, ss_typename_param_k A, ss_typename_param_k P>
inline ss_typename_type_k fixed_array_3d<T, A, P>::const_iterator fixed_array_3d<T, A, P>::begin() const
{
    return m_data;
}

template <ss_typename_param_k T, ss_typename_param_k A, ss_typename_param_k P>
inline ss_typename_type_k fixed_array_3d<T, A, P>::const_iterator fixed_array_3d<T, A, P>::end() const
{
    return m_data + size();
}

template <ss_typename_param_k T, ss_typename_param_k A, ss_typename_param_k P>
inline ss_typename_type_k fixed_array_3d<T, A, P>::reverse_iterator fixed_array_3d<T, A, P>::rbegin()
{
    return reverse_iterator(end());
}

template <ss_typename_param_k T, ss_typename_param_k A, ss_typename_param_k P>
inline ss_typename_type_k fixed_array_3d<T, A, P>::reverse_iterator fixed_array_3d<T, A, P>::rend()
{
    return reverse_iterator(begin());
}

template <ss_typename_param_k T, ss_typename_param_k A, ss_typename_param_k P>
inline ss_typename_type_k fixed_array_3d<T, A, P>::const_reverse_iterator fixed_array_3d<T, A, P>::rbegin() const
{
    return const_reverse_iterator(end());
}

template <ss_typename_param_k T, ss_typename_param_k A, ss_typename_param_k P>
inline ss_typename_type_k fixed_array_3d<T, A, P>::const_reverse_iterator fixed_array_3d<T, A, P>::rend() const
{
    return const_reverse_iterator(begin());
}

template <ss_typename_param_k T, ss_typename_param_k A, ss_typename_param_k P>
inline ss_typename_type_k fixed_array_3d<T, A, P>::value_type const *fixed_array_3d<T, A, P>::data() const
{
    return m_data;
}

// fixed_array_4d

template<   ss_typename_param_k T
        ,   ss_typename_param_k A
        ,   ss_typename_param_k P
        >
inline ss_typename_type_k fixed_array_4d<T, A, P>::index_type fixed_array_4d<T, A, P>::_calc_index(index_type i0, index_type i1, index_type i2, index_type i3) const
{
    return (((i0 * m_d1) + i1) * m_d2 + i2) * m_d3 + i3;
}

template<   ss_typename_param_k T
        ,   ss_typename_param_k A
        ,   ss_typename_param_k P
        >
inline void fixed_array_4d<T, A, P>::_range_check(ss_typename_param_k fixed_array_4d<T, A, P>::index_type i0, ss_typename_param_k fixed_array_4d<T, A, P>::index_type i1, ss_typename_param_k fixed_array_4d<T, A, P>::index_type i2, ss_typename_param_k fixed_array_4d<T, A, P>::index_type i3) const stlsoft_throw_1(stlsoft_ns_qual_std(out_of_range) )
{
#ifdef __STLSOFT_CF_EXCEPTION_SUPPORT
    if( !(i0 < m_d0) ||
        !(i1 < m_d1) ||
        !(i2 < m_d2) ||
        !(i3 < m_d3))
    {
        stlsoft_ns_qual_std(out_of_range)   x("fixed array index out of range");

#ifdef __STLSOFT_COMPILER_IS_MSVC
        if(1) // This is needed to avoid Visual C++ warning 4702: 'unreachable code'
#endif /* __STLSOFT_COMPILER_IS_MSVC */
        {
            throw x;
        }
    }
#else
    stlsoft_message_assert("fixed array index out of range", i0 < m_d0 && i1 < m_d1 && i2 < m_d2 && i3 < m_d3);
#endif /* __STLSOFT_CF_EXCEPTION_SUPPORT */
}

template<   ss_typename_param_k T
        ,   ss_typename_param_k A
        ,   ss_typename_param_k P
        >
inline void fixed_array_4d<T, A, P>::_range_check(ss_typename_param_k fixed_array_4d<T, A, P>::index_type i0) const stlsoft_throw_1(stlsoft_ns_qual_std(out_of_range) )
{
#ifdef __STLSOFT_CF_EXCEPTION_SUPPORT
    if(!(i0 < m_d0))
    {
        stlsoft_ns_qual_std(out_of_range)   x("fixed array index out of range");

#ifdef __STLSOFT_COMPILER_IS_MSVC
        if(1) // This is needed to avoid Visual C++ warning 4702: 'unreachable code'
#endif /* __STLSOFT_COMPILER_IS_MSVC */
        {
            throw x;
        }
    }
#else
    stlsoft_message_assert("fixed array index out of range", i0 < m_d0);
#endif /* __STLSOFT_CF_EXCEPTION_SUPPORT */
}

template<   ss_typename_param_k T
        ,   ss_typename_param_k A
        ,   ss_typename_param_k P
        >
inline fixed_array_4d<T, A, P>::fixed_array_4d(T *data, ss_typename_param_k fixed_array_4d<T, A, P>::index_type d0, ss_typename_param_k fixed_array_4d<T, A, P>::index_type d1, ss_typename_param_k fixed_array_4d<T, A, P>::index_type d2, ss_typename_param_k fixed_array_4d<T, A, P>::index_type d3)
    : m_data(data)
    , m_d0(d0)
    , m_d1(d1)
    , m_d2(d2)
    , m_d3(d3)
    , m_bProxy(true)
{}

template<   ss_typename_param_k T
        ,   ss_typename_param_k A
        ,   ss_typename_param_k P
        >
inline fixed_array_4d<T, A, P>::fixed_array_4d(ss_typename_param_k fixed_array_4d<T, A, P>::index_type d0, ss_typename_param_k fixed_array_4d<T, A, P>::index_type d1, ss_typename_param_k fixed_array_4d<T, A, P>::index_type d2, ss_typename_param_k fixed_array_4d<T, A, P>::index_type d3)
    : m_data(allocator_type::allocate(d0 * d1 * d2 * d3, NULL))
    , m_d0(d0)
    , m_d1(d1)
    , m_d2(d2)
    , m_d3(d3)
    , m_bProxy(false)
{
    if(policy_type::value)
    {
        for(index_type i0 = 0; i0 < m_d0; ++i0)
        {
            for(index_type i1 = 0; i1 < m_d1; ++i1)
            {
                for(index_type i2 = 0; i2 < m_d2; ++i2)
                {
                    for(index_type i3 = 0; i3 < m_d3; ++i3)
                    {
                        allocator_type::construct(m_data + _calc_index(i0, i1, i2, i3));
                    }
                }
            }
        }
    }
}

template<   ss_typename_param_k T
        ,   ss_typename_param_k A
        ,   ss_typename_param_k P
        >
inline fixed_array_4d<T, A, P>::fixed_array_4d(ss_typename_param_k fixed_array_4d<T, A, P>::index_type d0, ss_typename_param_k fixed_array_4d<T, A, P>::index_type d1, ss_typename_param_k fixed_array_4d<T, A, P>::index_type d2, ss_typename_param_k fixed_array_4d<T, A, P>::index_type d3, ss_typename_param_k fixed_array_4d<T, A, P>::value_type const &t)
    : m_data(allocator_type::allocate(d0 * d1 * d2 * d3, NULL))
    , m_d0(d0)
    , m_d1(d1)
    , m_d2(d2)
    , m_d3(d3)
    , m_bProxy(false)
{
    if(policy_type::value)
    {
        for(index_type i0 = 0; i0 < m_d0; ++i0)
        {
            for(index_type i1 = 0; i1 < m_d1; ++i1)
            {
                for(index_type i2 = 0; i2 < m_d2; ++i2)
                {
                    for(index_type i3 = 0; i3 < m_d3; ++i3)
                    {
                        allocator_type::construct(m_data + _calc_index(i0, i1, i2, i3), t);
                    }
                }
            }
        }
    }
}

template<   ss_typename_param_k T
        ,   ss_typename_param_k A
        ,   ss_typename_param_k P
        >
inline fixed_array_4d<T, A, P>::~fixed_array_4d() stlsoft_throw_0()
{
    if(!m_bProxy)
    {
        if(policy_type::value)
        {
            for(index_type i0 = 0; i0 < m_d0; ++i0)
            {
                for(index_type i1 = 0; i1 < m_d1; ++i1)
                {
                    for(index_type i2 = 0; i2 < m_d2; ++i2)
                    {
                        for(index_type i3 = 0; i3 < m_d3; ++i3)
                        {
                            allocator_type::destroy(m_data + _calc_index(i0, i1, i2, i3));
                        }
                    }
                }
            }
        }

        allocator_type::deallocate(m_data, size());
    }
}

template<   ss_typename_param_k T
        ,   ss_typename_param_k A
        ,   ss_typename_param_k P
        >
inline ss_typename_type_k fixed_array_4d<T, A, P>::reference fixed_array_4d<T, A, P>::at(ss_typename_param_k fixed_array_4d<T, A, P>::index_type i0, ss_typename_param_k fixed_array_4d<T, A, P>::index_type i1, ss_typename_param_k fixed_array_4d<T, A, P>::index_type i2, ss_typename_param_k fixed_array_4d<T, A, P>::index_type i3)
{
    _range_check(i0, i1, i2, i3);

    return *(m_data + _calc_index(i0, i1, i2, i3));
}

template<   ss_typename_param_k T
        ,   ss_typename_param_k A
        ,   ss_typename_param_k P
        >
inline ss_typename_type_k fixed_array_4d<T, A, P>::const_reference fixed_array_4d<T, A, P>::at(ss_typename_param_k fixed_array_4d<T, A, P>::index_type i0, ss_typename_param_k fixed_array_4d<T, A, P>::index_type i1, ss_typename_param_k fixed_array_4d<T, A, P>::index_type i2, ss_typename_param_k fixed_array_4d<T, A, P>::index_type i3) const
{
    _range_check(i0, i1, i2, i3);

    return *(m_data + _calc_index(i0, i1, i2, i3));
}

template<   ss_typename_param_k T
        ,   ss_typename_param_k A
        ,   ss_typename_param_k P
        >
inline ss_typename_type_k fixed_array_4d<T, A, P>::reference fixed_array_4d<T, A, P>::at_unchecked(ss_typename_param_k fixed_array_4d<T, A, P>::index_type i0, ss_typename_param_k fixed_array_4d<T, A, P>::index_type i1, ss_typename_param_k fixed_array_4d<T, A, P>::index_type i2, ss_typename_param_k fixed_array_4d<T, A, P>::index_type i3)
{
    stlsoft_message_assert("fixed array index out of range", i0 < m_d0 && i1 < m_d1 && i2 < m_d2 && i3 < m_d3);

    return *(m_data + _calc_index(i0, i1, i2, i3));
}

template<   ss_typename_param_k T
        ,   ss_typename_param_k A
        ,   ss_typename_param_k P
        >
inline ss_typename_type_k fixed_array_4d<T, A, P>::const_reference fixed_array_4d<T, A, P>::at_unchecked(ss_typename_param_k fixed_array_4d<T, A, P>::index_type i0, ss_typename_param_k fixed_array_4d<T, A, P>::index_type i1, ss_typename_param_k fixed_array_4d<T, A, P>::index_type i2, ss_typename_param_k fixed_array_4d<T, A, P>::index_type i3) const
{
    stlsoft_message_assert("fixed array index out of range", i0 < m_d0 && i1 < m_d1 && i2 < m_d2 && i3 < m_d3);

    return *(m_data + _calc_index(i0, i1, i2, i3));
}

template<   ss_typename_param_k T
        ,   ss_typename_param_k A
        ,   ss_typename_param_k P
        >
inline ss_typename_type_k fixed_array_4d<T, A, P>::dimension_type fixed_array_4d<T, A, P>::at(ss_typename_param_k fixed_array_4d<T, A, P>::index_type i0)
{
    _range_check(i0);

    return dimension_type(m_data + i0 * m_d1 * m_d2 * m_d3, m_d1, m_d2, m_d3);
}

template<   ss_typename_param_k T
        ,   ss_typename_param_k A
        ,   ss_typename_param_k P
        >
inline const ss_typename_type_k fixed_array_4d<T, A, P>::dimension_type fixed_array_4d<T, A, P>::at(ss_typename_param_k fixed_array_4d<T, A, P>::index_type i0) const
{
    _range_check(i0);

    return dimension_type(m_data + i0 * m_d1 * m_d2 * m_d3, m_d1, m_d2, m_d3);
}

template<   ss_typename_param_k T
        ,   ss_typename_param_k A
        ,   ss_typename_param_k P
        >
inline ss_typename_type_k fixed_array_4d<T, A, P>::dimension_type fixed_array_4d<T, A, P>::at_unchecked(ss_typename_param_k fixed_array_4d<T, A, P>::index_type i0)
{
    stlsoft_message_assert("fixed array index out of range", i0 < m_d0);

    return dimension_type(m_data + i0 * m_d1 * m_d2 * m_d3, m_d1, m_d2, m_d3);
}

template<   ss_typename_param_k T
        ,   ss_typename_param_k A
        ,   ss_typename_param_k P
        >
inline ss_typename_type_k fixed_array_4d<T, A, P>::dimension_type const fixed_array_4d<T, A, P>::at_unchecked(ss_typename_param_k fixed_array_4d<T, A, P>::index_type i0) const
{
    stlsoft_message_assert("fixed array index out of range", i0 < m_d0);

    return dimension_type(m_data + i0 * m_d1 * m_d2 * m_d3, m_d1, m_d2, m_d3);
}

template<   ss_typename_param_k T
        ,   ss_typename_param_k A
        ,   ss_typename_param_k P
        >
inline ss_typename_type_k fixed_array_4d<T, A, P>::dimension_type fixed_array_4d<T, A, P>::operator [](ss_typename_param_k fixed_array_4d<T, A, P>::index_type i0)
{
    stlsoft_message_assert("fixed array index out of range", i0 < m_d0);

    return dimension_type(m_data + i0 * m_d1 * m_d2 * m_d3, m_d1, m_d2, m_d3);
}

template<   ss_typename_param_k T
        ,   ss_typename_param_k A
        ,   ss_typename_param_k P
        >
inline ss_typename_type_k fixed_array_4d<T, A, P>::dimension_type const fixed_array_4d<T, A, P>::operator [](ss_typename_param_k fixed_array_4d<T, A, P>::index_type i0) const
{
    stlsoft_message_assert("fixed array index out of range", i0 < m_d0);

    return dimension_type(m_data + i0 * m_d1 * m_d2 * m_d3, m_d1, m_d2, m_d3);
}

template<   ss_typename_param_k T
        ,   ss_typename_param_k A
        ,   ss_typename_param_k P
        >
inline ss_typename_type_k fixed_array_4d<T, A, P>::reference fixed_array_4d<T, A, P>::front()
{
    return at(0, 0, 0, 0);
}

template<   ss_typename_param_k T
        ,   ss_typename_param_k A
        ,   ss_typename_param_k P
        >
inline ss_typename_type_k fixed_array_4d<T, A, P>::reference fixed_array_4d<T, A, P>::back()
{
    return at(m_d0 - 1, m_d1 - 1, m_d2 - 1, m_d3 - 1);
}

template<   ss_typename_param_k T
        ,   ss_typename_param_k A
        ,   ss_typename_param_k P
        >
inline ss_typename_type_k fixed_array_4d<T, A, P>::const_reference fixed_array_4d<T, A, P>::front() const
{
    return at(0, 0, 0, 0);
}

template<   ss_typename_param_k T
        ,   ss_typename_param_k A
        ,   ss_typename_param_k P
        >
inline ss_typename_type_k fixed_array_4d<T, A, P>::const_reference fixed_array_4d<T, A, P>::back() const
{
    return at(m_d0 - 1, m_d1 - 1, m_d2 - 1, m_d3 - 1);
}

template<   ss_typename_param_k T
        ,   ss_typename_param_k A
        ,   ss_typename_param_k P
        >
inline ss_typename_type_k fixed_array_4d<T, A, P>::index_type fixed_array_4d<T, A, P>::dimension0() const
{
    return m_d0;
}

template<   ss_typename_param_k T
        ,   ss_typename_param_k A
        ,   ss_typename_param_k P
        >
inline ss_typename_type_k fixed_array_4d<T, A, P>::index_type fixed_array_4d<T, A, P>::dimension1() const
{
    return m_d1;
}

template<   ss_typename_param_k T
        ,   ss_typename_param_k A
        ,   ss_typename_param_k P
        >
inline ss_typename_type_k fixed_array_4d<T, A, P>::index_type fixed_array_4d<T, A, P>::dimension2() const
{
    return m_d2;
}

template<   ss_typename_param_k T
        ,   ss_typename_param_k A
        ,   ss_typename_param_k P
        >
inline ss_typename_type_k fixed_array_4d<T, A, P>::index_type fixed_array_4d<T, A, P>::dimension3() const
{
    return m_d3;
}

template<   ss_typename_param_k T
        ,   ss_typename_param_k A
        ,   ss_typename_param_k P
        >
inline ss_typename_type_k fixed_array_4d<T, A, P>::index_type fixed_array_4d<T, A, P>::size() const
{
    return m_d0 * m_d1 * m_d2 * m_d3;
}

template<   ss_typename_param_k T
        ,   ss_typename_param_k A
        ,   ss_typename_param_k P
        >
inline ss_bool_t fixed_array_4d<T, A, P>::empty() const
{
    return false;
}

template<   ss_typename_param_k T
        ,   ss_typename_param_k A
        ,   ss_typename_param_k P
        >
inline /* static */ ss_typename_type_k fixed_array_4d<T, A, P>::size_type fixed_array_4d<T, A, P>::max_size()
{
    return static_cast<size_type>(-1) / sizeof(T);
}

template<   ss_typename_param_k T
        ,   ss_typename_param_k A
        ,   ss_typename_param_k P
        >
inline ss_typename_type_k fixed_array_4d<T, A, P>::iterator fixed_array_4d<T, A, P>::begin()
{
    return m_data;
}

template<   ss_typename_param_k T
        ,   ss_typename_param_k A
        ,   ss_typename_param_k P
        >
inline ss_typename_type_k fixed_array_4d<T, A, P>::iterator fixed_array_4d<T, A, P>::end()
{
    return m_data + size();
}

template<   ss_typename_param_k T
        ,   ss_typename_param_k A
        ,   ss_typename_param_k P
        >
inline ss_typename_type_k fixed_array_4d<T, A, P>::const_iterator fixed_array_4d<T, A, P>::begin() const
{
    return m_data;
}

template<   ss_typename_param_k T
        ,   ss_typename_param_k A
        ,   ss_typename_param_k P
        >
inline ss_typename_type_k fixed_array_4d<T, A, P>::const_iterator fixed_array_4d<T, A, P>::end() const
{
    return m_data + size();
}

template<   ss_typename_param_k T
        ,   ss_typename_param_k A
        ,   ss_typename_param_k P
        >
inline ss_typename_type_k fixed_array_4d<T, A, P>::reverse_iterator fixed_array_4d<T, A, P>::rbegin()
{
    return reverse_iterator(end());
}

template<   ss_typename_param_k T
        ,   ss_typename_param_k A
        ,   ss_typename_param_k P
        >
inline ss_typename_type_k fixed_array_4d<T, A, P>::reverse_iterator fixed_array_4d<T, A, P>::rend()
{
    return reverse_iterator(begin());
}

template<   ss_typename_param_k T
        ,   ss_typename_param_k A
        ,   ss_typename_param_k P
        >
inline ss_typename_type_k fixed_array_4d<T, A, P>::const_reverse_iterator fixed_array_4d<T, A, P>::rbegin() const
{
    return const_reverse_iterator(end());
}

template<   ss_typename_param_k T
        ,   ss_typename_param_k A
        ,   ss_typename_param_k P
        >
inline ss_typename_type_k fixed_array_4d<T, A, P>::const_reverse_iterator fixed_array_4d<T, A, P>::rend() const
{
    return const_reverse_iterator(begin());
}

template<   ss_typename_param_k T
        ,   ss_typename_param_k A
        ,   ss_typename_param_k P
        >
inline ss_typename_type_k fixed_array_4d<T, A, P>::value_type const *fixed_array_4d<T, A, P>::data() const
{
    return m_data;
}

#endif /* !__STLSOFT_DOCUMENTATION_SKIP_SECTION */

/* ////////////////////////////////////////////////////////////////////////// */

#ifndef _STLSOFT_NO_NAMESPACE
} // namespace stlsoft
#endif /* _STLSOFT_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* !_STLSOFT_INCL_H_STLSOFT_FIXED_ARRAY */

/* ////////////////////////////////////////////////////////////////////////// */
