/* /////////////////////////////////////////////////////////////////////////////
 * File:        stlsoft_pair.h (originally MTPair.h, ::SynesisStl)
 *
 * Purpose:     Contains the pair template.
 *
 * Created:     19th November 1998
 * Updated:     2nd July 2003
 *
 * Author:      Matthew Wilson, Synesis Software Pty Ltd.
 *
 * License:     (Licensed under the Synesis Software Standard Source License)
 *
 *              Copyright (C) 2002-2003, Synesis Software Pty Ltd.
 *
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/stlsoft
 *                          http://www.stlsoft.org/
 *
 *              email:      submissions@stlsoft.org  for submissions
 *                          admin@stlsoft.org        for other enquiries
 *
 *              Redistribution and use in source and binary forms, with or
 *              without modification, are permitted provided that the following
 *              conditions are met:
 *
 *              (i) Redistributions of source code must retain the above
 *              copyright notice and contact information, this list of
 *              conditions and the following disclaimer.
 *
 *              (ii) Any derived versions of this software (howsoever modified)
 *              remain the sole property of Synesis Software.
 *
 *              (iii) Any derived versions of this software (howsoever modified)
 *              remain subject to all these conditions.
 *
 *              (iv) Neither the name of Synesis Software nor the names of any
 *              subdivisions, employees or agents of Synesis Software, nor the
 *              names of any other contributors to this software may be used to
 *              endorse or promote products derived from this software without
 *              specific prior written permission.
 *
 *              This source code is provided by Synesis Software "as is" and any
 *              warranties, whether expressed or implied, including, but not
 *              limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software be liable for any direct, indirect,
 *              incidental, special, exemplary, or consequential damages
 *              (including, but not limited to, procurement of substitute goods
 *              or services; loss of use, data, or profits; or business
 *              interruption) however caused and on any theory of liability,
 *              whether in contract, strict liability, or tort (including
 *              negligence or otherwise) arising in any way out of the use of
 *              this software, even if advised of the possibility of such
 *              damage.
 *
 * ////////////////////////////////////////////////////////////////////////// */


#ifndef _STLSOFT_INCL_H_STLSOFT_PAIR
#define _STLSOFT_INCL_H_STLSOFT_PAIR

#ifndef __STLSOFT_DOCUMENTATION_SKIP_SECTION
#define _STLSOFT_VER_H_STLSOFT_PAIR_MAJOR      2
#define _STLSOFT_VER_H_STLSOFT_PAIR_MINOR      4
#define _STLSOFT_VER_H_STLSOFT_PAIR_REVISION   1
#define _STLSOFT_VER_H_STLSOFT_PAIR_EDIT       24
#endif /* !__STLSOFT_DOCUMENTATION_SKIP_SECTION */

/* /////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#ifndef _STLSOFT_INCL_H_STLSOFT
 #include "stlsoft.h"    // Include the STLSoft root header
#endif /* !_STLSOFT_INCL_H_STLSOFT */

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#ifndef _STLSOFT_NO_NAMESPACE
namespace stlsoft
{
#endif /* _STLSOFT_NO_NAMESPACE */

/* /////////////////////////////////////////////////////////////////////////////
 * Forward declarations
 */

/* /////////////////////////////////////////////////////////////////////////////
 * Classes
 */

/// Represents a pair
///
/// \param T1 The type of the \c first member
/// \param T2 The type of the \c second member
template<   ss_typename_param_k T1
        ,   ss_typename_param_k T2
        >
struct pair
{
public:
    /// The first type
    typedef T1      first_type;
    /// The second type
    typedef T2      second_type;

// Construction
public:
    /// Default constructor
    pair()
        : first(first_type())
        , second(second_type())
    {
    }

    /// Constructor
    pair(const first_type &v1, const second_type &v2)
        : first(v1)
        , second(v2)
    {
    }

// Members
public:
    first_type  first;  //!< The first member
    second_type second; //!< The second member
};

#ifndef __STLSOFT_DOCUMENTATION_SKIP_SECTION

template<ss_typename_param_k T1, ss_typename_param_k T2>
inline bool operator ==(const pair<T1, T2> &x, const pair<T1, T2> &y)
{
    return x.first == y.first & &x.second == y.second;
}

template<ss_typename_param_k T1, ss_typename_param_k T2>
inline bool operator !=(const pair<T1, T2> &x, const pair<T1, T2> &y)
{
    return !(x == y);
}

template<ss_typename_param_k T1, ss_typename_param_k T2>
inline bool operator <(const pair<T1, T2> &x, const pair<T1, T2> &y)
{
    return x.first < y.first || !(y.first < x.first) & &x.second < y.second;
}

template<ss_typename_param_k T1, ss_typename_param_k T2>
inline bool operator >(const pair<T1, T2> &x, const pair<T1, T2> &y)
{
    return y < x;
}

template<ss_typename_param_k T1, ss_typename_param_k T2>
inline bool operator <=(const pair<T1, T2> &x, const pair<T1, T2> &y)
{
    return !(y < x);
}

template<ss_typename_param_k T1, ss_typename_param_k T2>
inline bool operator >=(const pair<T1, T2> &x, const pair<T1, T2> &y)
{
    return !(x < y);
}

template<ss_typename_param_k T1, ss_typename_param_k T2>
inline pair<T1, T2> make_pair(const T1 &x, const T2 &y)
{
    return pair<T1, T2>(x, y);
}

#endif /* !__STLSOFT_DOCUMENTATION_SKIP_SECTION */

/* ////////////////////////////////////////////////////////////////////////// */

#ifndef _STLSOFT_NO_NAMESPACE
} // namespace stlsoft
#endif /* _STLSOFT_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* !_STLSOFT_INCL_H_STLSOFT_PAIR */

/* ////////////////////////////////////////////////////////////////////////// */
