/* /////////////////////////////////////////////////////////////////////////////
 * File:        stlsoft_ptr_access.h
 *
 * Purpose:     Contains the get_ptr access shim.
 *
 * Created:     10th January 2002
 * Updated:     2nd July 2003
 *
 * Author:      Matthew Wilson, Synesis Software Pty Ltd.
 *
 * License:     (Licensed under the Synesis Software Standard Source License)
 *
 *              Copyright (C) 2002-2003, Synesis Software Pty Ltd.
 *
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/stlsoft
 *                          http://www.stlsoft.org/
 *
 *              email:      submissions@stlsoft.org  for submissions
 *                          admin@stlsoft.org        for other enquiries
 *
 *              Redistribution and use in source and binary forms, with or
 *              without modification, are permitted provided that the following
 *              conditions are met:
 *
 *              (i) Redistributions of source code must retain the above
 *              copyright notice and contact information, this list of
 *              conditions and the following disclaimer.
 *
 *              (ii) Any derived versions of this software (howsoever modified)
 *              remain the sole property of Synesis Software.
 *
 *              (iii) Any derived versions of this software (howsoever modified)
 *              remain subject to all these conditions.
 *
 *              (iv) Neither the name of Synesis Software nor the names of any
 *              subdivisions, employees or agents of Synesis Software, nor the
 *              names of any other contributors to this software may be used to
 *              endorse or promote products derived from this software without
 *              specific prior written permission.
 *
 *              This source code is provided by Synesis Software "as is" and any
 *              warranties, whether expressed or implied, including, but not
 *              limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software be liable for any direct, indirect,
 *              incidental, special, exemplary, or consequential damages
 *              (including, but not limited to, procurement of substitute goods
 *              or services; loss of use, data, or profits; or business
 *              interruption) however caused and on any theory of liability,
 *              whether in contract, strict liability, or tort (including
 *              negligence or otherwise) arising in any way out of the use of
 *              this software, even if advised of the possibility of such
 *              damage.
 *
 * ////////////////////////////////////////////////////////////////////////// */


#ifndef _STLSOFT_INCL_H_STLSOFT_PTR_ACCESS
#define _STLSOFT_INCL_H_STLSOFT_PTR_ACCESS

#ifndef __STLSOFT_DOCUMENTATION_SKIP_SECTION
#define _STLSOFT_VER_H_STLSOFT_PTR_ACCESS_MAJOR     1
#define _STLSOFT_VER_H_STLSOFT_PTR_ACCESS_MINOR     2
#define _STLSOFT_VER_H_STLSOFT_PTR_ACCESS_REVISION  1
#define _STLSOFT_VER_H_STLSOFT_PTR_ACCESS_EDIT      11
#endif /* !__STLSOFT_DOCUMENTATION_SKIP_SECTION */

/* /////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#ifndef _STLSOFT_INCL_H_STLSOFT
 #include "stlsoft.h"               // Include the STLSoft root header
#endif /* !_STLSOFT_INCL_H_STLSOFT */
#ifndef _STLSOFT_PTR_ACCESS_NO_AUTO_PTR
 #include <memory>                  // std::auto_ptr
#endif /* _STLSOFT_PTR_ACCESS_NO_AUTO_PTR */

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#ifndef _STLSOFT_NO_NAMESPACE
namespace stlsoft
{
#endif /* _STLSOFT_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

/// \weakgroup concepts STLSoft Concepts

/// \weakgroup concepts_shims Shims
/// \ingroup concepts 

/// \weakgroup concepts_shims_pointer_access Pointer Access Shims
/// \ingroup concepts_shims
/// \brief These \ref concepts_shims "shims" retrieve the pointer for arbitrary pointer types

/// \defgroup stlsoft_pointer_access_shims Pointer Access Shims (STLSoft)
/// \ingroup STLSoft concepts_shims_pointer_access
/// \brief These \ref concepts_shims "shims" retrieve the pointer for arbitrary pointer types
/// @{

/* /////////////////////////////////////////////////////////////////////////////
 * get_ptr functions
 */

/* pointer to const. */
template <ss_typename_param_k T>
inline T const *get_ptr(T const *p)
{
    return p;
}

/* pointer to non-const. */
template <ss_typename_param_k T>
inline T *get_ptr(T *p)
{
    return p;
}

/* std::auto_ptr */
#ifndef _STLSOFT_PTR_ACCESS_NO_AUTO_PTR
template <ss_typename_param_k T>
inline T *get_ptr(stlsoft_ns_qual_std(auto_ptr)<T> &p)
{
    return p.get();
}

template <ss_typename_param_k T>
inline T const *get_ptr(stlsoft_ns_qual_std(auto_ptr)<T> const &p)
{
    return p.get();
}
#endif /* _STLSOFT_PTR_ACCESS_NO_AUTO_PTR */

/* ////////////////////////////////////////////////////////////////////////// */

/// @} // end of group stlsoft_pointer_access_shims

/* ////////////////////////////////////////////////////////////////////////// */

#ifndef _STLSOFT_NO_NAMESPACE
} // namespace stlsoft
#endif /* _STLSOFT_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* !_STLSOFT_INCL_H_STLSOFT_PTR_ACCESS */

/* ////////////////////////////////////////////////////////////////////////// */
