/* /////////////////////////////////////////////////////////////////////////////
 * File:        winstl_char_conversions.h (originally MLStrCnv.h, ::SynesisStd)
 *
 * Purpose:     Type conversions for Windows.
 *
 * Created:     
 * Updated:     22nd August 2003
 *
 * Author:      Matthew Wilson, Synesis Software Pty Ltd.
 *
 * License:     (Licensed under the Synesis Software Standard Source License)
 *
 *              Copyright (C) 2002-2003, Synesis Software Pty Ltd.
 *
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/winstl
 *                          http://www.winstl.org/
 *
 *              email:      submissions@winstl.org  for submissions
 *                          admin@winstl.org        for other enquiries
 *
 *              Redistribution and use in source and binary forms, with or
 *              without modification, are permitted provided that the following
 *              conditions are met:
 *
 *              (i) Redistributions of source code must retain the above
 *              copyright notice and contact information, this list of
 *              conditions and the following disclaimer.
 *
 *              (ii) Any derived versions of this software (howsoever modified)
 *              remain the sole property of Synesis Software.
 *
 *              (iii) Any derived versions of this software (howsoever modified)
 *              remain subject to all these conditions.
 *
 *              (iv) Neither the name of Synesis Software nor the names of any
 *              subdivisions, employees or agents of Synesis Software, nor the
 *              names of any other contributors to this software may be used to
 *              endorse or promote products derived from this software without
 *              specific prior written permission.
 *
 *              This source code is provided by Synesis Software "as is" and any
 *              warranties, whether expressed or implied, including, but not
 *              limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software be liable for any direct, indirect,
 *              incidental, special, exemplary, or consequential damages
 *              (including, but not limited to, procurement of substitute goods
 *              or services; loss of use, data, or profits; or business
 *              interruption) however caused and on any theory of liability,
 *              whether in contract, strict liability, or tort (including
 *              negligence or otherwise) arising in any way out of the use of
 *              this software, even if advised of the possibility of such
 *              damage.
 *
 * ////////////////////////////////////////////////////////////////////////// */


#ifndef _WINSTL_INCL_H_WINSTL_WINDOWS_CHAR_CONVERSIONS
#define _WINSTL_INCL_H_WINSTL_WINDOWS_CHAR_CONVERSIONS

#ifndef __STLSOFT_DOCUMENTATION_SKIP_SECTION
#define _WINSTL_VER_H_WINSTL_WINDOWS_CHAR_CONVERSIONS_MAJOR       1
#define _WINSTL_VER_H_WINSTL_WINDOWS_CHAR_CONVERSIONS_MINOR       0
#define _WINSTL_VER_H_WINSTL_WINDOWS_CHAR_CONVERSIONS_REVISION    4
#define _WINSTL_VER_H_WINSTL_WINDOWS_CHAR_CONVERSIONS_EDIT        6
#endif /* !__STLSOFT_DOCUMENTATION_SKIP_SECTION */

/* /////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#ifndef _WINSTL_INCL_H_WINSTL
 #include "winstl.h"                        // Include the WinSTL root header
#endif /* !_WINSTL_INCL_H_WINSTL */
#ifndef _WINSTL_INCL_H_WINSTL_PROCESSHEAP_ALLOCATOR
 #include "winstl_processheap_allocator.h"  // winstl::processheap_allocator
#endif /* !_WINSTL_INCL_H_WINSTL_PROCESSHEAP_ALLOCATOR */
#ifndef _STLSOFT_INCL_H_STLSOFT_STRING_ACCESS
 #include "stlsoft_string_access.h"         // stlsoft::c_str_ptr
#endif /* !_STLSOFT_INCL_H_STLSOFT_STRING_ACCESS */
#ifndef _WINSTL_INCL_H_WINSTL_STRING_ACCESS
 #include "winstl_string_access.h"          // winstl::c_str_ptr
#endif /* !_WINSTL_INCL_H_WINSTL_STRING_ACCESS */
#ifndef _STLSOFT_INCL_H_STLSOFT_AUTO_BUFFER
 #include "stlsoft_auto_buffer.h"           // stlsoft::auto_buffer
#endif /* !_STLSOFT_INCL_H_STLSOFT_AUTO_BUFFER */

/* ////////////////////////////////////////////////////////////////////////// */

#ifndef _WINSTL_NO_NAMESPACE
 #ifdef _STLSOFT_NO_NAMESPACE
/* There is no stlsoft namespace, so must define ::winstl */
namespace winstl
{
 #else
/* Define stlsoft::winstl_project */

namespace stlsoft
{

namespace winstl_project
{

 #endif /* _STLSOFT_NO_NAMESPACE */
#endif /* !_WINSTL_NO_NAMESPACE */

stlsoft_ns_using(c_str_ptr)
stlsoft_ns_using(c_str_len)

/* /////////////////////////////////////////////////////////////////////////////
 * Classes
 */

template <size_t CCH>
class ansi2Unicode
    : public auto_buffer<ws_char_w_t, processheap_allocator<ws_char_w_t>, CCH>
{
private:
    typedef auto_buffer<ws_char_w_t, processheap_allocator<ws_char_w_t>, CCH>   parent_class_type;
public:
    typedef ws_typename_type_k parent_class_type::size_type                     size_type;
    typedef ws_typename_type_k parent_class_type::pointer                       pointer;

public:
#ifdef __STLSOFT_CF_MEMBER_TEMPLATE_CTOR_SUPPORT
    template <ws_typename_param_k S>
    explicit ansi2Unicode(S const &s)
#else
    explicit ansi2Unicode(ws_char_a_t const *s)
#endif // __STLSOFT_CF_MEMBER_TEMPLATE_CTOR_SUPPORT
        : parent_class_type(c_str_len(s) + 1)
    {
        // If the auto_buffer failed to allocate the required memory, and
        // we're not in an exception-environment, then size() will be zero
        if(0 == parent_class_type::size())
        {
            // Since we know that auto_buffer's parameterising size must
            // always be greater that 0, then 
            data()[0] = '\0';
        }
        else
        {
            size_type   size    =   parent_class_type::size();
            pointer     p       =   data();

            // Note: cannot use -1 for length, since s might be a type that
            // could change length
            p[(0 == ::MultiByteToWideChar(0, 0, c_str_ptr(s), size, p, size)) ? 0 : size - 1] = '\0';
        }
    }

// Not to be implemented
private:
    ansi2Unicode(ansi2Unicode const &);
    ansi2Unicode &operator =(ansi2Unicode const &);
};

template <size_t CCH>
class unicode2Ansi
    : public auto_buffer<ws_char_a_t, processheap_allocator<ws_char_a_t>, CCH>
{
private:
    typedef auto_buffer<ws_char_a_t, processheap_allocator<ws_char_a_t>, CCH>   parent_class_type;
public:
    typedef ws_typename_type_k parent_class_type::size_type                     size_type;
    typedef ws_typename_type_k parent_class_type::pointer                       pointer;

public:
#ifdef __STLSOFT_CF_MEMBER_TEMPLATE_CTOR_SUPPORT
    template <ws_typename_param_k S>
    explicit unicode2Ansi(S const &s)
#else
    explicit unicode2Ansi(ws_char_w_t const *s)
#endif // __STLSOFT_CF_MEMBER_TEMPLATE_CTOR_SUPPORT
        : parent_class_type(c_str_len(s) + 1)
    {
        // If the auto_buffer failed to allocate the required memory, and
        // we're not in an exception-environment, then size() will be zero
        if(0 == parent_class_type::size())
        {
            // Since we know that auto_buffer's parameterising size must
            // always be greater that 0, then 
            data()[0] = '\0';
        }
        else
        {
            size_type   size    =   parent_class_type::size();
            pointer     p       =   data();

            // Note: cannot use -1 for length, since s might be a type that
            // could change length
            p[(0 == ::WideCharToMultiByte(0, 0, c_str_ptr(s), size, p, size, NULL, NULL)) ? 0 : size - 1] = '\0';
        }
    }

// Not to be implemented
private:
    unicode2Ansi(unicode2Ansi const &);
    unicode2Ansi &operator =(unicode2Ansi const &);
};

/* ////////////////////////////////////////////////////////////////////////// */

typedef ansi2Unicode<_MAX_PATH>     a2w;
typedef unicode2Ansi<_MAX_PATH>     w2a;

/* ////////////////////////////////////////////////////////////////////////// */

#ifndef _WINSTL_NO_NAMESPACE
 #ifdef _STLSOFT_NO_NAMESPACE
} // namespace winstl
 #else
} // namespace winstl_project
} // namespace stlsoft
 #endif /* _STLSOFT_NO_NAMESPACE */
#endif /* !_WINSTL_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* _WINSTL_INCL_H_WINSTL_WINDOWS_CHAR_CONVERSIONS */

/* ////////////////////////////////////////////////////////////////////////// */
