/* /////////////////////////////////////////////////////////////////////////////
 * File:        winstl_combobox_sequence.h
 *
 * Purpose:     Contains the combobox_sequence class.
 *
 * Created:     13th November 2002
 * Updated:     2nd July 2003
 *
 * Author:      Matthew Wilson, Synesis Software Pty Ltd.
 *
 * License:     (Licensed under the Synesis Software Standard Source License)
 *
 *              Copyright (C) 2002-2003, Synesis Software Pty Ltd.
 *
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/winstl
 *                          http://www.winstl.org/
 *
 *              email:      submissions@winstl.org  for submissions
 *                          admin@winstl.org        for other enquiries
 *
 *              Redistribution and use in source and binary forms, with or
 *              without modification, are permitted provided that the following
 *              conditions are met:
 *
 *              (i) Redistributions of source code must retain the above
 *              copyright notice and contact information, this list of
 *              conditions and the following disclaimer.
 *
 *              (ii) Any derived versions of this software (howsoever modified)
 *              remain the sole property of Synesis Software.
 *
 *              (iii) Any derived versions of this software (howsoever modified)
 *              remain subject to all these conditions.
 *
 *              (iv) Neither the name of Synesis Software nor the names of any
 *              subdivisions, employees or agents of Synesis Software, nor the
 *              names of any other contributors to this software may be used to
 *              endorse or promote products derived from this software without
 *              specific prior written permission.
 *
 *              This source code is provided by Synesis Software "as is" and any
 *              warranties, whether expressed or implied, including, but not
 *              limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software be liable for any direct, indirect,
 *              incidental, special, exemplary, or consequential damages
 *              (including, but not limited to, procurement of substitute goods
 *              or services; loss of use, data, or profits; or business
 *              interruption) however caused and on any theory of liability,
 *              whether in contract, strict liability, or tort (including
 *              negligence or otherwise) arising in any way out of the use of
 *              this software, even if advised of the possibility of such
 *              damage.
 *
 * ////////////////////////////////////////////////////////////////////////// */


#ifndef _WINSTL_INCL_H_WINSTL_COMBOBOX_SEQUENCE
#define _WINSTL_INCL_H_WINSTL_COMBOBOX_SEQUENCE

#ifndef __STLSOFT_DOCUMENTATION_SKIP_SECTION
#define _WINSTL_VER_H_WINSTL_COMBOBOX_SEQUENCE_MAJOR    1
#define _WINSTL_VER_H_WINSTL_COMBOBOX_SEQUENCE_MINOR    1
#define _WINSTL_VER_H_WINSTL_COMBOBOX_SEQUENCE_REVISION 5
#define _WINSTL_VER_H_WINSTL_COMBOBOX_SEQUENCE_EDIT     12
#endif /* !__STLSOFT_DOCUMENTATION_SKIP_SECTION */

/* /////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#ifndef _WINSTL_INCL_H_WINSTL
 #include "winstl.h"                        // Include the WinSTL root header
#endif /* !_WINSTL_INCL_H_WINSTL */
#ifndef _STLSOFT_INCL_H_STLSOFT_AUTO_DESTRUCTOR
 #include "stlsoft_auto_buffer.h"           // Include the STLSoft auto_buffer
#endif /* _STLSOFT_INCL_H_STLSOFT_AUTO_DESTRUCTOR */
#ifndef _WINSTL_INCL_H_WINSTL_PROCESSHEAP_ALLOCATOR
 #include "winstl_processheap_allocator.h"  // processheap_allocator
#endif /* _WINSTL_INCL_H_WINSTL_PROCESSHEAP_ALLOCATOR */
#ifndef _STLSOFT_INCL_H_STLSOFT_ITERATOR
 #include "stlsoft_iterator.h"
#endif /* !_STLSOFT_INCL_H_STLSOFT_ITERATOR */
#ifndef _WINSTL_INCL_H_WINSTL_CONTROL_FUNCTIONS
 #include "winstl_control_functions.h"      // combobox_addstring, etc.
#endif /* !_WINSTL_INCL_H_WINSTL_CONTROL_FUNCTIONS */

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#ifndef _WINSTL_NO_NAMESPACE
 #ifdef _STLSOFT_NO_NAMESPACE
/* There is no stlsoft namespace, so must define ::winstl */
namespace winstl
{
 #else
/* Define stlsoft::winstl_project */

namespace stlsoft
{

namespace winstl_project
{

 #endif /* _STLSOFT_NO_NAMESPACE */
#endif /* !_WINSTL_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

/// \weakgroup libraries STLSoft Libraries
/// \brief The individual libraries

/// \weakgroup libraries_control Control Library
/// \ingroup libraries
/// \brief This library provides facilities for defining and manipulating GUI controls

/// \defgroup winstl_control_library Control Library (WinSTL)
/// \ingroup WinSTL libraries_control
/// \brief This library provides facilities for defining and manipulating Win32 GUI controls
/// @{

/* /////////////////////////////////////////////////////////////////////////////
 * Classes
 */

/// Iterator for combobox_sequence class
///
/// This class acts as the iterator for the combobox_sequence class, and implements
/// the Random Access Iterator concept
///
/// \param S The string type
template <ws_typename_param_k S>
class combobox_sequence_const_iterator
    : public stlsoft_ns_qual(iterator_base) <   winstl_ns_qual_std(random_access_iterator_tag)
                                            ,   S
                                            ,   ws_ptrdiff_t
                                            ,   void * // Placate Borland C++ (to test, replace with void)
                                            ,   S
                                            >
{
public:
    /// The string type
    typedef S                                           value_type;
    /// The current parameterisation of the type
    typedef combobox_sequence_const_iterator<S>         class_type;
    /// The character type
    typedef TCHAR                                       char_type;
    /// The size type
    typedef ws_size_t                                   size_type;
    /// The difference type
    typedef ws_ptrdiff_t                                difference_type;
protected:
    typedef processheap_allocator<char_type>            allocator_type;
    typedef auto_buffer<char_type, allocator_type, 256> buffer_type;

public:
    /// Construct an instance from the combo-box \c hwndComboBox at the given \c index
    combobox_sequence_const_iterator(HWND hwndComboBox, int index)
        : m_hwnd(hwndComboBox)
        , m_index(index)
    {}

public:
    /// Dereferences the iterator and returns the current value
    value_type const &operator *() const
    {
        int         len =   combobox_gettextlen(m_hwnd, m_index);
        buffer_type buffer(1 + len);

        combobox_gettext(m_hwnd, m_index, buffer);

        mutable_access(m_value).assign(buffer, buffer.size());

        return m_value;
    }

public:
    /// Pre-increment operator
    class_type &operator ++()
    {
        ++m_index;

        return *this;
    }

    /// Pre-increment operator
    class_type operator ++(int)
    {
        class_type  ret(*this);

        operator ++();

        return ret;
    }

    /// Compares \c this and \c rhs for equivalence
    difference_type compare(class_type const &rhs) const
    {
        winstl_message_assert("invalid comparison between iterators from different ranges", m_hwnd == rhs.m_hwnd);

        return m_index - rhs.m_index;
    }

    /// Indicates whether \c this and \c rhs are equivalent
    bool operator == (class_type const &rhs) const
    {
        return compare(rhs) == 0;
    }

    /// Indicates whether \c this and \c rhs are not equivalent
    bool operator != (class_type const &rhs) const
    {
        return compare(rhs) != 0;
    }

    // Bidirectional iterator operations

    /// Pre-decrement operator
    class_type &operator --()
    {
        --m_index;

        return *this;
    }

    /// Post-decrement operator
    class_type operator --(int)
    {
        class_type  ret(*this);

        operator --();

        return ret;
    }

    // Random access operations

    /// Offset
    class_type &operator +=(difference_type index)
    {
        m_index += index;

        return *this;
    }

    /// Offset
    class_type &operator -=(difference_type index)
    {
        m_index -= index;

        return *this;
    }

    /// Indexing operator
    value_type operator [](difference_type index) const
    {
        return _get_value_at(m_hwnd, index);
    }

    /// Calculate the distance between \c this and \c rhs
    difference_type distance(class_type const &rhs) const
    {
        return m_index - rhs.m_index;
    }

    /// Pointer subtraction
    class_type operator -(difference_type n) const
    {
        return class_type(*this) -= n;
    }

    /// Pointer addition
    class_type operator +(difference_type n) const
    {
        return class_type(*this) += n;
    }

    /// Pointer difference
    difference_type operator -(class_type const &rhs) const
    {
        return distance(rhs);
    }

    /// Indicates whether \c this is less than \c rhs
    ws_bool_t operator <(class_type const &rhs) const
    {
        return compare(rhs) < 0;
    }

    /// Indicates whether \c this is greater than \c rhs
    ws_bool_t operator >(class_type const &rhs) const
    {
        return compare(rhs) > 0;
    }

    /// Indicates whether \c this is less than or equal \c rhs
    ws_bool_t operator <=(class_type const &rhs) const
    {
        return compare(rhs) <= 0;
    }

    /// Indicates whether \c this is greater than or equal \c rhs
    ws_bool_t operator >=(class_type const &rhs) const
    {
        return compare(rhs) >= 0;
    }

// Implementation
private:
    static value_type _get_value_at(HWND hwnd, difference_type index)
    {
        int         len =   combobox_gettextlen(hwnd, index);
        buffer_type buffer(1 + len);

        combobox_gettext(hwnd, index, buffer);

        return value_type(buffer, buffer.size());
    }

// Members
private:
    HWND                    m_hwnd;
    int                     m_index;
    ss_mutable_k value_type m_value;
};

/// STL-like sequence for combo-box contents
///
/// This class presents an STL-like sequence interface to a combo-box
///
/// \param S The string type
template <ws_typename_param_k S>
class combobox_sequence
{
public:
    /// The current parameterisation of the type
    typedef combobox_sequence<S>                            class_type;
    /// The non-mutating (const) iterator type
    typedef combobox_sequence_const_iterator<S>             const_iterator;
    /// The character type
    typedef TCHAR                                           char_type;
    /// The value type
    typedef S                                               value_type;
    /// The size type
    typedef ws_size_t                                       size_type;
    /// The difference type
    typedef ws_ptrdiff_t                                    difference_type;
#if defined(__STLSOFT_CF_BIDIRECTIONAL_ITERATOR_SUPPORT)
    /// The non-mutating (const) reverse iterator type
    typedef stlsoft_ns_qual(const_reverse_iterator_base)<   const_iterator
                                                        ,   value_type
                                                        ,   value_type // Return by value!
                                                        ,   void*
                                                        ,   difference_type
                                                        >   const_reverse_iterator;
#endif /* __STLSOFT_CF_BIDIRECTIONAL_ITERATOR_SUPPORT */

protected:
    typedef processheap_allocator<char_type>            allocator_type;
    typedef auto_buffer<char_type, allocator_type, 256> buffer_type;

// Construction
public:
    /// Constructs from the given combo-box
    combobox_sequence(HWND hwndComboBox)
        : m_hwnd(hwndComboBox)
    {}

// State
public:
    /// Returns the number of elements in the combo-box
    size_type size() const
    {
        return static_cast<size_type>(combobox_getcount(m_hwnd));
    }
    /// Indicates whether the combo-box is empty
    ws_bool_t empty() const
    {
        return size() == 0;
    }
    /// Returns the maximum number of items that the combo-box can contain
    static size_type max_size()
    {
        return static_cast<size_type>(-1) / sizeof(LPCTSTR);
    }

// Iteration
public:
    /// Begins the iteration
    ///
    /// \return An iterator representing the start of the sequence
    const_iterator  begin() const
    {
        return const_iterator(m_hwnd, 0);
    }
    /// Ends the iteration
    ///
    /// \return An iterator representing the end of the sequence
    const_iterator  end() const
    {
        return const_iterator(m_hwnd, static_cast<int>(combobox_getcount(m_hwnd)));
    }

#if defined(__STLSOFT_CF_BIDIRECTIONAL_ITERATOR_SUPPORT)
    /// Begins the reverse iteration
    ///
    /// \return An iterator representing the start of the reverse sequence
    const_reverse_iterator rbegin() const
    {
        return const_reverse_iterator(end());
    }
    /// Ends the reverse iteration
    ///
    /// \return An iterator representing the end of the reverse sequence
    const_reverse_iterator rend() const
    {
        return const_reverse_iterator(begin());
    }
#endif /* __STLSOFT_CF_BIDIRECTIONAL_ITERATOR_SUPPORT */

// Accessors
public:
    /// Returns the item at the given index
    value_type operator [](difference_type index) const
    {
        int         len =   combobox_gettextlen(m_hwnd, index);
        buffer_type buffer(1 + len);

        combobox_gettext(m_hwnd, index, buffer);

        return value_type(buffer, buffer.size());
    }

// Members
private:
    HWND    m_hwnd;
};

/* ////////////////////////////////////////////////////////////////////////// */

/// @} // end of group winstl_control_library

/* ////////////////////////////////////////////////////////////////////////// */

#ifndef _WINSTL_NO_NAMESPACE
 #ifdef _STLSOFT_NO_NAMESPACE
} // namespace winstl
 #else
} // namespace winstl_project
} // namespace stlsoft
 #endif /* _STLSOFT_NO_NAMESPACE */
#endif /* !_WINSTL_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* _WINSTL_INCL_H_WINSTL_COMBOBOX_SEQUENCE */

/* ////////////////////////////////////////////////////////////////////////// */
