/* /////////////////////////////////////////////////////////////////////////////
 * File:        winstl_drophandle_sequence.h
 *
 * Purpose:     Contains the basic_drophandle_sequence template class, and ANSI
 *              and Unicode specialisations thereof.
 *
 * Created:     13th November 2002
 * Updated:     2nd July 2003
 *
 * Author:      Matthew Wilson, Synesis Software Pty Ltd.
 *
 * License:     (Licensed under the Synesis Software Standard Source License)
 *
 *              Copyright (C) 2002-2003, Synesis Software Pty Ltd.
 *
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/winstl
 *                          http://www.winstl.org/
 *
 *              email:      submissions@winstl.org  for submissions
 *                          admin@winstl.org        for other enquiries
 *
 *              Redistribution and use in source and binary forms, with or
 *              without modification, are permitted provided that the following
 *              conditions are met:
 *
 *              (i) Redistributions of source code must retain the above
 *              copyright notice and contact information, this list of
 *              conditions and the following disclaimer.
 *
 *              (ii) Any derived versions of this software (howsoever modified)
 *              remain the sole property of Synesis Software.
 *
 *              (iii) Any derived versions of this software (howsoever modified)
 *              remain subject to all these conditions.
 *
 *              (iv) Neither the name of Synesis Software nor the names of any
 *              subdivisions, employees or agents of Synesis Software, nor the
 *              names of any other contributors to this software may be used to
 *              endorse or promote products derived from this software without
 *              specific prior written permission.
 *
 *              This source code is provided by Synesis Software "as is" and any
 *              warranties, whether expressed or implied, including, but not
 *              limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software be liable for any direct, indirect,
 *              incidental, special, exemplary, or consequential damages
 *              (including, but not limited to, procurement of substitute goods
 *              or services; loss of use, data, or profits; or business
 *              interruption) however caused and on any theory of liability,
 *              whether in contract, strict liability, or tort (including
 *              negligence or otherwise) arising in any way out of the use of
 *              this software, even if advised of the possibility of such
 *              damage.
 *
 * ////////////////////////////////////////////////////////////////////////// */


#ifndef _WINSTL_INCL_H_WINSTL_DROPHANDLE_SEQUENCE
#define _WINSTL_INCL_H_WINSTL_DROPHANDLE_SEQUENCE

#ifndef __STLSOFT_DOCUMENTATION_SKIP_SECTION
#define _WINSTL_VER_H_WINSTL_DROPHANDLE_SEQUENCE_MAJOR    1
#define _WINSTL_VER_H_WINSTL_DROPHANDLE_SEQUENCE_MINOR    3
#define _WINSTL_VER_H_WINSTL_DROPHANDLE_SEQUENCE_REVISION 7
#define _WINSTL_VER_H_WINSTL_DROPHANDLE_SEQUENCE_EDIT     24
#endif /* !__STLSOFT_DOCUMENTATION_SKIP_SECTION */

/* /////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#ifndef _WINSTL_INCL_H_WINSTL
 #include "winstl.h"                        // Include the WinSTL root header
#endif /* !_WINSTL_INCL_H_WINSTL */
#ifndef _WINSTL_INCL_H_WINSTL_PROCESSHEAP_ALLOCATOR
 #include "winstl_processheap_allocator.h"  // winstl::processheap_allocator
#endif /* !_WINSTL_INCL_H_WINSTL_PROCESSHEAP_ALLOCATOR */
#ifndef _STLSOFT_INCL_H_STLSOFT_AUTO_BUFFER
 #include "stlsoft_auto_buffer.h"           // stlsoft::auto_buffer
#endif /* !_STLSOFT_INCL_H_STLSOFT_AUTO_BUFFER */
#ifndef _STLSOFT_INCL_H_STLSOFT_FRAME_STRING
 #include "stlsoft_frame_string.h"          // stlsoft::basic_frame_string
#endif /* !_STLSOFT_INCL_H_STLSOFT_FRAME_STRING */
#ifdef __STLSOFT_CF_FRAME_STRING_SHIM_NOT_SUPPORTED
 #ifndef _STLSOFT_INCL_H_STLSOFT_SIMPLE_STRING
  #include "stlsoft_simple_string.h"        // stlsoft::basic_simple_string
 #endif /* !_STLSOFT_INCL_H_STLSOFT_SIMPLE_STRING */
#endif /* __STLSOFT_CF_FRAME_STRING_SHIM_NOT_SUPPORTED */
#ifndef _STLSOFT_INCL_H_STLSOFT_ITERATOR
 #include "stlsoft_iterator.h"              // stlsoft::iterator, stlsoft::reverse_iterator
#endif /* _STLSOFT_INCL_H_STLSOFT_ITERATOR */

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#ifndef _WINSTL_NO_NAMESPACE
 #ifdef _STLSOFT_NO_NAMESPACE
/* There is no stlsoft namespace, so must define ::winstl */
namespace winstl
{
 #else
/* Define stlsoft::winstl_project */

namespace stlsoft
{

namespace winstl_project
{

 #endif /* _STLSOFT_NO_NAMESPACE */
#endif /* !_WINSTL_NO_NAMESPACE */

/* /////////////////////////////////////////////////////////////////////////////
 * Typedefs
 */

#ifdef __STLSOFT_DROPHANDLE_SEQUENCE_VALUE_TYPE
 #undef __STLSOFT_DROPHANDLE_SEQUENCE_VALUE_TYPE
#endif /* __STLSOFT_DROPHANDLE_SEQUENCE_VALUE_TYPE */

#ifdef __STLSOFT_CF_FRAME_STRING_SHIM_NOT_SUPPORTED
 #define __STLSOFT_DROPHANDLE_SEQUENCE_VALUE_TYPE(C)    stlsoft_ns_qual(basic_simple_string)<C, stlsoft_ns_qual(char_traits)<C>, processheap_allocator<C> >
#else
 #define __STLSOFT_DROPHANDLE_SEQUENCE_VALUE_TYPE(C)    stlsoft_ns_qual(basic_frame_string)<C, _MAX_PATH>
#endif /* __STLSOFT_CF_FRAME_STRING_SHIM_NOT_SUPPORTED */

/* /////////////////////////////////////////////////////////////////////////////
 * Forward declarations
 */

#ifndef __STLSOFT_DOCUMENTATION_SKIP_SECTION

template <ws_typename_param_k C, ws_typename_param_k T>
class basic_drophandle_sequence;

#endif /* __STLSOFT_DOCUMENTATION_SKIP_SECTION */

/* /////////////////////////////////////////////////////////////////////////////
 * Classes
 */

#ifdef __STLSOFT_DOCUMENTATION_SKIP_SECTION
/// Traits for accessing the correct drop-handle functions for a given character type
///
/// drophandle_sequence_traits is a traits class for determining the correct
/// drop-handle structures and functions for a given character type.
///
/// \param C The character type
template <ws_typename_param_k C>
struct drophandle_sequence_traits
{
    /// Queries the information from the given drop handle
    ///
    /// \param hdrop The drop handle
    /// \param index The index of the requested item
    /// \param buffer The buffer within which to write the results
    /// \param cchBuffer The size of \c buffer in characters
    static ws_uint_t drag_query_file(HDROP hdrop, ws_uint_t index, C *buffer, ws_uint_t cchBuffer);
};

#else

template <ws_typename_param_k C>
struct drophandle_sequence_traits;

#ifdef __STLSOFT_CF_TEMPLATE_SPECIALISATION_SYNTAX
template <>
#endif /* __STLSOFT_CF_TEMPLATE_SPECIALISATION_SYNTAX */

struct drophandle_sequence_traits<ws_char_a_t>
{
    static ws_uint_t drag_query_file(HDROP hdrop, ws_uint_t index, ws_char_a_t *buffer, ws_uint_t cchBuffer)
    {
        return DragQueryFileA(hdrop, index, buffer, cchBuffer);
    }
};

#ifdef __STLSOFT_CF_TEMPLATE_SPECIALISATION_SYNTAX
template <>
#endif /* __STLSOFT_CF_TEMPLATE_SPECIALISATION_SYNTAX */
struct drophandle_sequence_traits<ws_char_w_t>
{
    static ws_uint_t drag_query_file(HDROP hdrop, ws_uint_t index, ws_char_w_t *buffer, ws_uint_t cchBuffer)
    {
        return DragQueryFileW(hdrop, index, buffer, cchBuffer);
    }
};

#endif /* __STLSOFT_DOCUMENTATION_SKIP_SECTION */


// class basic_drophandle_sequence_const_iterator
/// Iterator for the basic_drophandle_sequence class
///
/// \param C The character type
/// \param T The traits type. For translators that support default template arguments this defaults to drophandle_sequence_traits<C>
template<   ws_typename_param_k C
#ifdef __STLSOFT_CF_TEMPLATE_CLASS_DEFAULT_CLASS_ARGUMENT_SUPPORT
        ,   ws_typename_param_k T = drophandle_sequence_traits<C>
#else
        ,   ws_typename_param_k T /* = drophandle_sequence_traits<C> */
#endif /* __STLSOFT_CF_TEMPLATE_CLASS_DEFAULT_CLASS_ARGUMENT_SUPPORT */
        >
class basic_drophandle_sequence_const_iterator
    : public stlsoft_ns_qual(iterator_base)<winstl_ns_qual_std(bidirectional_iterator_tag), __STLSOFT_DROPHANDLE_SEQUENCE_VALUE_TYPE(C), ws_ptrdiff_t, void, __STLSOFT_DROPHANDLE_SEQUENCE_VALUE_TYPE(C)>
{
public:
    /// The character type
    typedef C                                                   char_type;
    /// The traits type
    typedef T                                                   traits_type;
    /// The string type
    typedef __STLSOFT_DROPHANDLE_SEQUENCE_VALUE_TYPE(C)         string_type;
    /// The current parameterisation of the type
    typedef basic_drophandle_sequence_const_iterator<C, T>      class_type;
    /// The value type
    typedef string_type                                         value_type;
    /// The index type
    typedef ws_uint_t                                           index_type;

// Construction
protected:
    /// Create an iterator based on the given drop handle
    ws_explicit_k basic_drophandle_sequence_const_iterator(HDROP hdrop, index_type index);
public:
    // Default constructor
    basic_drophandle_sequence_const_iterator();
    /// Copy constructor
    basic_drophandle_sequence_const_iterator(class_type const &rhs);

    /// Copy assignment operator
    basic_drophandle_sequence_const_iterator &operator =(class_type const &rhs);

// Operators
public:
    /// Pre-increment operator
    class_type &operator ++();
    /// Pre-decrement operator
    class_type &operator --();
    /// Post-increment operator
    const class_type operator ++(int);
    /// Post-decrement operator
    const class_type operator --(int);
    /// Dereference
    const value_type operator *() const;
    /// Evaluates whether \c this and \c rhs are equivalent
    ws_bool_t operator ==(class_type const &rhs) const;
    /// Evaluates whether \c this and \c rhs are not equivalent
    ws_bool_t operator !=(class_type const &rhs) const;

// Implementation
private:
    static index_type _sentinel();

// Members
private:
    friend class basic_drophandle_sequence<C, T>;

    HDROP       m_hdrop;
    index_type  m_index;
};


// class basic_drophandle_sequence
/// This class wraps an STL-like sequence interface over a Win32 drop handle
///
/// \param C The character type
/// \param T The traits type. For translators that support default template arguments this defaults to drophandle_sequence_traits<C>
template<   ws_typename_param_k C
#ifdef __STLSOFT_CF_TEMPLATE_CLASS_DEFAULT_CLASS_ARGUMENT_SUPPORT
        ,   ws_typename_param_k T = drophandle_sequence_traits<C>
#else
        ,   ws_typename_param_k T /* = drophandle_sequence_traits<C> */
#endif /* __STLSOFT_CF_TEMPLATE_CLASS_DEFAULT_CLASS_ARGUMENT_SUPPORT */
        >
class basic_drophandle_sequence
{
public:
    /// The character type
    typedef C                                                                               char_type;
    /// The traits type
    typedef T                                                                               traits_type;
    /// The current parameterisation of the type
    typedef basic_drophandle_sequence<C, T>                                                 class_type;
    /// The non-mutating (const) iterator type
    typedef basic_drophandle_sequence_const_iterator<C, T>                                  const_iterator;
    /// The value type
    typedef ws_typename_type_k const_iterator::value_type                                   value_type;
    /// The difference type
    typedef ws_ptrdiff_t                                                                    difference_type;
    /// The non-mutating (const) reverse iterator type
#if defined(__STLSOFT_CF_BIDIRECTIONAL_ITERATOR_SUPPORT)
    typedef stlsoft_ns_qual(const_reverse_bidirectional_iterator_base)< const_iterator,
                                                                        value_type,
                                                                        value_type, // Return by value!
                                                                        void*,
                                                                        difference_type>    const_reverse_iterator;
#endif /* __STLSOFT_CF_BIDIRECTIONAL_ITERATOR_SUPPORT */

// Construction
public:
    /// Construct a sequence based on the contents of the given drop handle
    ws_explicit_k basic_drophandle_sequence(HDROP hdrop);
    /// Release any resources aquired
    ~basic_drophandle_sequence() winstl_throw_0();

// Attributes
public:
    /// The number of elements in the sequence
    ws_size_t size() const;
    /// Indicates whether the sequence is empty
    ws_bool_t empty() const;

// Iteration
public:
    /// Begins the iteration
    ///
    /// \return An iterator representing the start of the sequence
    const_iterator  begin() const;
    /// Ends the iteration
    ///
    /// \return An iterator representing the end of the sequence
    const_iterator  end() const;

#if defined(__STLSOFT_CF_BIDIRECTIONAL_ITERATOR_SUPPORT)
    /// Begins the reverse iteration
    ///
    /// \return An iterator representing the start of the reverse sequence
    const_reverse_iterator  rbegin() const;
    /// Ends the reverse iteration
    ///
    /// \return An iterator representing the end of the reverse sequence
    const_reverse_iterator  rend() const;
#endif /* __STLSOFT_CF_BIDIRECTIONAL_ITERATOR_SUPPORT */

// Members
private:
    HDROP   m_hdrop;

// Implementation
private:

// Not to be implemented
private:
    basic_drophandle_sequence(class_type const &);
    const basic_drophandle_sequence &operator =(class_type const &);
};

/* Typedefs to commonly encountered types. */
/// Instantiation of the basic_drophandle_sequence template for the ANSI character type \c char
typedef basic_drophandle_sequence<ws_char_a_t, drophandle_sequence_traits<ws_char_a_t> >  drophandle_sequence_a;
/// Instantiation of the basic_drophandle_sequence template for the Unicode character type \c wchar_t
typedef basic_drophandle_sequence<ws_char_w_t, drophandle_sequence_traits<ws_char_w_t> >  drophandle_sequence_w;

///////////////////////////////////////////////////////////////////////////////

// basic_drophandle_sequence_const_iterator

template<   ws_typename_param_k C
        ,   ws_typename_param_k T
        >
inline /* static */ ws_typename_type_k basic_drophandle_sequence_const_iterator<C, T>::index_type basic_drophandle_sequence_const_iterator<C, T>::_sentinel()
{
    return 0x7fffffff;
}

template<   ws_typename_param_k C
        ,   ws_typename_param_k T
        >
inline /* ws_explicit_k */ basic_drophandle_sequence_const_iterator<C, T>::basic_drophandle_sequence_const_iterator(HDROP hdrop, ss_typename_type_k basic_drophandle_sequence_const_iterator<C, T>::index_type index)
    : m_hdrop(hdrop)
    , m_index(index)
{}

template<   ws_typename_param_k C
        ,   ws_typename_param_k T
        >
inline basic_drophandle_sequence_const_iterator<C, T>::basic_drophandle_sequence_const_iterator()
    : m_hdrop(0)
    , m_index(_sentinel())
{}

template<   ws_typename_param_k C
        ,   ws_typename_param_k T
        >
inline basic_drophandle_sequence_const_iterator<C, T>::basic_drophandle_sequence_const_iterator(ss_typename_type_k basic_drophandle_sequence_const_iterator<C, T>::class_type const &rhs)
    : m_hdrop(rhs.m_hdrop)
    , m_index(rhs.m_index)
{}

template<   ws_typename_param_k C
        ,   ws_typename_param_k T
        >
inline ws_typename_type_k basic_drophandle_sequence_const_iterator<C, T>::class_type &basic_drophandle_sequence_const_iterator<C, T>::operator =(ss_typename_type_k basic_drophandle_sequence_const_iterator<C, T>::class_type const &rhs)
{
    m_hdrop = rhs.m_hdrop;
    m_index = rhs.m_index;

    return *this;
}

template<   ws_typename_param_k C
        ,   ws_typename_param_k T
        >
inline ws_typename_type_k basic_drophandle_sequence_const_iterator<C, T>::class_type &basic_drophandle_sequence_const_iterator<C, T>::operator ++()
{
    stlsoft_message_assert("Incrementing the end iterator", m_index != _sentinel());

    char_type   ch;
    ws_uint_t   res =   traits_type::drag_query_file(m_hdrop, static_cast<ws_uint_t>(m_index + 1), &ch, 1);

    if(res == 0)
    {
        // Failed, so become 'end'
        m_index = _sentinel();
    }
    else
    {
        // Fine, so increment to next
        ++m_index;
    }

    return *this;
}

template<   ws_typename_param_k C
        ,   ws_typename_param_k T
        >
inline ws_typename_type_k basic_drophandle_sequence_const_iterator<C, T>::class_type &basic_drophandle_sequence_const_iterator<C, T>::operator --()
{
    // If currently at 'end' ....
    if(m_index == _sentinel())
    {
        // ... then determine the new end ...
        m_index = static_cast<index_type>(traits_type::drag_query_file(m_hdrop, 0xFFFFFFFF, NULL, 0));

        if(m_index == 0xFFFFFFFF)
        {
            // ... failed. Must set to end again!
            m_index = _sentinel();
        }
        else
        {
            // ... decrement from end
            --m_index;
        }
    }
    else
    {
        // ... decrement from current position
        --m_index;
    }

    return *this;
}

template<   ws_typename_param_k C
        ,   ws_typename_param_k T
        >
inline const ws_typename_type_k basic_drophandle_sequence_const_iterator<C, T>::class_type basic_drophandle_sequence_const_iterator<C, T>::operator ++(int)
{
    class_type  ret(*this);

    operator ++();

    return ret;
}

template<   ws_typename_param_k C
        ,   ws_typename_param_k T
        >
inline const ws_typename_type_k basic_drophandle_sequence_const_iterator<C, T>::class_type basic_drophandle_sequence_const_iterator<C, T>::operator --(int)
{
    class_type  ret(*this);

    operator --();

    return ret;
}

template<   ws_typename_param_k C
        ,   ws_typename_param_k T
        >
inline const ws_typename_type_k basic_drophandle_sequence_const_iterator<C, T>::value_type basic_drophandle_sequence_const_iterator<C, T>::operator *() const
{
    stlsoft_message_assert("Dereferencing the end iterator", m_index != _sentinel());

    char_type   sz[_MAX_PATH + 1];
    ws_uint_t   res =   traits_type::drag_query_file(m_hdrop, static_cast<ws_uint_t>(m_index), sz, stlsoft_num_elements(sz));

    if(res == 0xFFFFFFFF)
    {
        sz[0] = '\0';
    }
        
    return value_type(sz);
}

template<   ws_typename_param_k C
        ,   ws_typename_param_k T
        >
inline ws_bool_t basic_drophandle_sequence_const_iterator<C, T>::operator ==(ss_typename_type_k basic_drophandle_sequence_const_iterator<C, T>::class_type const &rhs) const
{
    stlsoft_message_assert("Comparing iterators from different sequences", m_hdrop == NULL || rhs.m_hdrop == NULL || rhs.m_hdrop);

    return m_index == rhs.m_index;
}

template<   ws_typename_param_k C
        ,   ws_typename_param_k T
        >
inline ws_bool_t basic_drophandle_sequence_const_iterator<C, T>::operator !=(ss_typename_type_k basic_drophandle_sequence_const_iterator<C, T>::class_type const &rhs) const
{
    return !operator ==(rhs);
}

// basic_drophandle_sequence

template <ws_typename_param_k C, ws_typename_param_k T>
inline basic_drophandle_sequence<C, T>::basic_drophandle_sequence(HDROP hdrop)
    : m_hdrop(hdrop)
{}

template <ws_typename_param_k C, ws_typename_param_k T>
inline basic_drophandle_sequence<C, T>::~basic_drophandle_sequence() winstl_throw_0()
{
    ::DragFinish(m_hdrop);
}

template <ws_typename_param_k C, ws_typename_param_k T>
inline ws_size_t basic_drophandle_sequence<C, T>::size() const
{
    return traits_type::drag_query_file(m_hdrop, static_cast<ws_uint_t>(-1), 0, 0);
}

template <ws_typename_param_k C, ws_typename_param_k T>
inline ws_bool_t basic_drophandle_sequence<C, T>::empty() const
{
    return size() == 0;
}

template <ws_typename_param_k C, ws_typename_param_k T>
inline ws_typename_type_k basic_drophandle_sequence<C, T>::const_iterator basic_drophandle_sequence<C, T>::begin() const
{
    char_type   ch;
    ws_uint_t   res =   traits_type::drag_query_file(m_hdrop, 0, &ch, 1);

    return const_iterator(m_hdrop, (res == 0) ? const_iterator::_sentinel() : 0);
}

template <ws_typename_param_k C, ws_typename_param_k T>
inline ws_typename_type_k basic_drophandle_sequence<C, T>::const_iterator basic_drophandle_sequence<C, T>::end() const
{
    return const_iterator(m_hdrop, const_iterator::_sentinel());
}

#if defined(__STLSOFT_CF_BIDIRECTIONAL_ITERATOR_SUPPORT)
template <ws_typename_param_k C, ws_typename_param_k T>
inline ws_typename_type_k basic_drophandle_sequence<C, T>::const_reverse_iterator basic_drophandle_sequence<C, T>::rbegin() const
{
    return const_reverse_iterator(end());
}

template <ws_typename_param_k C, ws_typename_param_k T>
inline ws_typename_type_k basic_drophandle_sequence<C, T>::const_reverse_iterator basic_drophandle_sequence<C, T>::rend() const
{
    return const_reverse_iterator(begin());
}
#endif /* __STLSOFT_CF_BIDIRECTIONAL_ITERATOR_SUPPORT */


/* ////////////////////////////////////////////////////////////////////////// */

#ifndef _WINSTL_NO_NAMESPACE
 #ifdef _STLSOFT_NO_NAMESPACE
} // namespace winstl
 #else
} // namespace winstl_project
} // namespace stlsoft
 #endif /* _STLSOFT_NO_NAMESPACE */
#endif /* !_WINSTL_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* _WINSTL_INCL_H_WINSTL_DROPHANDLE_SEQUENCE */

/* ////////////////////////////////////////////////////////////////////////// */
