/* /////////////////////////////////////////////////////////////////////////////
 * File:        winstl_findvolume_sequence.h
 *
 * Purpose:     Contains the basic_findvolume_sequence template class, and ANSI
 *              and Unicode specialisations thereof.
 *
 * Notes:       The original implementation of the class had the const_iterator
 *              and value_type as nested classes. Unfortunately, Visual C++ 5 &
 *              6 both had either compilation or linking problems so these are
 *              regretably now implemented as independent classes.
 *
 * Created:     15th January 2002
 * Updated:     21st August 2003
 *
 * Author:      Matthew Wilson, Synesis Software Pty Ltd.
 *
 * License:     (Licensed under the Synesis Software Standard Source License)
 *
 *              Copyright (C) 2002-2003, Synesis Software Pty Ltd.
 *
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/winstl
 *                          http://www.winstl.org/
 *
 *              email:      submissions@winstl.org  for submissions
 *                          admin@winstl.org        for other enquiries
 *
 *              Redistribution and use in source and binary forms, with or
 *              without modification, are permitted provided that the following
 *              conditions are met:
 *
 *              (i) Redistributions of source code must retain the above
 *              copyright notice and contact information, this list of
 *              conditions and the following disclaimer.
 *
 *              (ii) Any derived versions of this software (howsoever modified)
 *              remain the sole property of Synesis Software.
 *
 *              (iii) Any derived versions of this software (howsoever modified)
 *              remain subject to all these conditions.
 *
 *              (iv) Neither the name of Synesis Software nor the names of any
 *              subdivisions, employees or agents of Synesis Software, nor the
 *              names of any other contributors to this software may be used to
 *              endorse or promote products derived from this software without
 *              specific prior written permission.
 *
 *              This source code is provided by Synesis Software "as is" and any
 *              warranties, whether expressed or implied, including, but not
 *              limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software be liable for any direct, indirect,
 *              incidental, special, exemplary, or consequential damages
 *              (including, but not limited to, procurement of substitute goods
 *              or services; loss of use, data, or profits; or business
 *              interruption) however caused and on any theory of liability,
 *              whether in contract, strict liability, or tort (including
 *              negligence or otherwise) arising in any way out of the use of
 *              this software, even if advised of the possibility of such
 *              damage.
 *
 * ////////////////////////////////////////////////////////////////////////// */


#ifndef _WINSTL_INCL_H_WINSTL_FINDVOLUME_SEQUENCE
#define _WINSTL_INCL_H_WINSTL_FINDVOLUME_SEQUENCE

#ifndef __STLSOFT_DOCUMENTATION_SKIP_SECTION
#define _WINSTL_VER_H_WINSTL_FINDVOLUME_SEQUENCE_MAJOR    1
#define _WINSTL_VER_H_WINSTL_FINDVOLUME_SEQUENCE_MINOR    8
#define _WINSTL_VER_H_WINSTL_FINDVOLUME_SEQUENCE_REVISION 1
#define _WINSTL_VER_H_WINSTL_FINDVOLUME_SEQUENCE_EDIT     41
#endif /* !__STLSOFT_DOCUMENTATION_SKIP_SECTION */

/* /////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#ifndef _WINSTL_INCL_H_WINSTL
 #include "winstl.h"                    // Include the WinSTL root header
#endif /* !_WINSTL_INCL_H_WINSTL */
#ifndef _WINSTL_INCL_H_WINSTL_FILESYSTEM_TRAITS
 #include "winstl_filesystem_traits.h"  // file_traits
#endif /* !_WINSTL_INCL_H_WINSTL_FILESYSTEM_TRAITS */
#ifndef _STLSOFT_INCL_H_STLSOFT_ITERATOR
 #include "stlsoft_iterator.h"
#endif /* !_STLSOFT_INCL_H_STLSOFT_ITERATOR */

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#ifndef _WINSTL_NO_NAMESPACE
 #ifdef _STLSOFT_NO_NAMESPACE
/* There is no stlsoft namespace, so must define ::winstl */
namespace winstl
{
 #else
/* Define stlsoft::winstl_project */

namespace stlsoft
{

namespace winstl_project
{

 #endif /* _STLSOFT_NO_NAMESPACE */
#endif /* !_WINSTL_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

/// \weakgroup libraries STLSoft Libraries
/// \brief The individual libraries

/// \weakgroup libraries_filesystem File-System Library
/// \ingroup libraries
/// \brief This library provides facilities for defining and manipulating file-system objects

/// \weakgroup winstl_filesystem_library File-System Library (WinSTL)
/// \ingroup WinSTL libraries_filesystem
/// \brief This library provides facilities for defining and manipulating file-system objects for the Win32 API
/// @{

/* /////////////////////////////////////////////////////////////////////////////
 * Enumerations
 */

// The FindNextVolume API is not well documented so assume _MAX_PATH
// is sufficient for volume names
enum
{
    MAX_VOL_NAME    =   _MAX_PATH   //!< The maximum number of characters in a volume name
};

/* /////////////////////////////////////////////////////////////////////////////
 * Forward declarations
 */

#ifndef __STLSOFT_DOCUMENTATION_SKIP_SECTION

template <ws_typename_param_k C, ws_typename_param_k T>
class basic_findvolume_sequence_value_type;

template <ws_typename_param_k C, ws_typename_param_k T, ws_typename_param_k V>
class basic_findvolume_sequence_const_iterator;

#endif /* !__STLSOFT_DOCUMENTATION_SKIP_SECTION */

/* /////////////////////////////////////////////////////////////////////////////
 * Classes
 */

// class basic_findvolume_sequence
/// Presents an STL-like sequence interface over the volumes on a system
///
/// \note This class functions only on Windows 2000 and later
///
/// \param C The character type
/// \param T The traits type. On translators that support default template arguments this defaults to filesystem_traits<C>
template<   ws_typename_param_k C
#ifdef __STLSOFT_CF_TEMPLATE_CLASS_DEFAULT_CLASS_ARGUMENT_SUPPORT
        ,   ws_typename_param_k T = filesystem_traits<C>
#else
        ,   ws_typename_param_k T /* = filesystem_traits<C> */
#endif /* __STLSOFT_CF_TEMPLATE_CLASS_DEFAULT_CLASS_ARGUMENT_SUPPORT */
        >
class basic_findvolume_sequence
{
public:
    /// The character type
    typedef C                                                           char_type;
    /// The traits type
    typedef T                                                           traits_type;
    /// The current parameterisation of the type
    typedef basic_findvolume_sequence<C, T>                             class_type;
    /// The value type
    typedef basic_findvolume_sequence_value_type<C, T>                  value_type;
    /// The non-mutating (const) iterator type
    typedef basic_findvolume_sequence_const_iterator<C, T, value_type>  const_iterator;
    /// The reference type
    typedef value_type                                                  &reference;
    /// The non-mutable (const) reference type
    typedef const value_type                                            &const_reference;

// Iteration
public:
    /// Begins the iteration
    ///
    /// \return An iterator representing the start of the sequence
    const_iterator  begin() const;
    /// Ends the iteration
    ///
    /// \return An iterator representing the end of the sequence
    const_iterator  end() const;

// State
public:
    /// Evalulates whether the sequence is empty
    ws_bool_t       empty() const;
};

/* /////////////////////////////////////////////////////////////////////////////
 * Typedefs for commonly encountered types
 */

/// Instantiation of the basic_findvolume_sequence template for the ANSI character type \c char
typedef basic_findvolume_sequence<ws_char_a_t, filesystem_traits<ws_char_a_t> >     findvolume_sequence_a;
/// Instantiation of the basic_findvolume_sequence template for the Unicode character type \c wchar_t
typedef basic_findvolume_sequence<ws_char_w_t, filesystem_traits<ws_char_w_t> >     findvolume_sequence_w;
/// Instantiation of the basic_findvolume_sequence template for the Win32 character type \c TCHAR
typedef basic_findvolume_sequence<TCHAR, filesystem_traits<TCHAR> >                 findvolume_sequence;

/* ////////////////////////////////////////////////////////////////////////// */

// class basic_findvolume_sequence_value_type
/// Value type for the basic_findvolume_sequence
template<   ws_typename_param_k C
        ,   ws_typename_param_k T
        >
class basic_findvolume_sequence_value_type
{
public:
    /// The character type
    typedef C                                           char_type;
    /// The traits type
    typedef T                                           traits_type;
    /// The current parameterisation of the type
    typedef basic_findvolume_sequence_value_type<C, T>  class_type;

public:
    /// Default constructor
    basic_findvolume_sequence_value_type();
    /// Copy constructor
    basic_findvolume_sequence_value_type(const class_type &rhs);
private:
    basic_findvolume_sequence_value_type(char_type const *vol_name);
public:

    /// Copy assignment operator
    basic_findvolume_sequence_value_type &operator =(const class_type &rhs);

// Attributes
public:
    /// Implicit conversion to non-mutable pointer-to-const of the volume name
    operator char_type const * () const
    {
        return m_name;
    }

private:
    friend class basic_findvolume_sequence_const_iterator<C, T, class_type>;

    char_type   m_name[MAX_VOL_NAME + 1];
};

// class basic_findvolume_sequence_const_iterator
/// Iterator type for the basic_findvolume_sequence, supporting the Input Iterator concept
template<   ws_typename_param_k C
        ,   ws_typename_param_k T
        ,   ws_typename_param_k V
        >
class basic_findvolume_sequence_const_iterator
    : public stlsoft_ns_qual(iterator_base)<winstl_ns_qual_std(input_iterator_tag), V, ws_ptrdiff_t, void, V>
{
public:
    /// The character type
    typedef C                                                   char_type;
    /// The traits type
    typedef T                                                   traits_type;
    /// The traits type
    typedef V                                                   value_type;
    /// The current parameterisation of the type
    typedef basic_findvolume_sequence_const_iterator<C, T, V>   class_type;

private:
    basic_findvolume_sequence_const_iterator(basic_findvolume_sequence<C, T> const &l, HANDLE hSrch, char_type const *vol_name)
        : m_list(&l)
        , m_hSrch(hSrch)
    {
        traits_type::str_copy(m_name, vol_name);
    }
    basic_findvolume_sequence_const_iterator(basic_findvolume_sequence<C, T> const &l);
public:
    /// Default constructor
    basic_findvolume_sequence_const_iterator();
    /// Copy constructor
    basic_findvolume_sequence_const_iterator(const class_type &rhs);
    /// Destructor
    ~basic_findvolume_sequence_const_iterator() winstl_throw_0();

    /// Copy assignment operator
    basic_findvolume_sequence_const_iterator &operator =(const class_type &rhs);

public:
    /// Pre-increment operator
    class_type &operator ++();
    /// Post-increment operator
    void operator ++(int);
    /// Dereference to access the value at the current position
    const value_type operator *() const;
    /// Evaluates whether \c this and \c rhs are equivalent
    ws_bool_t operator ==(const class_type &rhs) const;
    /// Evaluates whether \c this and \c rhs are not equivalent
    ws_bool_t operator !=(const class_type &rhs) const;

// Implementation
private:
    void swap(class_type &rhs);

// Members
private:
    friend class basic_findvolume_sequence<C, T>;

    typedef basic_findvolume_sequence<C, T>     list_type;

    list_type const * const m_list;
    HANDLE                  m_hSrch;
    char_type               m_name[MAX_VOL_NAME + 1];
};

///////////////////////////////////////////////////////////////////////////////

// basic_findvolume_sequence

template <ws_typename_param_k C, ws_typename_param_k T>
inline ws_typename_type_k basic_findvolume_sequence<C, T>::const_iterator basic_findvolume_sequence<C, T>::begin() const
{
    char_type   vol_name[MAX_VOL_NAME + 1];
    HANDLE      hSrch   =   traits_type::find_first_volume(vol_name, winstl_num_elements(vol_name));

    if(hSrch == INVALID_HANDLE_VALUE)
    {
        return const_iterator(*this);
    }
    else
    {
        return const_iterator(*this, hSrch, vol_name);
    }
}

template <ws_typename_param_k C, ws_typename_param_k T>
inline ws_typename_type_k basic_findvolume_sequence<C, T>::const_iterator basic_findvolume_sequence<C, T>::end() const
{
    return const_iterator(*this);
}

template <ws_typename_param_k C, ws_typename_param_k T>
inline ws_bool_t basic_findvolume_sequence<C, T>::empty() const
{
    return begin() == end();
}

// basic_findvolume_sequence_value_type

template <ws_typename_param_k C, ws_typename_param_k T>
inline basic_findvolume_sequence_value_type<C, T>::basic_findvolume_sequence_value_type()
{
    m_name[0] = '\0';
}

template <ws_typename_param_k C, ws_typename_param_k T>
inline basic_findvolume_sequence_value_type<C, T>::basic_findvolume_sequence_value_type(const class_type &rhs)
{
    traits_type::str_copy(m_name, rhs.m_name);
}

template <ws_typename_param_k C, ws_typename_param_k T>
inline basic_findvolume_sequence_value_type<C, T>::basic_findvolume_sequence_value_type(char_type const *vol_name)
{
    traits_type::str_copy(m_name, vol_name);
}

template <ws_typename_param_k C, ws_typename_param_k T>
inline ws_typename_type_k basic_findvolume_sequence_value_type<C, T>::class_type &basic_findvolume_sequence_value_type<C, T>::operator =(const class_type &rhs)
{
    traits_type::str_copy(m_name, rhs.m_name);

    return *this;
}


#if 0
template <ws_typename_param_k C, ws_typename_param_k T>
#ifdef __STLSOFT_CF_FUNCTION_SIGNATURE_FULL_ARG_QUALIFICATION_REQUIRED
inline basic_findvolume_sequence_value_type<C, T>::operator basic_findvolume_sequence<C, T>::char_type const *() const
#else
inline basic_findvolume_sequence_value_type<C, T>::operator char_type const *() const
#endif /* __STLSOFT_CF_FUNCTION_SIGNATURE_FULL_ARG_QUALIFICATION_REQUIRED */
{
    return m_name;
}
#endif /* 0 */


// basic_findvolume_sequence_const_iterator

template <ws_typename_param_k C, ws_typename_param_k T, ws_typename_param_k V>
inline void basic_findvolume_sequence_const_iterator<C, T, V>::swap(ss_typename_type_k basic_findvolume_sequence_const_iterator<C, T, V>::class_type &rhs)
{
    winstl_assert(m_list == rhs.m_list);

    HANDLE      t_h =   rhs.m_hSrch;
    rhs.m_hSrch     =   m_hSrch;
    m_hSrch         =   t_h;

    char_type   name_t[MAX_VOL_NAME + 1];

    traits_type::str_copy(name_t, rhs.m_name);
    traits_type::str_copy(rhs.m_name, m_name);
    traits_type::str_copy(m_name, name_t);
}

template <ws_typename_param_k C, ws_typename_param_k T, ws_typename_param_k V>
inline basic_findvolume_sequence_const_iterator<C, T, V>::basic_findvolume_sequence_const_iterator()
    : m_list(NULL)
    , m_hSrch(INVALID_HANDLE_VALUE)
{
    m_name[0] = '\0';
}

template <ws_typename_param_k C, ws_typename_param_k T, ws_typename_param_k V>
inline basic_findvolume_sequence_const_iterator<C, T, V>::basic_findvolume_sequence_const_iterator(basic_findvolume_sequence<C, T> const &l)
    : m_list(&l)
    , m_hSrch(INVALID_HANDLE_VALUE)
{
    m_name[0] = '\0';
}

template <ws_typename_param_k C, ws_typename_param_k T, ws_typename_param_k V>
inline basic_findvolume_sequence_const_iterator<C, T, V>::basic_findvolume_sequence_const_iterator(const class_type &rhs)
    : m_list(rhs.m_list)
    , m_hSrch(INVALID_HANDLE_VALUE)
{
    // Since the FindVolume API does not provide handle cloning a new search is instituted and then
    // the resulting iterator is advanced to the rhs and swapped into this instance
    class_type  rhs_end(*rhs.m_list);

    if(rhs == rhs_end)
    {
        swap(rhs_end);
    }
    else
    {
        class_type  b   =   rhs.m_list->begin();

        while(b != rhs)
        {
            ++b;
        }

        swap(b);
    }
}

template <ws_typename_param_k C, ws_typename_param_k T, ws_typename_param_k V>
inline ws_typename_type_k basic_findvolume_sequence_const_iterator<C, T, V>::class_type &basic_findvolume_sequence_const_iterator<C, T, V>::operator =(ss_typename_type_k basic_findvolume_sequence_const_iterator<C, T, V>::class_type const &rhs)
{
    // Since the FindVolume API does not provide handle cloning a new search is instituted and then
    // the resulting iterator is advanced to the rhs and swapped into this instance
    class_type  rhs_end(*rhs.m_list);

    if(rhs == rhs_end)
    {
        const_cast<list_type const *&>(m_list) = rhs.m_list;

        swap(rhs_end);
    }
    else
    {
        class_type  b   =   rhs.m_list->begin();

        while(b != rhs)
        {
            ++b;
        }

        const_cast<list_type const *&>(m_list) = rhs.m_list;

        swap(b);
    }

    return *this;
}

template <ws_typename_param_k C, ws_typename_param_k T, ws_typename_param_k V>
inline basic_findvolume_sequence_const_iterator<C, T, V>::~basic_findvolume_sequence_const_iterator() winstl_throw_0()
{
    if(m_hSrch != INVALID_HANDLE_VALUE)
    {
        ::FindVolumeClose(m_hSrch);
    }
}

template <ws_typename_param_k C, ws_typename_param_k T, ws_typename_param_k V>
inline ws_typename_type_k basic_findvolume_sequence_const_iterator<C, T, V>::class_type &basic_findvolume_sequence_const_iterator<C, T, V>::operator ++()
{
    if(!traits_type::find_next_volume(m_hSrch, m_name, winstl_num_elements(m_name)))
    {
        ::FindVolumeClose(m_hSrch);

        m_hSrch = INVALID_HANDLE_VALUE;
    }

    return *this;
}

template <ws_typename_param_k C, ws_typename_param_k T, ws_typename_param_k V>
inline void basic_findvolume_sequence_const_iterator<C, T, V>::operator ++(int)
{
    operator ++();
}

template <ws_typename_param_k C, ws_typename_param_k T, ws_typename_param_k V>
inline const ws_typename_type_k basic_findvolume_sequence_const_iterator<C, T, V>::value_type basic_findvolume_sequence_const_iterator<C, T, V>::operator *() const
{
    if(m_hSrch != INVALID_HANDLE_VALUE)
    {
        return value_type(m_name);
    }
    else
    {
        return value_type();
    }
}

template <ws_typename_param_k C, ws_typename_param_k T, ws_typename_param_k V>
inline ws_bool_t basic_findvolume_sequence_const_iterator<C, T, V>::operator ==(class_type const &rhs) const
{
    ws_bool_t    eq;

    winstl_assert(m_list == rhs.m_list);    // Should only be comparing iterators from same container

    // Not equal if different lists, or if one but not both handles is the INVALID_HANDLE_VALUE
    // or if the data is not equal.
    if( m_list != rhs.m_list ||
        (m_hSrch == INVALID_HANDLE_VALUE) != (rhs.m_hSrch == INVALID_HANDLE_VALUE) ||
        (   m_hSrch != INVALID_HANDLE_VALUE &&
            traits_type::str_compare(m_name, rhs.m_name) != 0))
    {
        eq = ws_false_v;
    }
    else
    {
        eq = ws_true_v;
    }

    return eq;
}

template <ws_typename_param_k C, ws_typename_param_k T, ws_typename_param_k V>
inline ws_bool_t basic_findvolume_sequence_const_iterator<C, T, V>::operator !=(class_type const &rhs) const
{
    return ! operator ==(rhs);
}

/* ////////////////////////////////////////////////////////////////////////// */

/// @} // end of group winstl_filesystem_library

/* ////////////////////////////////////////////////////////////////////////// */

#ifndef _WINSTL_NO_NAMESPACE
 #ifdef _STLSOFT_NO_NAMESPACE
} // namespace winstl
 #else
} // namespace winstl_project
} // namespace stlsoft
 #endif /* _STLSOFT_NO_NAMESPACE */
#endif /* !_WINSTL_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* _WINSTL_INCL_H_WINSTL_FINDVOLUME_SEQUENCE */

/* ////////////////////////////////////////////////////////////////////////// */
