/* /////////////////////////////////////////////////////////////////////////////
 * File:        winstl_module_filename.h (originally MWModFNm.h; ::SynesisWin)
 *
 * Purpose:     Preposterously simple class that gets, and makes accessible, the
 *              module filename.
 *
 * Created:     31st March 2002
 * Updated:     2nd July 2003
 *
 * Author:      Matthew Wilson, Synesis Software Pty Ltd.
 *
 * License:     (Licensed under the Synesis Software Standard Source License)
 *
 *              Copyright (C) 2002-2003, Synesis Software Pty Ltd.
 *
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/winstl
 *                          http://www.winstl.org/
 *
 *              email:      submissions@winstl.org  for submissions
 *                          admin@winstl.org        for other enquiries
 *
 *              Redistribution and use in source and binary forms, with or
 *              without modification, are permitted provided that the following
 *              conditions are met:
 *
 *              (i) Redistributions of source code must retain the above
 *              copyright notice and contact information, this list of
 *              conditions and the following disclaimer.
 *
 *              (ii) Any derived versions of this software (howsoever modified)
 *              remain the sole property of Synesis Software.
 *
 *              (iii) Any derived versions of this software (howsoever modified)
 *              remain subject to all these conditions.
 *
 *              (iv) Neither the name of Synesis Software nor the names of any
 *              subdivisions, employees or agents of Synesis Software, nor the
 *              names of any other contributors to this software may be used to
 *              endorse or promote products derived from this software without
 *              specific prior written permission.
 *
 *              This source code is provided by Synesis Software "as is" and any
 *              warranties, whether expressed or implied, including, but not
 *              limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software be liable for any direct, indirect,
 *              incidental, special, exemplary, or consequential damages
 *              (including, but not limited to, procurement of substitute goods
 *              or services; loss of use, data, or profits; or business
 *              interruption) however caused and on any theory of liability,
 *              whether in contract, strict liability, or tort (including
 *              negligence or otherwise) arising in any way out of the use of
 *              this software, even if advised of the possibility of such
 *              damage.
 *
 * ////////////////////////////////////////////////////////////////////////// */


#ifndef _WINSTL_INCL_H_WINSTL_MODULE_FILENAME
#define _WINSTL_INCL_H_WINSTL_MODULE_FILENAME

#ifndef __STLSOFT_DOCUMENTATION_SKIP_SECTION
#define _WINSTL_VER_H_WINSTL_MODULE_FILENAME_MAJOR      1
#define _WINSTL_VER_H_WINSTL_MODULE_FILENAME_MINOR      6
#define _WINSTL_VER_H_WINSTL_MODULE_FILENAME_REVISION   3
#define _WINSTL_VER_H_WINSTL_MODULE_FILENAME_EDIT       24
#endif /* !__STLSOFT_DOCUMENTATION_SKIP_SECTION */

/* /////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#ifndef _WINSTL_INCL_H_WINSTL
 #include "winstl.h"                // Include the WinSTL root header
#endif /* !_WINSTL_INCL_H_WINSTL */
#ifndef _WINSTL_INCL_H_WINSTL_FILESYSTEM_TRAITS
 #include "winstl_filesystem_traits.h"  // file_traits
#endif /* !_WINSTL_INCL_H_WINSTL_FILESYSTEM_TRAITS */

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#ifndef _WINSTL_NO_NAMESPACE
 #ifdef _STLSOFT_NO_NAMESPACE
/* There is no stlsoft namespace, so must define ::winstl */
namespace winstl
{
 #else
/* Define stlsoft::winstl_project */

namespace stlsoft
{

namespace winstl_project
{

 #endif /* _STLSOFT_NO_NAMESPACE */
#endif /* !_WINSTL_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

/// \weakgroup libraries STLSoft Libraries
/// \brief The individual libraries

/// \weakgroup libraries_filesystem File-System Library
/// \ingroup libraries
/// \brief This library provides facilities for defining and manipulating file-system objects

/// \weakgroup winstl_filesystem_library File-System Library (WinSTL)
/// \ingroup WinSTL libraries_filesystem
/// \brief This library provides facilities for defining and manipulating file-system objects for the Win32 API
/// @{

/* /////////////////////////////////////////////////////////////////////////////
 * basic_module_filename
 *
 * This class wraps the GetModuleFilename() API function, and effectively acts
 * as an adaptor between HINSTANCE and a C-string to the name.
 */

/// Represents a module filename
///
/// \param C The character type
/// \param T The traits type. On translators that support default template arguments, this defaults to filesystem_traits<C>
template<   ws_typename_param_k C
#ifdef __STLSOFT_CF_TEMPLATE_CLASS_DEFAULT_CLASS_ARGUMENT_SUPPORT
        ,   ws_typename_param_k T = filesystem_traits<C>
#else
        ,   ws_typename_param_k T /* = filesystem_traits<C> */
#endif /* __STLSOFT_CF_TEMPLATE_CLASS_DEFAULT_CLASS_ARGUMENT_SUPPORT */
        >
class basic_module_filename
{
public:
    /// The char type
    typedef C                           char_type;
    /// The traits type
    typedef T                           traits_type;
    /// The current parameterisation of the type
    typedef basic_module_filename<C, T> class_type;
    /// The size type
    typedef ws_size_t                   size_type;

// Construction
public:
    /// Default constructor - represent the filename of the current process
    basic_module_filename();
    /// Represent the filename of the given instance
    ws_explicit_k basic_module_filename(HINSTANCE hinst);

// Operations
public:
    /// Copy the module filename to the given buffer
    size_type           get_filename(ws_char_a_t *buffer, size_type cchBuffer) const;
    /// Copy the module filename to the given buffer
    size_type           get_filename(ws_char_w_t *buffer, size_type cchBuffer) const;
    /// Copy the module filename to the given buffer
    static size_type    get_filename(HINSTANCE hinst, ws_char_a_t *buffer, size_type cchBuffer);
    /// Copy the module filename to the given buffer
    static size_type    get_filename(HINSTANCE hinst, ws_char_w_t *buffer, size_type cchBuffer);

// Attributes
public:
    /// Returns a non-mutable (const) pointer to the path
    char_type const *get_filename() const;
    /// Returns the length of the converted path
    size_type       length() const;

// Conversions
public:
    /// Implicit conversion to a non-mutable (const) pointer to the path
    operator char_type const *() const
    {
        return get_filename();
    }

// Members
private:
    char_type       m_path[1 + _MAX_PATH];
    HINSTANCE const m_hinst;
    size_type const m_len;

// Not to be implemented
private:
    basic_module_filename(class_type const &);
    basic_module_filename &operator =(class_type const &);
};

/* /////////////////////////////////////////////////////////////////////////////
 * Typedefs for commonly encountered types
 */

/// Instantiation of the basic_module_filename template for the ANSI character type \c char
typedef basic_module_filename<ws_char_a_t, filesystem_traits<ws_char_a_t> >     module_filename_a;
/// Instantiation of the basic_module_filename template for the Unicode character type \c wchar_t
typedef basic_module_filename<ws_char_w_t, filesystem_traits<ws_char_w_t> >     module_filename_w;
/// Instantiation of the basic_module_filename template for the Win32 character type \c TCHAR
typedef basic_module_filename<TCHAR, filesystem_traits<TCHAR> >                 module_filename;

/* //////////////////////////////////////////////////////////////////////////
 * Implementation
 */

#ifndef __STLSOFT_DOCUMENTATION_SKIP_SECTION

template<   ws_typename_param_k C
        ,   ws_typename_param_k T
        >
inline basic_module_filename<C, T>::basic_module_filename()
    : m_hinst(::GetModuleHandle(NULL))
    , m_len(get_filename(m_path, stlsoft_num_elements(m_path)))
{
    stlsoft_static_assert(stlsoft_raw_offsetof(class_type, m_hinst) < stlsoft_raw_offsetof(class_type, m_len));
}

template<   ws_typename_param_k C
        ,   ws_typename_param_k T
        >
inline basic_module_filename<C, T>::basic_module_filename(HINSTANCE hinst)
    : m_hinst(hinst)
    , m_len(get_filename(hinst, m_path, stlsoft_num_elements(m_path)))
{
    stlsoft_static_assert(stlsoft_raw_offsetof(class_type, m_hinst) < stlsoft_raw_offsetof(class_type, m_len));
}

template<   ws_typename_param_k C
        ,   ws_typename_param_k T
        >
inline ss_typename_type_k basic_module_filename<C, T>::size_type basic_module_filename<C, T>::get_filename(ws_char_a_t *buffer, ss_typename_type_k basic_module_filename<C, T>::size_type cchBuffer) const
{
    return static_cast<size_type>(::GetModuleFileNameA(m_hinst, buffer, cchBuffer));
}

template<   ws_typename_param_k C
        ,   ws_typename_param_k T
        >
inline ss_typename_type_k basic_module_filename<C, T>::size_type basic_module_filename<C, T>::get_filename(ws_char_w_t *buffer, ss_typename_type_k basic_module_filename<C, T>::size_type cchBuffer) const
{
    return static_cast<size_type>(::GetModuleFileNameW(m_hinst, buffer, cchBuffer));
}

template<   ws_typename_param_k C
        ,   ws_typename_param_k T
        >
inline /* static */ ss_typename_type_k basic_module_filename<C, T>::size_type basic_module_filename<C, T>::get_filename(HINSTANCE hinst, ws_char_a_t *buffer, ss_typename_type_k basic_module_filename<C, T>::size_type cchBuffer)
{
    return static_cast<size_type>(::GetModuleFileNameA(hinst, buffer, cchBuffer));
}

template<   ws_typename_param_k C
        ,   ws_typename_param_k T
        >
inline /* static */ ss_typename_type_k basic_module_filename<C, T>::size_type basic_module_filename<C, T>::get_filename(HINSTANCE hinst, ws_char_w_t *buffer, ss_typename_type_k basic_module_filename<C, T>::size_type cchBuffer)
{
    return static_cast<size_type>(::GetModuleFileNameW(hinst, buffer, cchBuffer));
}

template<   ws_typename_param_k C
        ,   ws_typename_param_k T
        >
inline ss_typename_type_k basic_module_filename<C, T>::char_type const *basic_module_filename<C, T>::get_filename() const
{
    return m_path;
}

template<   ws_typename_param_k C
        ,   ws_typename_param_k T
        >
inline ss_typename_type_k basic_module_filename<C, T>::size_type basic_module_filename<C, T>::length() const
{
    return m_len;
}

#endif /* !__STLSOFT_DOCUMENTATION_SKIP_SECTION */

/* /////////////////////////////////////////////////////////////////////////////
 * String access shims
 */

#ifndef __STLSOFT_DOCUMENTATION_SKIP_SECTION

template<   ws_typename_param_k C
        ,   ws_typename_param_k T
        >
inline ws_char_a_t const *c_str_ptr_a_null(basic_module_filename<C, T> const &mfn)
{
    return mfn; // Can never be empty, so no need to cater for NULL return
}

template<   ws_typename_param_k C
        ,   ws_typename_param_k T
        >
inline ws_char_w_t const *c_str_ptr_w_null(basic_module_filename<C, T> const &mfn)
{
    return mfn; // Can never be empty, so no need to cater for NULL return
}

template<   ws_typename_param_k C
        ,   ws_typename_param_k T
        >
inline ws_char_a_t const *c_str_ptr_a(basic_module_filename<C, T> const &mfn)
{
    return mfn;
}

template<   ws_typename_param_k C
        ,   ws_typename_param_k T
        >
inline ws_char_w_t const *c_str_ptr_w(basic_module_filename<C, T> const &mfn)
{
    return mfn;
}

template<   ws_typename_param_k C
        ,   ws_typename_param_k T
        >
inline ss_typename_type_k basic_module_filename<C, T>::size_type c_str_len_a(basic_module_filename<C, T> const &mfn)
{
    return mfn.length();
}

template<   ws_typename_param_k C
        ,   ws_typename_param_k T
        >
inline ss_typename_type_k basic_module_filename<C, T>::size_type c_str_size(basic_module_filename<C, T> const &mfn)
{
    return mfn.length() * sizeof(ss_typename_type_k basic_module_filename<C, T>::char_type);
}

#endif /* !__STLSOFT_DOCUMENTATION_SKIP_SECTION */

/* ////////////////////////////////////////////////////////////////////////// */

/// @} // end of group winstl_filesystem_library

/* ////////////////////////////////////////////////////////////////////////// */

#ifndef _WINSTL_NO_NAMESPACE
 #ifdef _STLSOFT_NO_NAMESPACE
} // namespace winstl
 #else
} // namespace winstl_project
} // namespace stlsoft
 #endif /* _STLSOFT_NO_NAMESPACE */
#endif /* !_WINSTL_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* _WINSTL_INCL_H_WINSTL_MODULE_FILENAME */

/* ////////////////////////////////////////////////////////////////////////// */
