/* /////////////////////////////////////////////////////////////////////////////
 * File:        winstl_reg_key.h
 *
 * Purpose:     Contains the basic_reg_key template class, and ANSI
 *              and Unicode specialisations thereof.
 *
 * Created:     19th January 2002
 * Updated:     22nd August 2003
 *
 * Author:      Matthew Wilson, Synesis Software Pty Ltd.
 *
 * License:     (Licensed under the Synesis Software Standard Source License)
 *
 *              Copyright (C) 2002-2003, Synesis Software Pty Ltd.
 *
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/winstl
 *                          http://www.winstl.org/
 *
 *              email:      submissions@winstl.org  for submissions
 *                          admin@winstl.org        for other enquiries
 *
 *              Redistribution and use in source and binary forms, with or
 *              without modification, are permitted provided that the following
 *              conditions are met:
 *
 *              (i) Redistributions of source code must retain the above
 *              copyright notice and contact information, this list of
 *              conditions and the following disclaimer.
 *
 *              (ii) Any derived versions of this software (howsoever modified)
 *              remain the sole property of Synesis Software.
 *
 *              (iii) Any derived versions of this software (howsoever modified)
 *              remain subject to all these conditions.
 *
 *              (iv) Neither the name of Synesis Software nor the names of any
 *              subdivisions, employees or agents of Synesis Software, nor the
 *              names of any other contributors to this software may be used to
 *              endorse or promote products derived from this software without
 *              specific prior written permission.
 *
 *              This source code is provided by Synesis Software "as is" and any
 *              warranties, whether expressed or implied, including, but not
 *              limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software be liable for any direct, indirect,
 *              incidental, special, exemplary, or consequential damages
 *              (including, but not limited to, procurement of substitute goods
 *              or services; loss of use, data, or profits; or business
 *              interruption) however caused and on any theory of liability,
 *              whether in contract, strict liability, or tort (including
 *              negligence or otherwise) arising in any way out of the use of
 *              this software, even if advised of the possibility of such
 *              damage.
 *
 * ////////////////////////////////////////////////////////////////////////// */


#ifndef _WINSTL_INCL_H_WINSTL_REG_KEY
#define _WINSTL_INCL_H_WINSTL_REG_KEY

#ifndef __STLSOFT_DOCUMENTATION_SKIP_SECTION
#define _WINSTL_VER_H_WINSTL_REG_KEY_MAJOR      1
#define _WINSTL_VER_H_WINSTL_REG_KEY_MINOR      9
#define _WINSTL_VER_H_WINSTL_REG_KEY_REVISION   3
#define _WINSTL_VER_H_WINSTL_REG_KEY_EDIT       43
#endif /* !__STLSOFT_DOCUMENTATION_SKIP_SECTION */

/* /////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#ifndef _WINSTL_INCL_H_WINSTL
 #include "winstl.h"                        // Include the WinSTL root header
#endif /* !_WINSTL_INCL_H_WINSTL */
#ifndef _WINSTL_INCL_H_WINSTL_REG_DEFS
 #include "winstl_reg_defs.h"               // The WinSTL reg API standard types
#endif /* _WINSTL_INCL_H_WINSTL_REG_DEFS */
#ifndef _WINSTL_INCL_H_WINSTL_REG_TRAITS
 #include "winstl_reg_traits.h"             // The WinSTL reg API reg_traits class
#endif /* _WINSTL_INCL_H_WINSTL_REG_TRAITS */
#ifndef _STLSOFT_INCL_H_STLSOFT_AUTO_DESTRUCTOR
 #include "stlsoft_auto_buffer.h"           // Include the STLSoft auto_buffer
#endif /* _STLSOFT_INCL_H_STLSOFT_AUTO_DESTRUCTOR */
#ifndef _WINSTL_INCL_H_WINSTL_AUTO_DESTRUCTOR
 #include "winstl_processheap_allocator.h"  // winstl::processheap_allocator
#endif /* _WINSTL_INCL_H_WINSTL_AUTO_DESTRUCTOR */
#ifndef _STLSOFT_INCL_H_STLSOFT_STRING_ACCESS
 #include "stlsoft_string_access.h"         // stlsoft::c_str_ptr
#endif /* !_STLSOFT_INCL_H_STLSOFT_STRING_ACCESS */

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#ifndef _WINSTL_NO_NAMESPACE
 #ifdef _STLSOFT_NO_NAMESPACE
/* There is no stlsoft namespace, so must define ::winstl */
namespace winstl
{
 #else
/* Define stlsoft::winstl_project */

namespace stlsoft
{

namespace winstl_project
{

 #endif /* _STLSOFT_NO_NAMESPACE */
#endif /* !_WINSTL_NO_NAMESPACE */

stlsoft_ns_using(c_str_ptr)

/* ////////////////////////////////////////////////////////////////////////// */

/// \weakgroup winstl_reg_library Registry Library
/// @{

/* /////////////////////////////////////////////////////////////////////////////
 * Forward declarations
 */

#ifndef __STLSOFT_DOCUMENTATION_SKIP_SECTION

template <ws_typename_param_k C, ws_typename_param_k T, ws_typename_param_k A>
class basic_reg_value_sequence;

template <ws_typename_param_k C, ws_typename_param_k T, ws_typename_param_k A>
class basic_reg_key_sequence;

#endif /* !__STLSOFT_DOCUMENTATION_SKIP_SECTION */

/* /////////////////////////////////////////////////////////////////////////////
 * Classes
 */

/// Represents a registry key
///
/// This class acts as the value type of classes that manipulate registry keys
/// and encapsulates the concept of a registry key.
///
/// \param C The character type
/// \param T The traits type. On translators that support default template arguments this defaults to reg_traits<C>
/// \param A The allocator type. On translators that support default template arguments this defaults to processheap_allocator<C>
template<   ws_typename_param_k C
#ifdef __STLSOFT_CF_TEMPLATE_CLASS_DEFAULT_CLASS_ARGUMENT_SUPPORT
        ,   ws_typename_param_k T = reg_traits<C>
        ,   ws_typename_param_k A = processheap_allocator<C>
#else
        ,   ws_typename_param_k T /* = reg_traits<C> */
        ,   ws_typename_param_k A /* = processheap_allocator<C> */
#endif /* __STLSOFT_CF_TEMPLATE_CLASS_DEFAULT_CLASS_ARGUMENT_SUPPORT */
        >
class basic_reg_key
{
public:
    /// The character type
    typedef C                                           char_type;
    /// The traits type
    typedef T                                           traits_type;
    /// The allocator type
    typedef A                                           allocator_type;
    /// The current parameterisation of the type
    typedef basic_reg_key<C, T, A>                      class_type;
    /// The size type
    typedef ws_typename_type_k traits_type::size_type   size_type;
    /// The string type
    typedef ws_typename_type_k traits_type::string_type string_type;
    /// The key type
#if defined(__STLSOFT_COMPILER_IS_MSVC) && \
    _MSC_VER == 1100
    /* WSCB: VC5 has an internal compiler error if use traits_type::hkey_type */
    typedef HKEY                                        hkey_type;
#else
    typedef ws_typename_type_k traits_type::hkey_type   hkey_type;
#endif /* 0 */

// Construction
public:
    /// Default constructor
    basic_reg_key();
    /// Construct from the named sub-key of the given parent
    basic_reg_key(hkey_type hkeyParent, string_type const &key_name, REGSAM samDesired = KEY_READ)
        : m_hkey(_open_key(hkeyParent, c_str_ptr(key_name), samDesired))
        , m_name(key_name)
    {} // Implementation is within class, otherwise VC5 will not link
    /// Construct from the named sub-key of the given parent
    basic_reg_key(class_type const &keyParent, string_type const &key_name, REGSAM samDesired = KEY_READ)
        : m_hkey(_open_key(keyParent.get_key_handle(), c_str_ptr(key_name), samDesired))
        , m_name(key_name)
    {} // Implementation is within class, otherwise VC5 will not link

    /// Copy constructor
    basic_reg_key(class_type const &rhs);
    /// Copy constructor
    ///
    /// \param samDesired a change of permissions
    basic_reg_key(class_type const &rhs, REGSAM samDesired);

protected:
    basic_reg_key(hkey_type *hkey, string_type const &key_name);
public:
    /// Destructor
    ~basic_reg_key() winstl_throw_0();

    /// Copy assignment operator
    class_type &operator =(class_type const &rhs);

// Attributes
public:
    /// The name of the key
    string_type name() const;
    /// The registry class of the key
    string_type reg_class() const;
    /// The number of sub-keys
    ws_uint_t   num_sub_keys() const;
    /// The number of values
    ws_uint_t   num_values() const;

    /// The key handle
    hkey_type   get_key_handle() const;

// Operations
public:
    /// Opens the named sub-key of this key
    class_type  open_sub_key(char_type const *sub_key_name, REGSAM samDesired = KEY_READ);
#ifdef __STLSOFT_CF_MEMBER_TEMPLATE_FUNCTION_SUPPORT
    template <ws_typename_param_k S>
    class_type  open_sub_key(S const &sub_key_name, REGSAM samDesired = KEY_READ)
    {
        return open_sub_key(c_str_ptr(sub_key_name), samDesired);
    }
#endif /* __STLSOFT_CF_MEMBER_TEMPLATE_FUNCTION_SUPPORT */

    /// Creates a named sub-key of this key
    class_type  create_sub_key(char_type const *sub_key_name, REGSAM samDesired = KEY_READ);
#ifdef __STLSOFT_CF_MEMBER_TEMPLATE_FUNCTION_SUPPORT
    template <ws_typename_param_k S>
    class_type  create_sub_key(S const &sub_key_name, REGSAM samDesired = KEY_READ)
    {
        return create_sub_key(c_str_ptr(sub_key_name), samDesired);
    }
#endif /* __STLSOFT_CF_MEMBER_TEMPLATE_FUNCTION_SUPPORT */

    /* The handle returned from this method MUST be closed with RegCloseKey() */
    hkey_type   dup_key_handle(REGSAM samDesired = KEY_ALL_ACCESS);

// Implementation
private:
    static hkey_type    _open_key(hkey_type hkeyParent, char_type const *key_name, REGSAM samDesired);
    void                _swap(class_type &rhs) winstl_throw_0();

// Members
private:
    friend  class basic_reg_value_sequence<C, T, A>;
    friend  class basic_reg_key_sequence<C, T, A>;

    hkey_type   m_hkey; // The parent key
    string_type m_name; // The name
};

/* Typedefs to commonly encountered types. */
/// Instantiation of the basic_reg_key template for the ANSI character type \c char
typedef basic_reg_key<ws_char_a_t, reg_traits<ws_char_a_t>, processheap_allocator<ws_char_a_t> >  reg_key_a;
/// Instantiation of the basic_reg_key template for the Unicode character type \c wchar_t
typedef basic_reg_key<ws_char_w_t, reg_traits<ws_char_w_t>, processheap_allocator<ws_char_w_t> >  reg_key_w;

/* ////////////////////////////////////////////////////////////////////////////
 * Handle access shims
 */

/* get_handle */

/// \brief Returns the corresponding registry handle of \c k
template<   ws_typename_param_k C
        ,   ws_typename_param_k T
        ,   ws_typename_param_k A
        >
inline HKEY get_handle(basic_reg_key<C, T, A> const &key)
{
    return key.get_key_handle();
}

/* ////////////////////////////////////////////////////////////////////////// */

template<   ws_typename_param_k C
        ,   ws_typename_param_k T
        ,   ws_typename_param_k A
        >
#if defined(__STLSOFT_COMPILER_IS_MSVC) && \
    _MSC_VER < 1100
inline /* static */ ws_typename_type_k basic_reg_key<C, T, A>::hkey_type basic_reg_key<C, T, A>::_open_key(hkey_type hkeyParent, char_type const *key_name, REGSAM samDesired)
#else
inline /* static */ ws_typename_type_k basic_reg_key<C, T, A>::hkey_type basic_reg_key<C, T, A>::_open_key(ws_typename_param_k basic_reg_key<C, T, A>::hkey_type hkeyParent, ws_typename_param_k basic_reg_key<C, T, A>::char_type const *key_name, REGSAM samDesired)
#endif /* __STLSOFT_COMPILER_IS_MSVC && _MSC_VER < 1100 */
{
    hkey_type   hkey;

    return (0 == traits_type::reg_open_key(hkeyParent, key_name, &hkey)) ? hkey : NULL;
}

// Construction
template <ws_typename_param_k C, ws_typename_param_k T, ws_typename_param_k A>
inline basic_reg_key<C, T, A>::basic_reg_key()
    : m_hkey(NULL)
{}

template <ws_typename_param_k C, ws_typename_param_k T, ws_typename_param_k A>
inline basic_reg_key<C, T, A>::basic_reg_key(ws_typename_type_k basic_reg_key<C, T, A>::hkey_type *hkey, ws_typename_type_k basic_reg_key<C, T, A>::string_type const &key_name)
    : m_hkey(*hkey)
    , m_name(key_name)
{}

#if 0 /* These methods are inline, as VC5 will not link otherwise. (Both 4.2 & 6 will!) */
template <ws_typename_param_k C, ws_typename_param_k T, ws_typename_param_k A>
inline basic_reg_key<C, T, A>::basic_reg_key(hkey_type hkeyParent, basic_reg_key<C, T, A>::string_type const &key_name)
    : m_hkey(NULL)
    , m_name(key_name)
{
    if(0 != traits_type::reg_open_key(hkeyParent, c_str_ptr(key_name), &m_hkey))
    {
        m_hkey = NULL;
    }
}

template <ws_typename_param_k C, ws_typename_param_k T, ws_typename_param_k A>
inline basic_reg_key<C, T, A>::basic_reg_key(ws_typename_param_k basic_reg_key<C, T, A>::class_type const &keyParent, ws_typename_param_k basic_reg_key<C, T, A>::string_type const &key_name, REGSAM samDesired /* = KEY_READ */)
    : m_hkey(NULL)
    , m_name(key_name)
{
    if(0 != traits_type::reg_open_key(keyParent.m_hkey, c_str_ptr(key_name), &m_hkey, samDesired))
    {
        m_hkey = NULL;
    }
}
#endif /* 0 */

template <ws_typename_param_k C, ws_typename_param_k T, ws_typename_param_k A>
inline basic_reg_key<C, T, A>::basic_reg_key(class_type const &rhs)
    : m_hkey(traits_type::key_dup(rhs.m_hkey))
    , m_name(rhs.m_name)
{}

template <ws_typename_param_k C, ws_typename_param_k T, ws_typename_param_k A>
inline basic_reg_key<C, T, A>::basic_reg_key(class_type const &rhs, REGSAM samDesired)
    : m_hkey(traits_type::key_dup(rhs.m_hkey, samDesired))
    , m_name(rhs.m_name)
{}

template <ws_typename_param_k C, ws_typename_param_k T, ws_typename_param_k A>
inline basic_reg_key<C, T, A>::~basic_reg_key() winstl_throw_0()
{
    if(m_hkey != NULL)
    {
        ::RegCloseKey(m_hkey);
    }
}

template <ws_typename_param_k C, ws_typename_param_k T, ws_typename_param_k A>
inline void basic_reg_key<C, T, A>::_swap(ss_typename_type_k basic_reg_key<C, T, A>::class_type &rhs) winstl_throw_0()
{
    m_name.swap(rhs.m_name);

    hkey_type   _hkey       =   m_hkey;
                m_hkey      =   rhs.m_hkey;
                rhs.m_hkey  =   _hkey;
}

template <ws_typename_param_k C, ws_typename_param_k T, ws_typename_param_k A>
inline ws_typename_type_k basic_reg_key<C, T, A>::class_type &basic_reg_key<C, T, A>::operator =(ss_typename_type_k basic_reg_key<C, T, A>::class_type const &rhs)
{
    class_type  _this(rhs);

    _swap(_this);

    return *this;
}

// Attributes
template <ws_typename_param_k C, ws_typename_param_k T, ws_typename_param_k A>
inline ws_typename_type_k basic_reg_key<C, T, A>::string_type basic_reg_key<C, T, A>::name() const
{
    return m_name;
}

template <ws_typename_param_k C, ws_typename_param_k T, ws_typename_param_k A>
inline ws_typename_type_k basic_reg_key<C, T, A>::string_type basic_reg_key<C, T, A>::reg_class() const
{
    size_type   cch_key_class   =   0;
    ws_long_t   res             =   traits_type::reg_query_info(m_hkey, NULL, &cch_key_class, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);

    if(res == 0)
    {
        stlsoft_ns_qual(auto_buffer)<char_type, allocator_type, CCH_REG_API_AUTO_BUFFER>  p(++cch_key_class);

        res = traits_type::reg_query_info(m_hkey, p, &cch_key_class, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);

        if(res == 0)
        {
            return string_type(p, cch_key_class);
        }
    }

    return string_type();
}

template <ws_typename_param_k C, ws_typename_param_k T, ws_typename_param_k A>
inline ws_uint_t basic_reg_key<C, T, A>::num_sub_keys() const
{
    ws_uint_t   c_sub_keys;
    ws_long_t   res         =   traits_type::reg_query_info(m_hkey, NULL, NULL, &c_sub_keys, NULL, NULL, NULL, NULL, NULL, NULL, NULL);

    if(res != 0)
    {
        c_sub_keys = 0;
    }

    return c_sub_keys;
}

template <ws_typename_param_k C, ws_typename_param_k T, ws_typename_param_k A>
inline ws_uint_t basic_reg_key<C, T, A>::num_values() const
{
    ws_uint_t   c_values;
    ws_long_t   res         =   traits_type::reg_query_info(m_hkey, NULL, NULL, NULL, NULL, NULL, &c_values, NULL, NULL, NULL, NULL);

    if(res != 0)
    {
        c_values = 0;
    }

    return c_values;
}

template <ws_typename_param_k C, ws_typename_param_k T, ws_typename_param_k A>
inline ws_typename_type_k basic_reg_key<C, T, A>::hkey_type basic_reg_key<C, T, A>::get_key_handle() const
{
    return m_hkey;
}

// Operations
template <ws_typename_param_k C, ws_typename_param_k T, ws_typename_param_k A>
inline ws_typename_type_k basic_reg_key<C, T, A>::class_type basic_reg_key<C, T, A>::open_sub_key(char_type const *sub_key_name, REGSAM samDesired /* = KEY_READ */)
{
    return class_type(m_hkey, sub_key_name);
}

template <ws_typename_param_k C, ws_typename_param_k T, ws_typename_param_k A>
inline ws_typename_type_k basic_reg_key<C, T, A>::class_type basic_reg_key<C, T, A>::create_sub_key(char_type const *sub_key_name, REGSAM samDesired /* = KEY_READ */)
{
    hkey_type hkey;

    return (ERROR_SUCCESS != traits_type::reg_create_key(m_hkey, sub_key_name, &hkey, samDesired)) ? class_type() : class_type(&hkey, sub_key_name);
}

/* The handle returned from this method MUST be closed with RegCloseKey() */
template <ws_typename_param_k C, ws_typename_param_k T, ws_typename_param_k A>
inline ws_typename_type_k basic_reg_key<C, T, A>::hkey_type basic_reg_key<C, T, A>::dup_key_handle(REGSAM samDesired /* = KEY_ALL_ACCESS */)
{
    return traits_type::key_dup(m_hkey);
}

/* ////////////////////////////////////////////////////////////////////////// */

/// @} // end of group winstl_reg_library

/* ////////////////////////////////////////////////////////////////////////// */

#ifndef _WINSTL_NO_NAMESPACE
 #ifdef _STLSOFT_NO_NAMESPACE
} // namespace winstl
 #else
} // namespace winstl_project
} // namespace stlsoft
 #endif /* _STLSOFT_NO_NAMESPACE */
#endif /* !_WINSTL_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* _WINSTL_INCL_H_WINSTL_REG_KEY */

/* ////////////////////////////////////////////////////////////////////////// */
